@@ -54,9 +54,10 @@ int gcif_get_fw_info(mfile *mf,
 /*
  * get_icmd_query_cap
  */
-int get_icmd_query_cap(mfile *mf, struct connectx4_icmd_query_cap_general *icmd_query_caps)
+int get_icmd_query_cap(mfile *mf, struct icmd_hca_icmd_query_cap_general *icmd_query_caps)
 {
-    SEND_ICMD_FLOW(mf, GET_ICMD_QUERY_CAP, connectx4_icmd_query_cap_general, icmd_query_caps, 1, 0);
+    SEND_ICMD_FLOW(mf, GET_ICMD_QUERY_CAP, icmd_hca_icmd_query_cap_general,
+                   icmd_query_caps, 1, 0);
 }
 
 int gcif_mh_sync(mfile *mf, struct connectx4_icmd_mh_sync *mh_sync)
