@@ -119,6 +119,11 @@ SubCmdMetaData::SubCmdMetaData()
     _sCmds.push_back(new SubCmd("", "set_forbidden_versions", SC_Set_Forbidden_Versions));
     _sCmds.push_back(new SubCmd("ir", "image_reactivation", SC_Image_Reactivation));
     _sCmds.push_back(new SubCmd("bc", "binary_compare", SC_Binary_Compare));
+    _sCmds.push_back(new SubCmd("", "rsa_sign", SC_RSA_Sign));
+    _sCmds.push_back(new SubCmd("", "import_hsm_key", SC_Import_Hsm_Key));
+#if !defined(UEFI_BUILD) && !defined(NO_OPEN_SSL)
+    _sCmds.push_back(new SubCmd("", "export_public_key", SC_Export_Public_Key));
+#endif
 }
 
 SubCmdMetaData::~SubCmdMetaData()
@@ -199,6 +204,7 @@ FlagMetaData::FlagMetaData()
     _flags.push_back(new Flag("", "skip_ci_req", 0));
     _flags.push_back(new Flag("", "use_dev_rom", 0));
     _flags.push_back(new Flag("", "private_key", 1));
+    _flags.push_back(new Flag("", "public_key", 1));
     _flags.push_back(new Flag("", "key_uuid", 1));
     _flags.push_back(new Flag("", "private_key2", 1));
     _flags.push_back(new Flag("", "hmac_key", 1));
@@ -206,6 +212,26 @@ FlagMetaData::FlagMetaData()
     _flags.push_back(new Flag("", "no_fw_ctrl", 0));
     _flags.push_back(new Flag("", "ir", 0));
     _flags.push_back(new Flag("", "latest_fw", 0));
+    _flags.push_back(new Flag("", "psid", 1));
+    _flags.push_back(new Flag("", "cc", 1));
+    _flags.push_back(new Flag("", "linkx", 0));
+    _flags.push_back(new Flag("", "downstream_device_id_start_index", 1));
+    _flags.push_back(new Flag("", "num_of_downstream_devices", 1));
+    _flags.push_back(new Flag("", "linkx_auto_update", 0));
+    _flags.push_back(new Flag("", "activate", 0));
+    _flags.push_back(new Flag("", "activate_delay_sec", 1));
+    _flags.push_back(new Flag("", "downstream_device_ids", 1));
+    _flags.push_back(new Flag("", "download_transfer", 0));
+#ifndef __WIN__
+    _flags.push_back(new Flag("", "private_key_label", 1));
+    _flags.push_back(new Flag("", "public_key_label", 1));
+    _flags.push_back(new Flag("", "hsm", 0));
+#endif
+#ifdef __WIN__
+	_flags.push_back(new Flag("", "cpu_util", 1));
+#endif
+    _flags.push_back(new Flag("", "output_file", 1));
+    _flags.push_back(new Flag("", "user_password", 1));
 }
 
 FlagMetaData::~FlagMetaData()
@@ -301,7 +327,15 @@ bool verifyNumOfArgs(string name, string value)
     if (name == "flash_params") {
         expected = 3;
     }
+
     int actual = countArgs(value);
+    if (name == "downstream_device_ids") {
+        if (actual < 1) {
+            printf(FLINT_TOO_FEW_MINIMUM_ARGS_ERROR, 1, actual);
+            return false;
+        }
+        return true;
+    }
     if (actual < expected) {
         printf(FLINT_TOO_FEW_ARGS_ERROR, expected, actual);
         return false;
@@ -312,6 +346,20 @@ bool verifyNumOfArgs(string name, string value)
     return true;
 }
 
+bool strToInt(string str, int& num, int base = 0)
+{
+    char *endp;
+    char *numStr = strcpy(new char[str.size() + 1], str.c_str());
+    num = strtol(numStr, &endp, base);
+    if (*endp) {
+        delete[] numStr;
+        return false;
+    }
+    delete[] numStr;
+    return true;
+}
+
+
 bool strToNum(string str, u_int64_t& num, int base = 0)
 {
     char *endp;
@@ -344,6 +392,48 @@ bool getGUIDFromStr(string str, guid_t& guid, string prefixErr = "")
     return true;
 }
 
+bool isElementPresent(vector<string> strv, string str)
+{
+    for (vector<string>::iterator it = strv.begin(); it < strv.end(); it++) {
+        if (*it == str) {
+            return true;
+        }
+    }
+    return false;
+}
+
+bool stringsCommaSplit(string str, std::vector<int> &deviceIds)
+{
+    size_t pos;
+    std::vector<string> strv;
+    bool res = true;
+    while ((pos = str.find(',')) != string::npos) {
+        string tmp = str.substr(0, pos);
+        if (!isElementPresent(strv, tmp)) {
+            strv.push_back((string)tmp);
+        }
+        str = str.substr(pos + 1);
+    }
+    if (str != "") {
+        if (!isElementPresent(strv, str)) {
+            strv.push_back((string)str);
+        }
+    }
+    for (std::vector<string>::iterator it = strv.begin(); it < strv.end(); it++) {
+        int num;
+        if (false == strToInt((*it).c_str(), num)) {
+            return false;
+        }
+        if (num < 0 || num > 255) {
+            printf("Index value should be between 0 and 255.\n");
+            return false;
+        }
+        deviceIds.push_back(num);
+    }
+    return res;
+}
+
+
 guid_t incrGuid(guid_t baseGuid, unsigned int incrVal)
 {
     u_int64_t g = baseGuid.h;
@@ -372,18 +462,16 @@ bool parseFlashParams(string params, flash_params_t& fp)
         return false;
     }
     // Step 2 extract params
-    fp.type_name = strcpy(new char[paramVec[0].length() + 1], paramVec[0].c_str());
-    if (!fp.type_name) {
-        return false;
-    }
+    size_t length = paramVec[0].length();
+    const char* paramStr = paramVec[0].c_str();
+    memset(fp.type_name, 0, MAX_FLASH_NAME);
+    strncpy(fp.type_name, paramStr, length);
     u_int64_t tmp;
     if (!strToNum(paramVec[1], tmp)) {
-        delete[] fp.type_name;
         return false;
     }
     fp.log2size = (int)tmp;
     if (!strToNum(paramVec[2], tmp)) {
-        delete[] fp.type_name;
         return false;
     }
     fp.num_of_flashes = (int)tmp;
@@ -685,11 +773,21 @@ void Flint::initCmdParser()
                ' ',
                "",
                "another flag for override cache replacement", true);
+
+    AddOptions("hsm",
+        ' ',
+        "",
+        "flag for the sign command", true);
+
     AddOptions("private_key",
                ' ',
                "<key_file>",
                "path to PEM formatted private key to be used by the sign command");
 
+    AddOptions("public_key",
+        ' ',
+        "<key_file>",
+        "path to PEM formatted public key to be used by the sign command");
     AddOptions("key_uuid",
                ' ',
                "<uuid_file>",
@@ -708,6 +806,82 @@ void Flint::initCmdParser()
                "<uuid_file>",
                "UUID matching the given private key to be used by the sign command");
 
+    AddOptions("psid",
+        ' ',
+        "<PSID>",
+        "Use this PSID while burning livefish device using MFA2 archive");
+
+    AddOptions("cc",
+        ' ',
+        "<Congestion_Control>",
+        "Use this flag while burning to device a Congestion Control Component");
+
+    AddOptions("linkx",
+        ' ',
+        "",
+        "Use this flag while burning to device a LinkX Component");
+
+    AddOptions("downstream_device_id_start_index",
+        ' ',
+        "<downstream_device_id_start_index>",
+        "Use this flag while burning to device a LinkX Component. Begin from 0");
+
+    AddOptions("num_of_downstream_devices",
+        ' ',
+        "<num_of_downstream_devices>",
+        "Use this flag while burning to device a LinkX Component to specify the number of devices to burn");
+
+    AddOptions("linkx_auto_update",
+        ' ',
+        "",
+        "Use this flag while burning all cable devices connected to host.");
+
+    AddOptions("activate",
+        ' ',
+        "",
+        "Use this flag to apply the activation of all cable devices connected to host. By default, the activation is not performed.");
+
+    /*AddOptions("activate_delay_sec",
+        ' ',
+        "<timeout in seconds>",
+        "Use this flag to activate all cable devices connected to host with delay (default - 0, immediately). Important: 'activate' flag must be set.  This flag is relevant only for cable components.");*/
+
+    AddOptions("download_transfer",
+        ' ',
+        "",
+        "Use this flag to perform the download and transfer of all cable data for cables. By default, the download and transfer are not performed . This flag is relevant only for cable components.");
+
+    AddOptions("downstream_device_ids",
+        ' ',
+        "<list of ports>",
+        "Use this flag to specify the LNKX ports to perform query. List must be only comma-separated numbers, without spaces.");
+
+#ifndef __WIN__
+    AddOptions("public_key_label",
+        ' ',
+        "<string>",
+        "public key label to be used by the sign --hsm command");
+
+    AddOptions("private_key_label",
+        ' ',
+        "<string>",
+        "private key label to be used by the sign --hsm command");
+#endif
+    AddOptions("output_file",
+        ' ',
+        "<string>",
+        "output file name for exporting the public key from PEM/BIN");
+    AddOptions("user_password",
+        ' ',
+        "<string>",
+        "the HSM user password string in order to work with HSM device");
+#ifdef __WIN__
+	AddOptions("cpu_util",
+        ' ',
+		"<CPU_UTIL>",
+		"Use this flag to reduce CPU utilization while burning, Windows only. Legal values are from 1 (lowest CPU) to 5 (highest CPU)");
+#endif
+
     for (map_sub_cmd_t_to_subcommand::iterator it = _subcommands.begin(); it != _subcommands.end(); it++) {
         if (it->first == SC_ResetCfg) {
             // hidden command so "forget" mentioning it
@@ -723,7 +897,7 @@ void Flint::initCmdParser()
 
     AddOptionalSectionData("RETURN VALUES", "0", "Successful completion.");
     AddOptionalSectionData("RETURN VALUES", "1", "An error has occurred.");
-    AddOptionalSectionData("RETURN VALUES", "7", "For burn command - FW already updated - burn was aborted.");
+    AddOptionalSectionData("RETURN VALUES", "7", "For burn command - burning new firmware option was not chosen by the user when prompted, thus the firmware burning process was aborted.");
 
 
     for (map_sub_cmd_t_to_subcommand::iterator it = _subcommands.begin(); it != _subcommands.end(); it++) {
@@ -892,8 +1066,7 @@ ParseStatus Flint::HandleOption(string name, string value)
         _flintParams.use_dev_img_info = true;
     } else if (name == "ir") {
         _flintParams.image_reactivation = true;
-    }
-    else if (name == "banks") {
+    } else if (name == "banks") {
         _flintParams.banks_specified = true;
         u_int64_t banksNum;
         if (!strToNum(value, banksNum)) {
@@ -912,6 +1085,9 @@ ParseStatus Flint::HandleOption(string name, string value)
     } else if (name == "private_key") {
         _flintParams.privkey_specified = true;
         _flintParams.privkey_file = value;
+    } else if (name == "public_key") {
+        _flintParams.pubkey_specified = true;
+        _flintParams.pubkey_file = value;
     } else if (name == "key_uuid") {
         _flintParams.uuid_specified = true;
         _flintParams.privkey_uuid = value;
@@ -926,6 +1102,93 @@ ParseStatus Flint::HandleOption(string name, string value)
         _flintParams.privkey2_uuid = value;
     } else if (name == "latest_fw") {
         _flintParams.use_latest_fw_version = true;
+    } else if (name == "psid") {
+        _flintParams.use_psid = true;
+        _flintParams.psid = value;
+    }
+    else if (name == "cc") {
+        _flintParams.congestion_control = true;
+        _flintParams.congestion_control_param = value;
+    }
+    else if (name == "linkx") {
+        _flintParams.linkx_control = true;
+    }
+    else if (name == "cpu_util") {
+		_flintParams.use_cpu_utilization = true;
+		u_int64_t cpu_percent = 0;
+		if (!strToNum(value, cpu_percent)) {
+			return PARSE_ERROR;
+		}
+		_flintParams.cpu_percent = (int)cpu_percent;
+    } else if (name == "hsm") {
+        _flintParams.hsm_specified = true;
+    } else if (name == "private_key_label") {
+        _flintParams.private_key_label_specified = true;
+        _flintParams.private_key_label = value;
+    } else if (name == "public_key_label") {
+        _flintParams.public_key_label_specified = true;
+        _flintParams.public_key_label = value;
+    } else if (name == "output_file") {
+        _flintParams.output_file_specified = true;
+        _flintParams.output_file = value;
+    } else if (name == "user_password") {
+        _flintParams.hsm_password_specified = true;
+        _flintParams.hsm_password = value;
+    } else if (name == "downstream_device_id_start_index") {
+        _flintParams.cable_device_index_specified = true;
+        int device_index = 0;
+        if (!strToInt(value, device_index)) {
+            return PARSE_ERROR;
+        }
+        if (device_index < 0) {
+            return PARSE_ERROR;
+        }
+        _flintParams.cableDeviceIndex = device_index;
+    } 
+    
+    else if (name == "activate") {
+    _flintParams.activate = true;
+    }
+
+    else if (name == "activate_delay_sec") {
+        u_int64_t activate_delay_sec = 0;
+        if (!strToNum(value, activate_delay_sec)) {
+            return PARSE_ERROR;
+        }
+        _flintParams.activate_delay_sec = activate_delay_sec;
+    }
+    else if (name == "linkx_auto_update") {
+        _flintParams.linkx_auto_update = true;
+    } 
+
+    else if (name == "download_transfer") {
+    _flintParams.download_transfer = true;
+    }
+    
+    else if (name == "downstream_device_ids") {
+        if (value == "all") {
+            _flintParams.linkx_auto_update = true;
+        }
+        else {
+            _flintParams.downstream_device_ids_specified = true;
+            std::vector<int> deviceIds;
+            if (!stringsCommaSplit(value, deviceIds)) {
+                return PARSE_ERROR;
+            }
+            _flintParams.downstream_device_ids = deviceIds;
+        }
+    }
+    else if (name == "num_of_downstream_devices") {
+        int cableDeviceSize = 0;
+        if (!strToInt(value, cableDeviceSize)) {
+            return PARSE_ERROR;
+        }
+        if (cableDeviceSize <= 0 || cableDeviceSize > 255) {
+            printf("Cable size should be between 1 and 255.\n");
+            return PARSE_ERROR;
+        }
+        _flintParams.cableDeviceSize = cableDeviceSize;
+        _flintParams.cable_device_size_specified = true;
     }
     else {
         cout << "Unknown Flag: " << name;
