@@ -63,8 +63,10 @@ typedef enum {
 #define FLINT_INVALID_COMMAD_ERROR            "Invalid command: %s\n"
 #define FLINT_TOO_MANY_ARGS_ERROR             "Too many arguments. Expected: %d , Received: %d\n"
 #define FLINT_TOO_FEW_ARGS_ERROR              "Too few arguments. Expected: %d , Received: %d\n"
+#define FLINT_TOO_FEW_MINIMUM_ARGS_ERROR      "Too few arguments. Expected at least: %d , Received: %d\n"
 #define FLINT_NO_COMMAND_ERROR                "No command found."
 #define FLINT_OPEN_FWOPS_DEVICE_ERROR         "Cannot open Device: %s. %s\n"
+#define FLINT_OPEN_FWOPS_DEVICE_ERROR_1       "Cannot open Device: %s.\n"
 #define FLINT_OPEN_FWOPS_IMAGE_ERROR          "Cannot open Image: %s. %s\n"
 #define FLINT_DEVICE_AND_IMAGE_ERROR          "Please specify either Device or Image.\n"
 #define FLINT_NO_DEVICE_ERROR                 "Please specify Device.\n"
@@ -77,10 +79,12 @@ typedef enum {
 #define FLINT_CMD_ARGS_ERROR5                 "Command \"%s\" does not require arguments\n"
 #define FLINT_INVALID_OPTION_ERROR            "Unknown option \"%s\" for the \"%s\" command. you can use %s.\n"
 #define FLINT_INVALID_FLAG_ERROR              "Invalid switch \"%s\" is specified.\n"
+#define FLINT_INVALID_FLAG_ERROR_5TH_GEN      "Invalid switch \"%s\" is specified for 5th gen device.\n"
 #define FLINT_INVALID_FLAG_WITH_FLAG_ERROR    "Cannot specify \"%s\" flag with \"%s\" flag.\n"
 #define FLINT_INVALID_FLAG_WITHOUT_FLAG_ERROR "\"%s\" flag must be specified with \"%s\" flag.\n"
 #define FLINT_INVALID_FLAG_WITH_CMD_ERROR     "Cannot specify flag: %s with Command: %s\n"
 #define FLINT_CMD_VERIFY_ERROR                "FW image verification failed: %s. AN HCA DEVICE CAN NOT BOOT FROM THIS IMAGE.\n"
+#define FLINT_CMD_VERIFY_ERROR_1              "FW image verification failed: No valid FS4 image found.Check the flash parameters, if specified..AN HCA DEVICE CAN NOT BOOT FROM THIS IMAGE"
 #define FLINT_FAILED_QUERY_ERROR              "Failed to query %s: %s.%s\n"
 #define FLINT_COMMAND_DEVICE_IMAGE_ERROR      "Command \"%s\" requires both image and device to be specified.\n"
 #define FLINT_COMMAND_DEVICE_ERROR            "Command \"%s\" requires device, but an image file was given.\n"
@@ -102,6 +106,7 @@ typedef enum {
 #define FLINT_IO_OPEN_ERROR                   "Failed to open %s: %s\n"
 #define FLINT_IMAGE_READ_ERROR                "Failed to read image. %s\n"
 #define FLINT_READ_ERROR                      "Failed to read from %s. %s\n"
+#define FLINT_READ_FILE_ERROR                 "Failed to read from %s.\n"
 #define FLINT_WIN_NOT_SUPP_ERROR              "Command \"%s\" is not supported in windows.\n"
 #define FLINT_WIN_ONLY_SUPP_ERROR             "Command  \"%s\" is supported only in windows.\n"
 #define FLINT_GEN_COMMAND_ERROR               "Failed to execute command %s. %s\n"
@@ -144,7 +149,34 @@ typedef enum {
 #define FLINT_SIGN_ERROR                      "Failed to sign the image: %s\n"
 #define FLINT_HMAC_ERROR                      "Failed to add HMAC: %s\n"
 #define FLINT_FAILED_IMAGE_REACTIVATION_ERROR "Failed to execute image reactivation on device %s. Error: %s.\n"
-
+#define FLINT_NO_MFA2                         "MFA2 funcionality is not supported"
+#define FLINT_NO_HSM                          "HSM funcionality is not supported"
+#define FLINT_ILLEGAL_CPU_VALUE               "Illegal value for CPU utilization. Values must be between 1 (low CPU) and 5 (high CPU).\n"
+#define FLINT_FILE_SIZE_ERROR                 "Can not get file size for \"%s\".\n"
+#define FAILED_TO_VERIFY_PARAMS               "Failed to verify params(internal error)."
+#define INVALID_GUID_SYNTAX                   "Invalid GUID syntax (%s) %s\n"
+#define FAILED_GET_CONSOLE_MODE               "Failed to get console mode.\n"
+#define USER_ABORT                            "Aborted by user\n"
+#define SECTION_NOT_FOUNT                     "%s section not found in the given image."
+#define UNCOMPRESSS_ERROR                     "Failed uncompressing FW configuration section. uncompress returns %d"
+#define OPEN_WRITE_FILE_ERROR                 "Can not open file %s for write: %s."
+#define IMAGE_SIGN_TYPE_ERROR                 "Image signing is applicable only for selected FW images. Please check your image type.\n"
+#define HSM_INIT_ERROR                        "HSM init has failed! Please check if the HSM card installed and configured properly.\n"
+#define HSM_PRIVATE_KEY_DUPLICATE             "Creating HSM signature has failed - the private key label is duplicated.\n"
+#define HSM_PUBLIC_KEY_DUPLICATE              "Creating HSM signature has failed - the public key label is duplicated.\n"
+#define HSM_SIGNATURE_CREATION_FAILED         "Creating HSM signature has failed\n"
+#define HSM_UUID_MISSING                      "To Sign the image with RSA you must provide UUID with HSM sign.\n"
+#define HSM_PRIVATE_KEY_LABEL_MISSING         "Must supply private key label for sign with HSM sign.\n"
+#define HSM_PASSWORD_MISSING                  "Must supply HSM user password for sign with HSM sign.\n"
+#define SIGN_PRIVATE_KEY_NOT_FOUND            "Can't find private key file %s \n"
+#define SIGN_PUBLIC_KEY_NOT_FOUND             "Can't find public key file %s \n"
+#define HSM_BOOT_SIGNATURE_CREATION_FAILED    "Creating HSM BOOT signature has failed\n"
+#define HSM_CRITICAL_SIGNATURE_CREATION_FAILED "Creating HSM critical signature has failed\n"
+#define HSM_NON_CRITICAL_SIGNATURE_CREATION_FAILED "Creating HSM non-critical signature has failed\n"
+#define HSM_SECURE_BOOT_SIGNATURE_FAILED      "Inserting secure BOOT signatures has failed : %s.\n"
+#define HSM_SECURE_FW_SIGNATURE_FAILED        "Creation secured FW signatures has failed.\n"
+#define LINKX_QUERY_DEVICE_NOT_SUPPORTED      "Linkx query for device %s is not supported.\n"
+#define LINKX_BURN_DEVICE_NOT_SUPPORTED       "Linkx burn for device %s is not supported.\n"
 /**************************
  * Flint Warning Messages
  *************************/
@@ -158,4 +190,6 @@ typedef enum {
 #define FLINT_INTERRUPT_WARRNING       "\n-W- An internal error occurred. This program cannot be interrupted.\n"
 #define FLINT_SET_GUIDS_WARRNING       "-W- GUIDs are already set, re-burning image with the new GUIDs ...\n"
 #define FLINT_OCR_WARRNING             "\n-W- Firmware flash cache access is enabled. Running in this mode may cause the firmware to hang.\n"
+#define FLINT_OPEN_LOG_FILE_WARNING    "-W- Failed to open log file \"%s\": %s. No logs will be saved\n"
+
 #endif
