@@ -91,6 +91,7 @@ void TerminationHandler(int signum)
         printf("\n Received signal %d. Cleaning up ...\n", signum);
         fflush(stdout);
         //sleep(1); // Legacy from the Old Flint
+        gFlint->GetSubCommands()[gFlint->GetFlintParams().cmd]->cleanInterruptedCommand();
         delete gFlint;
         gFlint = NULL;
         printf(" Done.\n");
@@ -167,6 +168,13 @@ map_sub_cmd_t_to_subcommand Flint::initSubcommandMap()
     cmdMap[SC_Add_Hmac] = new AddHmacSubCommand();
     cmdMap[SC_Set_Public_Keys] = new SetPublicKeysSubCommand();
     cmdMap[SC_Set_Forbidden_Versions] = new SetForbiddenVersionsSubCommand();
+    cmdMap[SC_Image_Reactivation] = new ImageReactivationSubCommand();
+    cmdMap[SC_RSA_Sign] = new SignRSASubCommand();
+    cmdMap[SC_Binary_Compare] = new  BinaryCompareSubCommand();
+    cmdMap[SC_Import_Hsm_Key] = new  ImportHsmKeySubCommand();
+#if !defined(UEFI_BUILD) && !defined(NO_OPEN_SSL)
+    cmdMap[SC_Export_Public_Key] = new ExportPublicSubCommand();
+#endif
     return cmdMap;
 }
 
@@ -189,9 +197,6 @@ Flint::Flint() :
 Flint::~Flint()
 {
     deInitSubcommandMap(_subcommands);
-    if (_flintParams.flash_params.type_name) {
-        delete[] _flintParams.flash_params.type_name;
-    }
 }
 
 FlintStatus Flint::run(int argc, char *argv[])
@@ -200,7 +205,7 @@ FlintStatus Flint::run(int argc, char *argv[])
     //There are some memory allocations parseCmdLine
     ParseStatus status;
     try {
-        status = this->parseCmdLine(argc, argv);;
+        status = this->parseCmdLine(argc, argv);
     } catch (exception& e) {
         cout << "-E- " << e.what() << endl;
         return FLINT_FAILED;
