@@ -56,7 +56,6 @@ using namespace std;
 #include "mlxarchive/mlxarchive_mfa2_utils.h"
 using namespace mfa2;
 #endif
-
 //we might need to close the log from the main program in case of interrupt
 void close_log();
 void print_time_to_log();
@@ -73,6 +72,9 @@ typedef enum what_to_ver {
 } what_to_ver_t;
 /*Subcommand classes:*/
 
+
+
+
 #define FLINT_ERR_LEN 1024
 
 class SubCommand
@@ -81,6 +83,7 @@ protected:
     FwOperations *_fwOps;
     FwOperations *_imgOps;
     FBase *_io;
+
     what_to_ver_t _v;
     int _maxCmdParamNum;
     int _minCmdParamNum;
@@ -154,15 +157,18 @@ protected:
     inline void closeLog() {close_log();}
     //print errors to an err buff, log if needed and stdout
     void reportErr(bool shouldPrint, const char *format, ...);
-
+    bool readFromFile(const string& filePath, std::vector<u_int8_t>& buff);
+    bool getFileSize(const string& filePath, long& fileSize);
     bool writeToFile(string filePath, const std::vector<u_int8_t>& buff);
     FlintStatus writeImageToFile(const char *file_name, u_int8_t *data, u_int32_t length);
 
     bool dumpFile(const char *confFile, std::vector<u_int8_t>& data, const char *sectionName);
     bool unzipDataFile(std::vector<u_int8_t> data, std::vector<u_int8_t> &newData, const char *sectionName);
     const char* fwImgTypeToStr(u_int8_t fwImgType);
-
-
+    FlintStatus UnlockDevice(FwOperations*);
+    FlintStatus LockDevice(FwOperations*);
+    void ClearGuidStruct(FwOperations::sg_params_t& sgParams);
+    bool stringsCommaSplit(string str, std::vector<u_int32_t> &deviceIds);
 public:
     SubCommand() : _fwOps(NULL), _imgOps(NULL), _io(NULL), _v(Wtv_Uninitilized), _maxCmdParamNum(-1),  _minCmdParamNum(-1), _mccSupported(false), _imageReactivation(false)
 #ifndef NO_MSTARCHIVE        
@@ -174,6 +180,7 @@ public:
     }
     virtual ~SubCommand();
     virtual FlintStatus executeCommand() = 0;
+    virtual void cleanInterruptedCommand() {}//by default do nothing
     inline void setParams(const FlintParams& flintParams) {_flintParams = flintParams;}
     inline string& getName() {return this->_name;}
     inline string& getDesc() {return this->_desc;}
@@ -184,6 +191,7 @@ public:
     inline string& getParamExp() {return this->_paramExp;}
     inline string& getExample() {return this->_example;}
 };
+
 class BurnSubCommand : public SubCommand
 {
 private:
@@ -193,9 +201,10 @@ private:
     FwOperations::ExtBurnParams _burnParams;
     bool _devQueryRes;
     int _unknownProgress; // used to trace the progress of unknown progress.
+    FwCompsMgr* fwCompsAccess;
     FlintStatus burnFs3();
     FlintStatus burnFs2();
-    bool checkFwVersion();
+    bool checkFwVersion(bool CreateFromImgInfo = true, u_int16_t fw_ver0 = 0, u_int16_t fw_ver1 = 0, u_int16_t fw_ver2 = 0);
     bool checkPSID();
     void updateBurnParams();
     bool dealWithExpRom();
@@ -203,10 +212,16 @@ private:
     bool dealWithGuids();
     bool dealWithVSD();
     FlintStatus burnMFA2();
+    FlintStatus burnMFA2LiveFish(dm_dev_id_t devid_t);
+    FlintStatus burnCongestionControl();
+    bool verifyMFA2Params(bool IsLiveFish);
+    FlintStatus BurnLinkX(string deviceName, int deviceIndex, int deviceSize, string binaryFileName, bool linkx_auto_update,
+        bool activationNeeded, bool downloadTransferNeeded, int activate_delay_sec, ProgressCallBackAdvSt* ProgressFuncAdv);
 public:
     BurnSubCommand();
     ~BurnSubCommand();
     FlintStatus executeCommand();
+    virtual void cleanInterruptedCommand();
     bool verifyParams();
 };
 
@@ -223,7 +238,6 @@ private:
 public:
     BinaryCompareSubCommand();
     ~BinaryCompareSubCommand();
-
     FlintStatus executeCommand();
     bool verifyParams();
 };
@@ -231,11 +245,17 @@ public:
 class QuerySubCommand : public SubCommand
 {
 private:
+    fw_info_t _imgInfo;
+    FlintStatus printImageInfo(const fw_info_t& fwInfo);
     string printSecurityAttrInfo(u_int32_t m);
     FlintStatus printInfo(const fw_info_t& fwInfo, bool fullQuery);
     bool displayFs3Uids(const fw_info_t& fwInfo);
     bool displayFs2Uids(const fw_info_t& fwInfo);
     bool checkMac(u_int64_t mac, string& warrStr);
+    FlintStatus queryMFA2();
+    bool PrintLinkXQuery(string& outputString, const string& host, int deviceIndex, const comp_status_st& ComponentStatus, const component_linkx_st& linkx_data, char* delimeter);
+    FlintStatus QueryLinkX(string deviceName, string outputFile, std::vector<int> deviceIds);
+
 public:
     QuerySubCommand();
     ~QuerySubCommand();
@@ -271,6 +291,34 @@ public:
     bool verifyParams();
 };
 
+class ImportHsmKeySubCommand : public SubCommand
+{
+public:
+    ImportHsmKeySubCommand();
+    ~ImportHsmKeySubCommand();
+    FlintStatus executeCommand();
+    bool verifyParams();
+};
+
+class ExportPublicSubCommand : public SubCommand
+{
+public:
+    ExportPublicSubCommand();
+    ~ExportPublicSubCommand();
+    FlintStatus executeCommand();
+    bool verifyParams();
+};
+
+class SignRSASubCommand : public SubCommand
+{
+private:
+public:
+    SignRSASubCommand();
+    ~SignRSASubCommand();
+    FlintStatus executeCommand();
+    bool verifyParams();
+};
+
 class SetPublicKeysSubCommand : public SubCommand
 {
 private:
@@ -308,6 +356,7 @@ public:
     FlintStatus executeCommand();
     bool verifyParams();
 };
+
 class SwResetSubCommand : public SubCommand
 {
 private:
