@@ -42,6 +42,11 @@ extern "C" {
 
 #include "mtcr_com_defs.h"
 #include "mtcr_mf.h"
+#if defined(MTCR_UL) && !defined(NO_INBAND)
+#include <mtcr_ul/mtcr_ib.h>
+#endif
+
+#define SLV_ADDRS_NUM 128
 
 typedef enum mtcr_access_method {
     MTCR_ACCESS_ERROR  = MST_ERROR,
@@ -60,8 +65,8 @@ int mread4(mfile *mf, unsigned int offset, u_int32_t *value);
 int mwrite4(mfile *mf, unsigned int offset, u_int32_t value);
 
 
-int mread4_block (mfile *mf, unsigned int offset, u_int32_t* data, int byte_len);
-int mwrite4_block (mfile *mf, unsigned int offset, u_int32_t* data, int byte_len);
+int mread4_block(mfile *mf, unsigned int offset, u_int32_t *data, int byte_len);
+int mwrite4_block(mfile *mf, unsigned int offset, u_int32_t *data, int byte_len);
 
 int msw_reset(mfile *mf);
 int mhca_reset(mfile *mf);
@@ -81,7 +86,7 @@ int mdevices(char *buf, int len, int mask);
  * len will be updated to hold the array length
  *
  */
-dev_info* mdevices_info(int mask, int* len);
+dev_info* mdevices_info(int mask, int *len);
 
 /*
  *  * Get list of MST (Mellanox Software Tools) devices info records.
@@ -90,10 +95,10 @@ dev_info* mdevices_info(int mask, int* len);
  *  * Verbosity will decide whether to get all the Physical functions or not.
  */
 
-dev_info* mdevices_info_v(int mask, int* len, int verbosity);
+dev_info* mdevices_info_v(int mask, int *len, int verbosity);
 
-void mdevice_info_destroy(dev_info* dev_info, int len);
-void mdevices_info_destroy(dev_info* dev_info, int len);
+void mdevice_info_destroy(dev_info *dev_info, int len);
+void mdevices_info_destroy(dev_info *dev_info, int len);
 
 int mget_mdevs_type(mfile *mf, u_int32_t *mtype);
 
@@ -101,13 +106,13 @@ int mget_mdevs_type(mfile *mf, u_int32_t *mtype);
  * Open Mellanox Software tools (mst) driver. Device type==INFINIHOST
  * Return valid mfile ptr or 0 on failure
  */
-mfile *mopen(const char *name);
+mfile* mopen(const char *name);
 
-mfile *mopend(const char *name, int type);
+mfile* mopend(const char *name, DType dtype);
 
-mfile *mopen_fw_ctx(void* fw_cmd_context, void* fw_cmd_func, void* extra_data);
+//mfile* mopen_fw_ctx(void *fw_cmd_context, void *fw_cmd_func, void *extra_data);
 
-mfile *mopen_adv(const char *name, MType mtype);
+mfile* mopen_adv(const char *name, MType mtype);
 
 /*
  * Close Mellanox driver
@@ -126,49 +131,55 @@ int mos_reg_access(mfile *mf, int reg_access, void *reg_data, u_int32_t cmd_type
 int maccess_reg_cmdif(mfile *mf, reg_access_t reg_access, void *reg_data, u_int32_t cmd_type);
 
 int maccess_reg(mfile     *mf,
-                u_int16_t  reg_id,
+                u_int16_t reg_id,
                 maccess_reg_method_t reg_method,
-                void*      reg_data,
-                u_int32_t  reg_size,
-                u_int32_t	r_size_reg, // used when sending via icmd interface (how much data should be read back to the user)
-                u_int32_t	w_size_reg, // used when sending via icmd interface (how much data should be written to the scratchpad)
-                         	 	 	    // if you dont know what you are doing then r_size_reg = w_size_reg = your_register_size
+                void *reg_data,
+                u_int32_t reg_size,
+                u_int32_t r_size_reg,   // used when sending via icmd interface (how much data should be read back to the user)
+                u_int32_t w_size_reg,   // used when sending via icmd interface (how much data should be written to the scratchpad)
+                                        // if you dont know what you are doing then r_size_reg = w_size_reg = your_register_size
                 int       *reg_status);
 
-int icmd_send_command(mfile *mf, int opcode, void* data, int data_size, int skip_write);
+int icmd_send_command(mfile *mf, int opcode, void *data, int data_size, int skip_write);
 
 int icmd_clear_semaphore(mfile *mf);
 
 
-int tools_cmdif_send_inline_cmd(mfile* mf, u_int64_t in_param, u_int64_t* out_param,
-                                u_int32_t input_modifier, u_int16_t opcode, u_int8_t  opcode_modifier);
+int tools_cmdif_send_inline_cmd(mfile *mf, u_int64_t in_param, u_int64_t *out_param,
+                                u_int32_t input_modifier, u_int16_t opcode, u_int8_t opcode_modifier);
 
-int tools_cmdif_send_mbox_command(mfile* mf, u_int32_t input_modifier, u_int16_t opcode, u_int8_t  opcode_modifier,
-                                  int data_offs_in_mbox, void* data, int data_size, int skip_write);
+int tools_cmdif_send_mbox_command(mfile *mf, u_int32_t input_modifier, u_int16_t opcode, u_int8_t opcode_modifier,
+                                  int data_offs_in_mbox, void *data, int data_size, int skip_write);
 
 int tools_cmdif_unlock_semaphore(mfile *mf);
 
 
-int mget_max_reg_size(mfile *mf);
+int mget_max_reg_size(mfile *mf, maccess_reg_method_t reg_method);
+int supports_reg_access_gmp(mfile *mf, maccess_reg_method_t reg_method);
 
 const char* m_err2str(MError status);
 
-int mread_buffer(mfile *mf, unsigned int offset, u_int8_t* data, int byte_len);
-int mwrite_buffer(mfile *mf, unsigned int offset, u_int8_t* data, int byte_len);
+int mread_buffer(mfile *mf, unsigned int offset, u_int8_t *data, int byte_len);
+int mwrite_buffer(mfile *mf, unsigned int offset, u_int8_t *data, int byte_len);
 
-int mget_vsec_supp(mfile* mf);
+int mget_vsec_supp(mfile *mf);
 
-int mget_addr_space(mfile* mf);
-int mset_addr_space(mfile* mf, int space);
+int mget_addr_space(mfile *mf);
+int mset_addr_space(mfile *mf, int space);
 
-int mclear_pci_semaphore(const char* name);
+int mclear_pci_semaphore(const char *name);
 
 int mvpd_read4(mfile *mf, unsigned int offset, u_int8_t value[4]);
 
-#ifndef NO_INBAND
-int mib_smp_set(mfile* mf, u_int8_t* data, u_int16_t attr_id, u_int32_t attr_mod);
-int mib_smp_get(mfile* mf, u_int8_t* data, u_int16_t attr_id, u_int32_t attr_mod);
-#endif
+int mvpd_write4(mfile *mf, unsigned int offset, u_int8_t value[4]);
+
+MTCR_API int MWRITE4_SEMAPHORE(mfile* mf, int offset, int value);
+
+MTCR_API int MREAD4_SEMAPHORE(mfile* mf, int offset, u_int32_t* ptr);
+
+int allocate_kernel_memory_page(mfile *mf, mtcr_alloc_page* user_alloc_page);
+
+void set_increase_poll_time(int new_value);
 
 #ifdef __cplusplus
 }
