@@ -63,11 +63,16 @@ int is_four_byte_address_needed(mflash *mfl, MfError *status)
         return 0;
     case DeviceQuantum:
     case DeviceConnectX6:
+    case DeviceConnectX7:
     case DeviceConnectX6DX:
     case DeviceConnectX6LX:
     case DeviceSpectrum2:
     case DeviceSpectrum3:
+    case DeviceQuantum2:
+    case DeviceSpectrum4:
     case DeviceBlueField2:
+    case DeviceGearBox:
+    case DeviceGearBoxManager:
         return 1;
     default:
         *status = MFE_UNSUPPORTED_DEVICE;
@@ -99,11 +104,16 @@ int is_flash_enable_needed(mflash *mfl, MfError *status)
     case DeviceBlueField2:
     case DeviceQuantum:
     case DeviceConnectX6:
+    case DeviceConnectX7:
     case DeviceConnectX6DX:
     case DeviceConnectX6LX:
     case DeviceSpectrum2:
     case DeviceSpectrum3:
+    case DeviceQuantum2:
+    case DeviceSpectrum4:
     case DeviceSecureHost:
+    case DeviceGearBox:
+    case DeviceGearBoxManager:
         return 0;
     default:
         *status = MFE_UNSUPPORTED_DEVICE;
@@ -113,7 +123,7 @@ int is_flash_enable_needed(mflash *mfl, MfError *status)
 }
 
 //When (*status != MFE_OK) return value is undefined
-int is_icmdif_supported(mflash *mfl, MfError *status)
+int is_icmdif_supported(mflash *mfl, MfError *status, int *is7NmSuppported)
 {
     *status = MFE_OK;
 
@@ -137,9 +147,17 @@ int is_icmdif_supported(mflash *mfl, MfError *status)
     case DeviceQuantum:
     case DeviceSpectrum2:
     case DeviceSpectrum3:
+    case DeviceSpectrum4:
     case DeviceConnectX6:
     case DeviceConnectX6DX:
     case DeviceConnectX6LX:
+    case DeviceGearBox:
+    case DeviceGearBoxManager:
+        *is7NmSuppported = 0;
+        return 1;
+    case DeviceConnectX7:
+    case DeviceQuantum2:
+        *is7NmSuppported = 1;
         return 1;
     default:
         *status = MFE_UNSUPPORTED_DEVICE;
@@ -173,9 +191,14 @@ int is_supports_sw_reset(mflash *mfl, MfError *status)
     case DeviceConnectX6:
     case DeviceConnectX6DX:
     case DeviceConnectX6LX:
+    case DeviceConnectX7:
     case DeviceSpectrum2:
     case DeviceSpectrum3:
+    case DeviceQuantum2:
+    case DeviceSpectrum4:
     case DeviceSecureHost:
+    case DeviceGearBox:
+    case DeviceGearBoxManager:
         return 0;
     default:
         *status = MFE_UNSUPPORTED_DEVICE;
