@@ -81,6 +81,7 @@ MlxcfgDBManager::~MlxcfgDBManager()
     VECTOR_ITERATOR(Param*, fetchedParams, it) {
         delete *it;
     }
+
     if (!_db) {
         return;
     }
@@ -101,7 +102,7 @@ void MlxcfgDBManager::checkDBVersion()
             break;
         }
     }
-
+    sqlite3_finalize(stmt); // releasing allocated stmt
     if (dbVersion != _supportedVersion) {
         throw MlxcfgException("Unsupported database version");
     }
@@ -210,6 +211,7 @@ int MlxcfgDBManager::selectParamCallBack(void *object, int argc, char **argv,
     return 0;
 }
 
+
 int MlxcfgDBManager::selectParamByMlxconfigNameCallBack(void *object, int argc,
                                                         char **argv, char **azColName)
 {
@@ -362,26 +364,49 @@ TLVConf* MlxcfgDBManager::getTLVByIndexAndClassAux(u_int32_t id, TLVClass c)
     return NULL;
 }
 
-TLVConf* MlxcfgDBManager::getTLVByParamMlxconfigName(std::string n)
+bool MlxcfgDBManager::isParamMlxconfigNameExist(std::string n)
+{
+    execSQL(selectParamByMlxconfigNameCallBack, this,
+        SQL_SELECT_PARAM_BY_MLXCONFIG_NAME, n.c_str());
+
+    return (n == _paramSqlResult->_mlxconfigName);
+}
+
+TLVConf* MlxcfgDBManager::getTLVByParamMlxconfigName(std::string n, u_int32_t index)
 {
     u_int32_t port;
     string tlvName;
 
     VECTOR_ITERATOR(TLVConf*, fetchedTLVs, it) {
-        if ((*it)->findParamByMlxconfigName(n)) {
+        if ((*it)->findParamByMlxconfigName(n) || (*it)->findParamByMlxconfigName(n + getArraySuffixByInterval(index))) {
             return (*it);
         }
     }
 
-    //printf("-D- Try to find it in DB\n");
-    //Try to find it in DB:
+    //Try to find it in DB:dd
     execSQL(selectParamByMlxconfigNameCallBack, this,
             SQL_SELECT_PARAM_BY_MLXCONFIG_NAME, n.c_str());
 
+    // if not found try to find it with continuance array suffix
     if (!_paramSqlResult) {
+        string newName = n + getArraySuffixByInterval(index);
+
+        execSQL(selectParamByMlxconfigNameCallBack, this,
+            SQL_SELECT_PARAM_BY_MLXCONFIG_NAME, newName.c_str());
+    }
+
+    if (!_paramSqlResult) {
+        string suffix = "";
+        if (index > 0) {
+            if (isIndexedStartFromOneSupported(n)) {
+                index++;
+            }
+            suffix = "[" + numToStr(index) + "]";
+        }
+
         throw MlxcfgException(
-                  "Unknown Parameter: %s",
-                  n.c_str());
+            "Unknown Parameter: %s",
+            (n + suffix).c_str());
     }
 
     port = _paramSqlResult->_port;
