@@ -79,6 +79,9 @@ void MlxCfg::printHelp()
            IDENT "SYNOPSIS:\n"
            IDENT2 MLXCFG_NAME " [Options] <Commands> [Parameters]\n");
 
+    printf(IDENT "DESCRIPTION:\n"
+          IDENT2 "Allows the user to change some of the device configurations without having to\n"
+          IDENT2 "create and burn a new firmware.");
     // print options
     printf("\n");
     printf(IDENT "OPTIONS:\n");
@@ -97,19 +100,20 @@ void MlxCfg::printHelp()
     printf("\n");
     printf(IDENT "COMMANDS:\n");
     printf(IDENT2 "%-24s : %s\n", "clear_semaphore", "clear the tool semaphore.");
-    printf(IDENT2 "%-24s : %s\n", "i[show_confs]", "display informations about all configurations.");
+    printf(IDENT2 "%-24s : %s\n", "i[show_confs]", "display information about all configurations.");
     printf(IDENT2 "%-24s : %s\n", "q[uery]", "query supported configurations.");
     printf(IDENT2 "%-24s : %s\n", "r[eset]", "reset all configurations to their default value.");
     printf(IDENT2 "%-24s : %s\n", "s[et]", "set configurations to a specific device.");
     printf(IDENT2 "%-24s : %s\n", "set_raw", "set raw configuration file.(only "  FIFTH_GENERATION_LIST ".)");
+    printf(IDENT2 "%-24s : %s\n", "get_raw", "get raw configuration.(only "  FIFTH_GENERATION_LIST ".)");
     printf(IDENT2 "%-24s : %s\n", "backup", "backup configurations to a file (only "  FIFTH_GENERATION_LIST ".). Use set_raw command to restore file.");
     printf(IDENT2 "%-24s : %s\n", "gen_tlvs_file", "Generate List of all TLVs. TLVs output file name must be specified. (*)");
     printf(IDENT2 "%-24s : %s\n", "g[en_xml_template]", "Generate XML template. TLVs input file name and XML output file name must be specified. (*)");
     printf(IDENT2 "%-24s : %s\n", "xml2raw", "Generate Raw file from XML file. XML input file name and raw output file name must be specified. (*)");
     printf(IDENT2 "%-24s : %s\n", "raw2xml", "Generate XML file from Raw file. raw input file name and XML output file name must be specified. (*)");
-    printf(IDENT2 "%-24s : %s\n", "xml2bin", "Generate Bin file from XML file. XML input file name and bin output file name must be specified. (*)");
-    printf(IDENT2 "%-24s : %s\n", "create_conf", "Generate Configuration file from XML file. XML input file name and bin output file name must be specified. (*)");
-    printf(IDENT2 "%-24s : %s\n", "apply", "Apply a Configuration file. bin input file name must be specified. (*)");
+    printf(IDENT2 "%-24s : %s\n", "xml2bin", "Generate binary configuration dump file from XML file. XML input file name and bin output file name must be specified. (*)");
+    printf(IDENT2 "%-24s : %s\n", "create_conf", "Generate configuration file from XML file. XML input file name and bin output file name must be specified. (*)");
+    printf(IDENT2 "%-24s : %s\n", "apply", "Apply a configuration file, that was created with create_conf command. bin input file name must be specified. (*)");
 
     // print supported commands
     printf("\n");
@@ -204,6 +208,7 @@ mlxCfgStatus MlxCfg::extractQueryCfgArgs(int argc, char *argv[])
         if (isIndexedMlxconfigName(mlxconfigName)) {
             string indexStr = parseIndexStr(mlxconfigName);
             vector<u_int32_t> indexes;
+            bool isStartFromOneSupported = false;
             extractIndexes(indexStr, indexes);
             if (indexes.size() > 1) {
                 size_t p = mlxconfigName.find('[');
@@ -211,14 +216,38 @@ mlxCfgStatus MlxCfg::extractQueryCfgArgs(int argc, char *argv[])
                     return err(true, "Expected a parameter with index");
                 }
                 mlxconfigName = mlxconfigName.substr(0, p);
+                isStartFromOneSupported = isIndexedStartFromOneSupported(mlxconfigName);
+
                 VECTOR_ITERATOR(u_int32_t, indexes, it) {
                     ParamView paramView;
+                    if (isStartFromOneSupported) {
+                        if ((int32_t)*it <= 0) {
+                            printf("-E- Index is out of range.Minimal index is 1\n");
+                            return MLX_CFG_ERROR_NO_USAGE;
+                        }
+                        else {
+                            (*it)--;
+                        }
+                    }
                     paramView.mlxconfigName = mlxconfigName + "[" + numToStr(*it) + "]";
                     _mlxParams.setParams.push_back(paramView);
                 }
                 continue;
             } else {
-                pv.mlxconfigName = mlxconfigName;
+
+                size_t p = mlxconfigName.find('[');
+                string subStr = mlxconfigName.substr(0, p);
+                if (isIndexedStartFromOneSupported(subStr)) {
+                    if ((int32_t)indexes[0] <= 0) {
+                        printf("-E- Index is out of range.Minimal index is 1\n");
+                        return MLX_CFG_ERROR_NO_USAGE;
+                    }
+                    pv.mlxconfigName = subStr + "[" + numToStr(indexes[0] - 1) + "]";
+
+                }
+                else {
+                    pv.mlxconfigName = mlxconfigName;
+                }
             }
         } else {
             pv.mlxconfigName = mlxconfigName;
@@ -262,21 +291,46 @@ mlxCfgStatus MlxCfg::extractSetCfgArgs(int argc, char *argv[])
             string indexStr = parseIndexStr(tag);
             vector<u_int32_t> indexes;
             extractIndexes(indexStr, indexes);
+            bool isStartFromOneSupported = false;
             if (indexes.size() > 1) {
                 size_t p = tag.find('[');
                 if (p == std::string::npos) {
                     return err(true, "Expected a parameter with index");
                 }
                 string mlxconfigName = tag.substr(0, p);
+                isStartFromOneSupported = isIndexedStartFromOneSupported(mlxconfigName);
                 VECTOR_ITERATOR(u_int32_t, indexes, it) {
                     ParamView paramView;
                     paramView.setVal = strVal;
+
+                    if (isStartFromOneSupported) {
+                        if (*it == 0) {
+                            printf("-E- Index 0 is out of range.Minimal index is 1\n");
+                            return MLX_CFG_ERROR_NO_USAGE;
+                        }
+                        else {
+                            (*it)--;
+                        }
+                    }
+
                     paramView.mlxconfigName = mlxconfigName + "[" + numToStr(*it) + "]";
                     _mlxParams.setParams.push_back(paramView);
                 }
                 continue;
             } else {
-                pv.mlxconfigName = tag;
+                size_t p = tag.find('[');
+                string subStr = tag.substr(0, p);
+                if (isIndexedStartFromOneSupported(subStr)) {
+                    if (indexes[0] == 0) {
+                        printf("-E- Index 0 is out of range.Minimal index is 1\n");
+                        return MLX_CFG_ERROR_NO_USAGE;
+                    }
+                    pv.mlxconfigName = subStr + "[" + numToStr(indexes[0] - 1) + "]";
+
+                }
+                else {
+                    pv.mlxconfigName = tag;
+                }
             }
         } else {
             pv.mlxconfigName = tag;
@@ -351,6 +405,9 @@ mlxCfgStatus MlxCfg::parseArgs(int argc, char *argv[])
         } else if (arg == "set_raw") {
             _mlxParams.cmd = Mc_Set_Raw;
             break;
+        } else if (arg == "get_raw") {
+            _mlxParams.cmd = Mc_Get_Raw;
+            break;
         } else if (arg == "backup") {
             _mlxParams.cmd = Mc_Backup;
             break;
@@ -400,18 +457,21 @@ mlxCfgStatus MlxCfg::parseArgs(int argc, char *argv[])
         return err(true, "%s command expects device to be specified.",
                    _mlxParams.cmd == Mc_Set ?
                    "set" : _mlxParams.cmd == Mc_Set_Raw ?
-                   "set_raw" : _mlxParams.cmd == Mc_Clr_Sem ?
+                   "set_raw" : _mlxParams.cmd == Mc_Get_Raw ?
+                   "get_raw" : _mlxParams.cmd == Mc_Clr_Sem ?
                    "clear_semaphore" : _mlxParams.cmd == Mc_Backup ?
                    "backup" : _mlxParams.cmd == Mc_Apply ?
                    "apply" : "show_confs");
     }
-    if ((_mlxParams.cmd == Mc_Set_Raw && _mlxParams.rawTlvFile.size() == 0 )) {
+    if (((_mlxParams.cmd == Mc_Set_Raw || _mlxParams.cmd == Mc_Get_Raw) &&
+            _mlxParams.rawTlvFile.size() == 0 )) {
         return err(true, "set_raw command expects raw TLV file to be specified.");
     }
     if ((_mlxParams.cmd == Mc_Backup && _mlxParams.rawTlvFile.size() == 0 )) {
         return err(true, "backup command expects file to be specified.");
     }
-    if (((_mlxParams.cmd != Mc_Set_Raw && _mlxParams.cmd != Mc_Backup) &&
+    if ((((_mlxParams.cmd != Mc_Set_Raw && _mlxParams.cmd != Mc_Get_Raw) &&
+            _mlxParams.cmd != Mc_Backup) &&
          _mlxParams.rawTlvFile.size() != 0 )) {
         return err(true, "raw TLV file can only be specified with set_raw command.");
     }
