@@ -52,6 +52,7 @@ typedef enum {
     Mc_Reset,
     Mc_Clr_Sem,
     Mc_Set_Raw,
+    Mc_Get_Raw,
     Mc_Backup,
     Mc_ShowConfs,
     Mc_GenTLVsFile,
@@ -64,6 +65,18 @@ typedef enum {
     Mc_UnknownCmd
 } mlxCfgCmd;
 
+typedef struct QueryOutputItem {
+    string mlxconfigName;
+    u_int32_t nextVal;
+    string strNextVal;
+    u_int32_t currVal;
+    string strCurrVal;
+    u_int32_t defVal;
+    string strDefVal;
+    u_int32_t setVal;
+    string strSetVal;
+} QueryOutputItem;
+
 using namespace std;
 
 class MlxCfgParams
@@ -113,6 +126,9 @@ private:
     mlxCfgStatus extractSetCfgArgs(int argc, char *argv[]);
     mlxCfgStatus extractQueryCfgArgs(int argc, char *argv[]);
 
+    void removeContinuanceArray(std::vector<QueryOutputItem>& OutputItemOut, std::vector<QueryOutputItem>& OutputItemIn);
+    void editAndPushItem(std::vector<QueryOutputItem>& queryOutputItemVector, QueryOutputItem& item, u_int32_t arrayIndex);
+
     const char * getConfigWarning(const string & mlx_config_name,
             const string & set_val);
 
@@ -129,8 +145,8 @@ private:
     // reset Cmd
     mlxCfgStatus resetDevsCfg();
     mlxCfgStatus resetDevCfg(const char *dev);
-    // Set Raw TLV file
-    mlxCfgStatus setDevRawCfg();
+    // Set\Get Raw TLV file
+    mlxCfgStatus devRawCfg(RawTlvMode mode);
     mlxCfgStatus backupCfg();
     mlxCfgStatus tlvLine2DwVec(const std::string& tlvStringLine, std::vector<u_int32_t>& tlvVec);
 
@@ -155,6 +171,13 @@ private:
     mlxCfgStatus createConf();
     mlxCfgStatus apply();
 
+    // static print functions
+    static int printParam(string param, u_int32_t val);
+    static int printValue(string strVal, u_int32_t val);
+    static void printSingleParam(const char *name,
+            QueryOutputItem & queryOutItem, u_int8_t verbose, bool printNewCfg);
+
+
     bool askUser(const char *question, bool add_prefix=true, bool add_suffix=true);
     mlxCfgStatus err(bool report, const char *errMsg, ...);
     void printErr();
