@@ -38,6 +38,7 @@
 #ifndef FLINT_BASE_H
 #define FLINT_BASE_H
 
+
 #ifndef UEFI_BUILD
     #ifndef __FreeBSD__
     #include <malloc.h>
@@ -197,28 +198,7 @@ namespace std {}; using namespace std;
 
 #define FS2_BOOT_START   0x38
 #define FS_DATA_OFF      0x28
-#define SWITCHX_HW_ID    581
-#define SWITCH_IB_HW_ID  583
-#define SPECTRUM_HW_ID   585
-#define SWITCH_IB2_HW_ID 587
-#define QUANTUM_HW_ID    589
-#define SPECTRUM2_HW_ID  590
-#define SPECTRUM3_HW_ID  592
 
-
-#define CX4_HW_ID         521
-#define CX4LX_HW_ID       523
-#define CX5_HW_ID         525
-#define CX6_HW_ID         527
-#define CX6DX_HW_ID       530
-#define CX6LX_HW_ID       534
-#define BF_HW_ID          529
-#define BF2_HW_ID         532
-#define CX2_HW_ID         400
-#define CX3_HW_ID         501
-#define CX3_PRO_HW_ID     503
-#define IS4_HW_ID         435
-#define CONNECT_IB_HW_ID  511
 #define CONNECT_IB_SW_ID  4113
 
 // FS3 defines
@@ -320,6 +300,8 @@ typedef enum fs3_section {
     FS3_IMAGE_SIGNATURE_512 = 0xa3,
     FS3_PUBLIC_KEYS_4096 = 0xa4,
     FS3_HMAC          = 0xa5,
+    FS4_RSA_PUBLIC_KEY = 0xa6,
+    FS4_RSA_4096_SIGNATURES = 0xa7,
     FS3_MFG_INFO      = 0xe0,
     FS3_DEV_INFO      = 0xe1,
     FS3_NV_DATA1      = 0xe2,
@@ -328,8 +310,9 @@ typedef enum fs3_section {
     FS3_FW_NV_LOG     = 0xe5,
     FS3_NV_DATA0      = 0xe6, // replaces FS3_NV_DATA1
     FS3_CRDUMP_MASK_DATA = 0xe9,
-    FS4_FW_INTERNAL_USAGE = 0xea,
-    FS4_FW_INTERNAL_USAGE1 = 0xeb,
+    FS4_PART_TYPE_FW_INTERNAL_USAGE = 0xea,
+    FS4_PART_TYPE_PROGRAMMABLE_HW_FW1 = 0xeb,
+    FS4_PART_TYPE_PROGRAMMABLE_HW_FW2 = 0xec,
     FS4_HW_PTR        = 0xfb,
     FS4_TOOLS_AREA    = 0xfc,
     FS3_ITOC          = 0xfd,
@@ -374,7 +357,8 @@ enum CommandType {
     CMD_SET_SIGNATURE,
     CMD_SET_PUBLIC_KEYS,
     CMD_SET_FORBIDDEN_VERSIONS,
-    CMD_SET_HMAC
+    CMD_SET_HMAC,
+    CMD_SET_PUBLIC_4096_RSA_KEY
 };
 
 
