@@ -31,8 +31,10 @@
  */
 
 #include "fsctrl_ops.h"
+
 #include <tools_utils.h>
 #include <bit_slice.h>
+
 #include <vector>
 
 bool FsCtrlOperations::unsupportedOperation()
@@ -121,9 +123,13 @@ bool FsCtrlOperations::FsIntQuery()
     if (fwQuery.running_fw_version.version_string_length) {
         strcpy(_fwImgInfo.ext_info.product_ver, fwQuery.product_ver);
     }
-    // if nextBootFwVer, only fw version is needed, return.
+    // if nextBootFwVer, only fw version is needed and chip type, return.
     if (nextBootFwVer) {
-         return true;
+        const u_int32_t *swId = (u_int32_t*)NULL;
+        if (!getInfoFromHwDevid(fwQuery.hw_dev_id, _fwImgInfo.ext_info.chip_type, &swId)) {
+            return false;
+        }
+        return true;
     }
 
     _fsCtrlImgInfo.fs3_uids_info.cx4_uids.base_mac.uid = fwQuery.base_mac.uid;
@@ -153,6 +159,8 @@ bool FsCtrlOperations::FsIntQuery()
                                     ((fwQuery.security_type.secure_fw == 1) ? SMM_SECURE_FW : 0) |
                                     ((fwQuery.security_type.dev_fw    == 1) ? SMM_DEV_FW    : 0));
 
+    _fsCtrlImgInfo.sec_boot = fwQuery.sec_boot;
+    _fsCtrlImgInfo.life_cycle = fwQuery.life_cycle;
     std::vector<FwComponent> compsMap;
     if (!_fwCompsAccess->getFwComponents(compsMap, false)) {
         return errmsg(FwCompsErrToFwOpsErr(_fwCompsAccess->getLastError()), "Failed to get the FW Components MAP, err[%d]", _fwCompsAccess->getLastError());
@@ -188,38 +196,40 @@ bool FsCtrlOperations::FsIntQuery()
     strncpy(_fsCtrlImgInfo.name, fwQuery.name, NAME_LEN);
     strncpy(_fsCtrlImgInfo.description, fwQuery.description, DESCRIPTION_LEN);
     strncpy(_fsCtrlImgInfo.deviceVsd, fwQuery.deviceVsd, VSD_LEN);
+    if (FwType() == FIT_FS3) {
+        memcpy(_fwImgInfo.ext_info.vsd, fwQuery.deviceVsd, VSD_LEN);
+    }
     strncpy(_fsCtrlImgInfo.image_vsd, fwQuery.imageVsd, VSD_LEN);
     return true;
 }
 
-
 bool FsCtrlOperations::FwReactivateImage()
 {
     if (!_fwCompsAccess->fwReactivateImage()) {
         fw_comps_error_t errCode = _fwCompsAccess->getLastError();
         switch (errCode) {
         case FWCOMPS_IMAGE_REACTIVATION_PROHIBITED_FW_VER_ERR:
-            return errmsg(MLXFW_IMAGE_REACTIVATION_PROHIBITED_FW_VER_ERR, "Prohibited by current FW version\n");
+            return errmsg(MLXFW_IMAGE_REACTIVATION_PROHIBITED_FW_VER_ERR, "Prohibited by current FW version");
         case FWCOMPS_IMAGE_REACTIVATION_FIRST_PAGE_COPY_FAILED:
-            return errmsg(MLXFW_IMAGE_RACTIVATION_FIRST_PAGE_COPY_FAILED, "First page copy failed\n");
+            return errmsg(MLXFW_IMAGE_RACTIVATION_FIRST_PAGE_COPY_FAILED, "First page copy failed");
         case FWCOMPS_IMAGE_REACTIVATION_FIRST_PAGE_ERASE_FAILED:
-            return errmsg(MLXFW_IMAGE_REACTIVATION_FIRST_PAGE_ERASE_FAILED, "First page erase failed\n");
+            return errmsg(MLXFW_IMAGE_REACTIVATION_FIRST_PAGE_ERASE_FAILED, "First page erase failed");
         case FWCOMPS_IMAGE_REACTIVATION_FIRST_PAGE_RESTORE_FAILED:
-            return errmsg(MLXFW_IMAGE_REACTIVATION_FIRST_PAGE_RESTORE_FAILED, "First page restore failed\n");
+            return errmsg(MLXFW_IMAGE_REACTIVATION_FIRST_PAGE_RESTORE_FAILED, "First page restore failed");
         case FWCOMPS_IMAGE_REACTIVATION_FW_DEACTIVATION_FAILED:
-            return errmsg(MLXFW_IMAGE_REACTIVATION_FW_DEACTIVATION_FAILED, "FW deactivation failed\n");
+            return errmsg(MLXFW_IMAGE_REACTIVATION_FW_DEACTIVATION_FAILED, "FW deactivation failed");
         case FWCOMPS_IMAGE_REACTIVATION_FW_ALREADY_ACTIVATED:
-            return errmsg(MLXFW_IMAGE_REACTIVATION_FW_ALREADY_ACTIVATED, "FW already reactivated\n");
+            return errmsg(MLXFW_IMAGE_REACTIVATION_FW_ALREADY_ACTIVATED, "FW already reactivated");
         case FWCOMPS_IMAGE_REACTIVATION_ERROR_DEVICE_RESET_REQUIRED:
-            return errmsg(MLXFW_IMAGE_REACTIVATION_ERROR_DEVICE_RESET_REQUIRED, "FW reset required\n");
+            return errmsg(MLXFW_IMAGE_REACTIVATION_ERROR_DEVICE_RESET_REQUIRED, "FW reset required");
         case FWCOMPS_IMAGE_REACTIVATION_FW_PROGRAMMING_NEEDED:
-            return errmsg(MLXFW_IMAGE_REACTIVATION_FW_PROGRAMMING_NEEDED, "FW programming needed\n");
+            return errmsg(MLXFW_IMAGE_REACTIVATION_FW_PROGRAMMING_NEEDED, "FW programming needed");
         case FWCOMPS_GENERAL_ERR:
             return errmsg(MLXFW_ERR, "FW ICMD related error");
         case FWCOMPS_IMAGE_REACTIVATION_WAITING_TIME_EXPIRED:
-            return errmsg(MLXFW_ERR, "Image reactivation - timeout expired.\n");
+            return errmsg(MLXFW_ERR, "Image reactivation - timeout expired");
         case FWCOMPS_IMAGE_REACTIVATION_FW_NOT_SUPPORTED:
-            return errmsg(MLXFW_ERR, "Image reactivation - FW doesn't support this operation.\n");
+            return errmsg(MLXFW_ERR, "Image reactivation - FW doesn't support this operation");
         default:
             return errmsg(MLXFW_IMAGE_REACTIVATION_UNKNOWN_ERROR, "Unknown error occurred\n");
         }
@@ -389,10 +399,12 @@ bool FsCtrlOperations::VerifyAllowedParams(ExtBurnParams &burnParams, bool isSec
     return true;
 }
 
-bool FsCtrlOperations::FwBurnAdvanced(std::vector <u_int8_t> imageOps4MData, ExtBurnParams& burnParams)
+
+bool FsCtrlOperations::FwBurnAdvanced(std::vector <u_int8_t> imageOps4MData, ExtBurnParams& burnParams, FwComponent::comps_ids_t ComponentId)
 {
-    return _Burn(imageOps4MData, burnParams);
+    return _Burn(imageOps4MData, burnParams, ComponentId);
 }
+
 bool FsCtrlOperations::FwBurnAdvanced(FwOperations *imageOps, ExtBurnParams &burnParams)
 {
     if (imageOps == NULL) {
@@ -402,7 +414,6 @@ bool FsCtrlOperations::FwBurnAdvanced(FwOperations *imageOps, ExtBurnParams &bur
         return false;
     }
     fw_info_t fw_query;
-
     memset(&fw_query, 0, sizeof(fw_info_t));
     if (!imageOps->FwQuery(&fw_query, true)) {
         return errmsg(FwCompsErrToFwOpsErr(_fwCompsAccess->getLastError()), "Failed to query the image\n");
@@ -425,7 +436,7 @@ bool FsCtrlOperations::FwBurnAdvanced(FwOperations *imageOps, ExtBurnParams &bur
     }
     // Check TimeStamp
     if (!TestAndSetTimeStamp(imageOps)) {
-       return false;
+        return false;
     }
     std::vector <u_int8_t> imageOps4MData;
     if (!imageOps->FwExtract4MBImage(imageOps4MData, true)) {
@@ -434,7 +445,7 @@ bool FsCtrlOperations::FwBurnAdvanced(FwOperations *imageOps, ExtBurnParams &bur
     return _Burn(imageOps4MData, burnParams);
 }
 
-bool FsCtrlOperations::_Burn(std::vector <u_int8_t> imageOps4MData, ExtBurnParams& burnParams)
+bool FsCtrlOperations::_Burn(std::vector <u_int8_t> imageOps4MData, ExtBurnParams& burnParams, FwComponent::comps_ids_t ComponentId)
 {
 #ifdef UEFI_BUILD
     burnParams.ProgressFuncAdv.uefi_func =  burnParams.progressFunc;
@@ -444,7 +455,7 @@ bool FsCtrlOperations::_Burn(std::vector <u_int8_t> imageOps4MData, ExtBurnParam
     FwComponent bootImageComponent;
     std::vector<FwComponent> compsToBurn;
 
-    bootImageComponent.init(imageOps4MData, imageOps4MData.size(), FwComponent::COMPID_BOOT_IMG);
+    bootImageComponent.init(imageOps4MData, imageOps4MData.size(), ComponentId);
     compsToBurn.push_back(bootImageComponent);
     if (!_fwCompsAccess->lock_flash_semaphore()) {
         return errmsg(FwCompsErrToFwOpsErr(_fwCompsAccess->getLastError()), "%s", _fwCompsAccess->getLastErrMsg());
@@ -457,6 +468,7 @@ bool FsCtrlOperations::_Burn(std::vector <u_int8_t> imageOps4MData, ExtBurnParam
     return true;
 }
 
+
 bool FsCtrlOperations::FwBurnBlock(FwOperations *imageOps, ProgressCallBack progressFunc)
 {
     (void) imageOps;
@@ -584,7 +596,7 @@ bool FsCtrlOperations::FwShiftDevData(PrintCallBack progressFunc)
 
 const char* FsCtrlOperations::FwGetResetRecommandationStr()
 {
-#if defined( __VMKERNEL_UW_VMKLINUX__) || defined(__VMKERNEL_UW_NATIVE__)
+#if defined(__VMKERNEL_UW_NATIVE__)
     return REBOOT_REQUIRED_STR;
 #else
     return REBOOT_OR_FWRESET_REQUIRED_STR;
@@ -759,3 +771,19 @@ bool FsCtrlOperations::FwQueryTimeStamp(struct tools_open_ts_entry& timestamp, s
     delete tsObj;
     return rc ? false : true;
 }
+
+bool FsCtrlOperations::GetSecureBootInfo()
+{
+    return _signatureMngr->GetSecureBootInfo();
+}
+
+bool FsCtrlOperations::IsLifeCycleSupported()
+{
+    return _signatureMngr->IsLifeCycleSupported();
+}
+
+bool FsCtrlOperations::IsCableQuerySupported()
+{
+    return _signatureMngr->IsCableQuerySupported();
+}
+
