@@ -39,11 +39,12 @@
 #include "aux_tlv_ops.h"
 #include "mlxfwops_com.h"
 #include "signature_manager_factory.h"
-
+#include "fw_version.h"
+#include "tools_layouts/cx4fw_layouts.h"
 #ifdef CABLES_SUPP
 #include <cable_access/cable_access.h>
 #endif
-
+#include <fw_comps_mgr/fw_comps_mgr.h>
 typedef f_prog_func_str VerifyCallBack;
 typedef f_prog_func ProgressCallBack;
 typedef f_prog_func_ex ProgressCallBackEx;
@@ -52,12 +53,11 @@ typedef f_prog_func_str PrintCallBack;
 typedef fw_ver_info_t FwVerInfo;
 
 typedef int (*PrintCallBackAdv) (int completion, char *str);
-
-extern bool nextBootFwVer;
+MLXFWOP_API extern bool nextBootFwVer;
 #define GLOBAL_ALIGNMENT 0x80
+#define UUID_LEN 16
+enum SHATYPE { SHA256, SHA512};
 class MLXFWOP_API FwOperations : public FlintErrMsg {
-
-
 public:
     #define EXP_ROM_GEN_DEVID 0
     #define MISS_MATCH_DEV_ID 0xffff
@@ -68,7 +68,14 @@ public:
     typedef fwOpsParams fw_ops_params_t;
     typedef sgParams sg_params_t;
     // typedef std::tr1::function<void (void)> VerifyCallback;
-
+    enum {
+        IMG_SIG_TYPE_UNKNOWN = 0,
+        IMG_SIG_TYPE_BIN = 1,
+        IMG_SIG_TYPE_MFA = 2,
+        IMG_SIG_TYPE_CF = 3,
+        IMG_SIG_TYPE_CC = 4,
+        IMG_SIG_OPEN_FILE_FAILED = 5
+    };
     FwOperations(FBase *ioAccess) :
         _ioAccess(ioAccess), _isCached(false), _wasVerified(false),
         _quickQuery(false), _printFunc((PrintCallBack)NULL), _fname((const char*)NULL), \
@@ -80,14 +87,12 @@ public:
         memset(&_fwParams, 0, sizeof(_fwParams));
     };
 
-
-    void CreateSignatureManager() {
-        if (_ioAccess != NULL && _ioAccess->is_flash() == false) {//NOT RELEVANT FOR IMAGE OBJECT
-            return;
-        }
-        u_int32_t hwDevId = GetHwDevId();
-        _signatureMngr = SignatureManagerFactory::GetInstance()->CreateSignatureManager(hwDevId, _fwImgInfo.ext_info.dev_rev);
+    bool IsFifthGen() {
+        return (_ioAccess != NULL && _ioAccess->is_flash() && _ioAccess->is_fifth_gen());
     }
+
+    bool CreateSignatureManager();
+
     virtual ~FwOperations()
     {
         if (_ioAccess) {
@@ -102,15 +107,16 @@ public:
         mfile *mf = _ioAccess->is_flash() ? ((Flash *)_ioAccess)->getMfileObj() : (mfile *)NULL;
         return mf;
     }
+    FBase* GetIoAccess() { return _ioAccess; }
     virtual u_int8_t FwType() = 0;
-    static FwVerInfo FwVerLessThan(u_int16_t r1[3], u_int16_t r2[3]);
-    static bool IsFwSupportingRomModify(u_int16_t fw_ver[3]);
+    static bool IsFwSupportingRomModify(const FwVersion&);
     static bool CntxEthOnly(u_int32_t devid);
     static void SetDevFlags(chip_type_t chipType, u_int32_t devType, fw_img_type_t fwType, bool &ibDev, bool &ethDev);
     static bool checkMatchingExpRomDevId(const fw_info_t& info);
     static bool checkMatchingExpRomDevId(u_int16_t dev_type, const roms_info_t& roms_info);
     static const char* expRomType2Str(u_int16_t type);
-
+    static chip_type GetChipType(string chip);
+    static chip_type_t getChipType(u_int32_t devid);
     bool readBufAux(FBase& f, u_int32_t o, void *d, int l, const char *p);
     virtual bool FwQuery(fw_info_t *fwInfo, bool readRom = true, bool isStripedImage = false, bool quickQuery = true, bool ignoreDToc = false, bool verbose = false) = 0;
     virtual bool FwVerify(VerifyCallBack verifyCallBackFunc, bool isStripedImage = false, bool showItoc = false, bool ignoreDToc = false) = 0; // Add callback print
@@ -118,19 +124,28 @@ public:
     //on call of FwReadData with Null image we get image_size
     virtual bool FwReadData(void *image, u_int32_t *image_size, bool verbose = false) = 0;
     virtual bool FwReadBlock(u_int32_t addr, u_int32_t size, std::vector<u_int8_t>& dataVec);
-    virtual bool FwReactivateImage() 
-    { 
-        return errmsg("Operation not supported."); 
+    virtual u_int32_t GetPublicKeySecureBootPtr();
+    virtual bool FwReactivateImage()
+    {
+        return errmsg("Operation not supported.");
     }
+    virtual bool PreparePublicKeyData(const char *public_key_file, vector <u_int8_t>& publicKeyData, unsigned int& pem_offset);
+    virtual bool PrepareSecureBootSections(vector<u_int8_t>& encShaBinData, vector<u_int8_t>& encShaCritical, vector<u_int8_t>& encShaNonCritical,
+        vector <u_int32_t> uuidData, vector <u_int8_t> publicKeyData, unsigned int pemOffset);
     virtual bool FwInsertSHA256(PrintCallBack printFunc = (PrintCallBack)NULL);
+    virtual bool InsertSecureFWSignature(vector<u_int8_t> signature, const char *uuid, PrintCallBack printFunc);
+    virtual bool InsertSecureBootSignature(vector<u_int8_t> encShaBinData, vector<u_int8_t> encShaCritical, vector<u_int8_t> encShaNonCritical);
     virtual bool FwSignWithOneRSAKey(const char *privPemFile, const char *uuid, PrintCallBack printFunc = (PrintCallBack)NULL);
     virtual bool FwSignWithTwoRSAKeys(const char *privPemFile1, const char *uuid1,
                                       const char *privPemFile2, const char *uuid2, PrintCallBack printFunc = (PrintCallBack)NULL);
     virtual bool FwSignWithHmac(const char *key_file);
+    virtual bool FwSignWithRSA(const char *private_key_file, const char *public_key_file, const char *guid_key_file);
+    virtual bool FwSignWithRSA(const char *public_key_file, const char *uuid, vector<u_int8_t>& bin_data, vector<u_int8_t>& critical, vector<u_int8_t>& non_critical);
     virtual bool PrepItocSectionsForHmac(vector<u_int8_t>& critical, vector<u_int8_t>& non_critical);
     virtual bool IsCriticalSection(u_int8_t sect_type);
 
-    virtual bool FwExtract4MBImage(vector<u_int8_t>& img, bool maskMagicPatternAndDevToc, bool verbose = false);
+    virtual bool FwExtract4MBImage(vector<u_int8_t>& img, bool maskMagicPatternAndDevToc, bool verbose = false, bool ignoreImageStart = false);
+    virtual bool RestoreDevToc(vector<u_int8_t>& img, char* psid, dm_dev_id_t devid_t, const cx4fw_uid_entry& base_guid, const cx4fw_uid_entry& base_mac);
     virtual bool FwSetPublicKeys(char *fname, PrintCallBack callBackFunc = (PrintCallBack)NULL);
     virtual bool FwSetForbiddenVersions(char *fname, PrintCallBack callBackFunc = (PrintCallBack)NULL);
     virtual bool CalcHMAC(const vector<u_int8_t>& key, const vector<u_int8_t>& data, vector<u_int8_t>& digest);
@@ -140,7 +155,8 @@ public:
 
     virtual bool FwBurn(FwOperations *imageOps, u_int8_t forceVersion, ProgressCallBack progressFunc = (ProgressCallBack)NULL) = 0;
     virtual bool FwBurnAdvanced(FwOperations *imageOps, ExtBurnParams& burnParams) = 0;
-    virtual bool FwBurnAdvanced(std::vector <u_int8_t> imageOps4MData, ExtBurnParams& burnParams);
+    virtual bool getExtendedHWAravaPtrs(VerifyCallBack verifyCallBackFunc, FBase* ioAccess, bool IsBurningProcess);
+    virtual bool FwBurnAdvanced(std::vector <u_int8_t> imageOps4MData, ExtBurnParams& burnParams, FwComponent::comps_ids_t ComponentId = FwComponent::COMPID_BOOT_IMG);
     virtual bool FwBurnBlock(FwOperations *imageOps, ProgressCallBack progressFunc) = 0; //Add: callback progress, question arr, callback question, configurations
     virtual bool FwWriteBlock(u_int32_t addr, std::vector<u_int8_t> dataVec, ProgressCallBack progressFunc = (ProgressCallBack)NULL);
 
@@ -169,7 +185,11 @@ public:
 
     virtual bool CheckIfAlignmentIsNeeded(FwOperations*) { return false; }
     virtual bool AlignDeviceSections(FwOperations*/*imageOps*/) { return errmsg("Align device sections is not supported"); }
-
+    virtual bool RemoveWriteProtection() 
+    { 
+        return true; 
+        //Though Remove write protection is not supported, return true.
+    }
     void FwCleanUp();
     virtual bool FwInit() = 0;
     virtual bool FsIntQuery() { return true; }
@@ -199,12 +219,22 @@ public:
 
     virtual bool IsFsCtrlOperations();
     virtual bool PrepItocSectionsForCompare(vector<u_int8_t>& critical, vector<u_int8_t>& non_critical);
-
+    virtual bool GetSecureBootInfo();
+    virtual bool IsCableQuerySupported();
+    virtual bool IsLifeCycleSupported();
     void GetFwParams(fw_ops_params_t&);
-
+    virtual void CleanInterruptedCommand() {}//by default do nothing
+    virtual bool FwCalcSHA(SHATYPE, vector<u_int8_t>&, vector<u_int8_t>&);
     //bool GetExpRomVersionWrapper();
     void getSupporteHwId(u_int32_t **supportedHwId, u_int32_t &supportedHwIdNum);
 
+    static FwVersion createFwVersion(const fw_info_com_t*);
+    static FwVersion createFwVersion(u_int16_t fw_ver0, u_int16_t fw_ver1, u_int16_t fw_ver2);
+    static FwVersion createRunningFwVersion(const fw_info_com_t*);
+    static int       getFileSignature(const char *fname);
+#ifndef UEFI_BUILD
+    static bool CheckPemKeySize(const string privPemFileStr, u_int32_t& keySize);
+#endif
     class MLXFWOP_API RomInfo : FlintErrMsg {
 public:
         RomInfo(const std::vector<u_int8_t>& romSector, bool resEndi = true);
@@ -268,15 +298,19 @@ public:
         void *progressUserData;
         //data
         char *userVsd;
+        bool use_cpu_utilization;
+        int cpu_utilization;
         std::vector<guid_t> userUids; //contains either guids or uids
         ExtBurnStatus burnStatus;
-
+        chip_type_t chip_type;
+        bool use_chip_type;
         ExtBurnParams() : userGuidsSpecified(false), userMacsSpecified(false), userUidSpecified(false),
             vsdSpecified(false), blankGuids(false), burnFailsafe(true), allowPsidChange(false),
             useImagePs(false), useImageGuids(false), singleImageBurn(true), noDevidCheck(false),
             skipCiReq(false), ignoreVersionCheck(false), useImgDevData(false), useDevImgInfo(false),
             burnRomOptions(BRO_DEFAULT), shift8MBIfNeeded(false), progressFunc((ProgressCallBack)NULL),
-            progressFuncEx((ProgressCallBackEx)NULL), progressUserData(NULL), userVsd((char*)NULL)
+            progressFuncEx((ProgressCallBackEx)NULL), progressUserData(NULL), userVsd((char*)NULL), use_cpu_utilization(false), cpu_utilization(-1),
+            chip_type(CT_UNKNOWN), use_chip_type(false)
         { ProgressFuncAdv.func = (f_prog_func_adv)NULL; ProgressFuncAdv.opaque = NULL;}
 
         void updateParamsForBasicImage(ProgressCallBack progressFunc)
@@ -329,6 +363,7 @@ public:
         int isCableFw;
         bool noFwCtrl;
         bool mccUnsupported;
+        bool canSkipFwCtrl;
     };
 
     struct sgParams {
@@ -372,13 +407,6 @@ protected:
     enum {
         MAX_SW_DEVICES_PER_HW = 32
     };
-    enum {
-        IMG_SIG_TYPE_UNKNOWN = 0,
-        IMG_SIG_TYPE_BIN = 1,
-        IMG_SIG_TYPE_MFA = 2,
-        IMG_SIG_TYPE_CF  = 3,
-        IMG_SIG_OPEN_FILE_FAILED = 4
-    };
 
     enum {
         IMG_VER_FS2 = 0,
@@ -451,7 +479,8 @@ public:
                     VerifyCallBack verifyCallBackFunc = (VerifyCallBack)NULL);
     u_int32_t CalcImageCRC(u_int32_t *buff, u_int32_t size);
     bool writeImage(ProgressCallBack progressFunc, u_int32_t addr, void *data, int cnt, bool isPhysAddr = false, bool readModifyWrite = false, int totalSz = -1, int alreadyWrittenSz = 0);
-    bool writeImageEx(ProgressCallBackEx progressFuncEx, void *progressUserData, ProgressCallBack progressFunc, u_int32_t addr, void *data, int cnt, bool isPhysAddr = false, bool readModifyWrite = false, int totalSz = -1, int alreadyWrittenSz = 0);
+    bool writeImageEx(ProgressCallBackEx progressFuncEx, void *progressUserData, ProgressCallBack progressFunc, u_int32_t addr, void *data, int cnt, bool isPhysAddr = false, 
+    bool readModifyWrite = false, int totalSz = -1, int alreadyWrittenSz = 0, bool cpuUtilization = false, int cpuPercent = -1);
     //////////////////////////////////////////////////////////////////
     bool GetSectData(std::vector<u_int8_t>& file_sect, const u_int32_t *buff, const u_int32_t size);
     ////////////////////////////////////////////////////////////////////
@@ -495,7 +524,6 @@ public:
     std::vector<u_int8_t> _fwConfSect;
     std::vector<u_int8_t> _hashFileSect;
     std::vector<u_int8_t> _readSectList;
-
     bool _sectionsToRead[H_LAST];
     bool _wasVerified;
     bool _quickQuery;
@@ -521,7 +549,7 @@ private:
     static int      getMfaImg(char *fileName, char *psid, u_int8_t **imgbuf, char *errBuf, int errBufSize);
     static int      getMfaImg(u_int8_t *mfa_buf, int size, char *psid, u_int8_t **imgbuf, char *errMsg, int errBufSize);
 #endif
-    static int      getFileSignature(const char *fname);
+    
     static int      getBufferSignature(u_int8_t *buf, u_int32_t size);
     static u_int8_t CheckFwFormat(FBase& f, bool getFwFormatFromImg = false);
     static u_int8_t IsFS4Image(FBase& f, u_int32_t *found_images);
@@ -537,7 +565,7 @@ private:
      * @param[in] errBuff - pointer to dist error buffer
      * @param[in] errStr - pointer to source string
      * @param[in] bufSize - size of error buffer */
-    static void WriteToErrBuff(char *errBuff, const char *errStr, int bufSize);
+    static void WriteToErrBuff(char *errBuff, char *errStr, int bufSize);
 
     void BackUpFwParams(fw_ops_params_t& fwParams);
     // Methods
