@@ -57,6 +57,35 @@
 #define EFIAPI
 #endif
 
+#define CONNECTX_HW_ID   400
+#define SWITCHX_HW_ID    581
+#define SWITCH_IB_HW_ID  583
+#define SPECTRUM_HW_ID   585
+#define SWITCH_IB2_HW_ID 587
+#define QUANTUM_HW_ID    589
+#define SPECTRUM2_HW_ID  590
+#define SPECTRUM3_HW_ID  592
+#define QUANTUM2_HW_ID   599
+#define SPECTRUM4_HW_ID  596
+#define GEARBOX_HW_ID    594
+#define GB_MANAGER_HW_ID 595
+#define CX4_HW_ID         521
+#define CX4LX_HW_ID       523
+#define CX5_HW_ID         525
+#define CX6_HW_ID         527
+#define CX6DX_HW_ID       530
+#define CX6LX_HW_ID       534
+#define CX7_HW_ID         536
+#define BF_HW_ID          529
+#define BF2_HW_ID         532
+#define CX2_HW_ID         400
+#define CX3_HW_ID         501
+#define CX3_PRO_HW_ID     503
+#define IS4_HW_ID         435
+#define CONNECT_IB_HW_ID  511
+#define INBAND_MAX_REG_SIZE 44
+#define MCDA_REG_HEADER  16
+
 typedef enum {
     PROG_WITH_PRECENTAGE,
     PROG_WITHOUT_PRECENTAGE,
@@ -72,6 +101,7 @@ typedef enum
     RMA = 3,
     NUM_OF_LIFE_CYCLES = 4
 }life_cycle_t;
+
 typedef int EFIAPI (*f_prog_func)(int completion);
 typedef int EFIAPI (*f_prog_func_ex)(int completion, void *opaque);
 typedef int EFIAPI (*f_prog_func_adv)(int completion, const char *str, prog_t, void *opaque);
@@ -113,6 +143,7 @@ typedef struct {
         } \
     } while (0) \
 
+
 enum {
     MLXFW_OK = 0,
     MLXFW_ERR,
@@ -252,18 +283,27 @@ typedef enum chip_type {
     CT_SPECTRUM2,
     CT_CONNECTX6DX,
     CT_CONNECTX6LX,
+    CT_CONNECTX7,
     CT_SPECTRUM3,
-    CT_BLUEFIELD2
+    CT_BLUEFIELD2,
+    CT_CONNECTX3,
+    CT_QUANTUM2,
+    CT_SPECTRUM4,
+    CT_GEARBOX,
+    CT_GEARBOX_MGR
 } chip_type_t;
 
 #define IS_HCA(chipType) \
     (((chipType) == CT_CONNECTX) || ((chipType) == CT_CONNECT_IB) || ((chipType) == CT_CONNECTX4) || ((chipType) == CT_CONNECTX4_LX) || \
       ((chipType) == CT_CONNECTX5) || ((chipType) == CT_BLUEFIELD) || ((chipType) == CT_CONNECTX6) || \
-        ((chipType) == CT_CONNECTX6DX) || ((chipType) == CT_CONNECTX6LX) || ((chipType) == CT_BLUEFIELD2))
+        ((chipType) == CT_CONNECTX6DX) || ((chipType) == CT_CONNECTX6LX) || ((chipType) == CT_CONNECTX7) || ((chipType) == CT_BLUEFIELD2))
+
+
 typedef enum chip_family_type {
     CFT_UNKNOWN = 0,
     CFT_HCA,
     CFT_SWITCH,
+    CFT_GEARBOX
 } chip_family_type_t;
 
 typedef struct guid {
@@ -429,11 +469,11 @@ typedef enum fw_hndl_type {
 } fw_hndl_type_t;
 
 typedef enum fw_img_type {
-    FIT_FS2,
-    FIT_FS3,
-    FIT_FC1,
-    FIT_FS4,
-    FIT_FSCTRL,
+    FIT_FS2 = 0,
+    FIT_FS3 = 1,
+    FIT_FC1 = 2,
+    FIT_FS4 = 3,
+    FIT_FSCTRL = 4,
 } fw_img_type_t;
 
 enum ExpRomProto {
