@@ -42,7 +42,10 @@
 #include <iostream>
 #include <mfa.h>
 #include <mlxfwops.h>
-
+#include <pldm_pkg.h>
+#include <pldm_dev_id_record.h>
+#include <pldm_component_image.h>
+#include <tools_layouts/image_info_layouts.h>
 
 using namespace std;
 
@@ -52,6 +55,7 @@ ImageAccess::ImageAccess(int compareFFV)
     _imgFwOps   = NULL;
     _compareFFV = compareFFV;
     memset(&_imgFwParams, 0, sizeof(_imgFwParams));
+    memset(_errBuff, 0, sizeof(_errBuff));
 }
 
 ImageAccess::~ImageAccess()
@@ -63,8 +67,9 @@ ImageAccess::~ImageAccess()
     }
 }
 
-bool ImageAccess::hasMFAs(string directory) {
-    DIR* d;
+bool ImageAccess::hasMFAs(string directory)
+{
+    DIR *d;
     struct dirent *dir;
     int rc;
     d = opendir(directory.c_str());
@@ -92,7 +97,7 @@ bool ImageAccess::hasMFAs(string directory) {
 int ImageAccess::queryDirPsid(string &path, string &psid, string &selector_tag, int image_type, vector<PsidQueryItem> &riv)
 {
     int res = 0;
-    DIR* d;
+    DIR *d;
     struct dirent *dir;
     int found = 0;
     int rc;
@@ -134,7 +139,8 @@ clean_up:
     return res;
 }
 
-bool ImageAccess::openImg(fw_hndl_type_t hndlType, char *psid, char * fileHndl) {
+bool ImageAccess::openImg(fw_hndl_type_t hndlType, char *psid, char *fileHndl)
+{
 
     memset(_errBuff, 0, sizeof(_errBuff));
     _imgFwParams.errBuff      = _errBuff;
@@ -163,13 +169,13 @@ int ImageAccess::queryPsid(const string &fname, const string &psid,
                            PsidQueryItem &ri)
 {
     int res = 0;
-    mfa_desc* mfa_d = NULL;
+    mfa_desc *mfa_d = NULL;
     fw_info_t img_query;
-    string    iniName = "";
+    string iniName = "";
     vector<u_int8_t> sect; // to get fw configuration.
     vector<u_int8_t> dest;
 
-    if(!openImg(FHT_FW_FILE, (char*)psid.c_str(), (char*)fname.c_str())) {
+    if (!openImg(FHT_FW_FILE, (char*)psid.c_str(), (char*)fname.c_str())) {
         return 0;
     }
 
@@ -177,7 +183,7 @@ int ImageAccess::queryPsid(const string &fname, const string &psid,
     ri.found = 0;
 
     memset(&img_query, 0, sizeof(img_query));
-    if(!_imgFwOps->FwQuery(&img_query, true)) {
+    if (!_imgFwOps->FwQuery(&img_query, true)) {
         _errMsg  = "Failed to query " +  (string)_imgFwOps->err();
         _log += _errMsg;
         goto clean_up;
@@ -196,26 +202,26 @@ int ImageAccess::queryPsid(const string &fname, const string &psid,
         ri.isFailSafe = img_query.fw_info.is_failsafe;
         if (_imgFwOps->FwGetSection(H_FW_CONF, sect)) {
             if (unzipDataFile(sect, dest, "Fw Configuration")) {
-                 char * ptr = strstr ((char *)&dest[0],"Name =");
-                 int counter = 7;
-                 while(ptr and *ptr != '\n' and *ptr != '\r') {
-                     if (counter-- > 0) {
-                         ptr++;
-                         continue;
-                     }
-                     iniName += *ptr;
-                     ptr++;
-                 }
-             }
+                char *ptr = strstr((char*)&dest[0], "Name =");
+                int counter = 7;
+                while (ptr && * ptr != '\n' && * ptr != '\r') {
+                    if (counter-- > 0) {
+                        ptr++;
+                        continue;
+                    }
+                    iniName += *ptr;
+                    ptr++;
+                }
+            }
         }
         ri.iniName = iniName;
         if (!mfa_open_file(&mfa_d, (char*)fname.c_str())) {
-            char* pn =  mfa_get_board_metadata(mfa_d, (char*)psid.c_str(), (char*)"PN");
+            char *pn =  mfa_get_board_metadata(mfa_d, (char*)psid.c_str(), (char*)"PN");
             if (pn != NULL) {
                 ri.pns = pn;
                 free(pn);
             }
-            char* desc =  mfa_get_board_metadata(mfa_d, (char*)psid.c_str(), (char*)"DESCRIPTION");//TODO: Make more efficient
+            char *desc =  mfa_get_board_metadata(mfa_d, (char*)psid.c_str(), (char*)"DESCRIPTION");//TODO: Make more efficient
             if (desc != NULL) {
                 ri.description = desc;
                 free(desc);
@@ -224,7 +230,8 @@ int ImageAccess::queryPsid(const string &fname, const string &psid,
         }
         if (!_compareFFV) {
             ImgVersion imgv;
-            imgv.setVersion("FW", 3, img_query.fw_info.fw_ver);
+            imgv.setVersion("FW", 3, img_query.fw_info.fw_ver,
+                    img_query.fw_info.branch_ver);
             ri.imgVers.push_back(imgv);
         } else {
             u_int16_t fwVer[4];
@@ -238,15 +245,12 @@ int ImageAccess::queryPsid(const string &fname, const string &psid,
         }
         for (int i = 0; i < img_query.fw_info.roms_info.num_of_exp_rom; i++) {
             ImgVersion imgVer;
-            const char* tpc = _imgFwOps->expRomType2Str(img_query.fw_info.roms_info.rom_info[i].exp_rom_product_id);
+            const char *tpc = _imgFwOps->expRomType2Str(img_query.fw_info.roms_info.rom_info[i].exp_rom_product_id);
             if (tpc == NULL) {
-	        //imgVer.setExpansionRomtoUnknown();
+                //imgVer.setExpansionRomtoUnknown();
                 tpc = "UNKNOWN_ROM";
             }
             int sz = img_query.fw_info.roms_info.rom_info[i].exp_rom_num_ver_fields;
-            if (img_query.fw_info.roms_info.rom_info[i].exp_rom_product_id == 0xf) {
-                sz = 1;
-            }
             imgVer.setVersion(tpc, sz, img_query.fw_info.roms_info.rom_info[i].exp_rom_ver);
             ri.imgVers.push_back(imgVer);
         }
@@ -270,7 +274,7 @@ int ImageAccess::getImage(const string &fname, const string &psid, string &selec
     int type = getFileSignature(fname);
 
     if (type == IMG_SIG_TYPE_MFA) {
-        mfa_desc* mfa_d;
+        mfa_desc *mfa_d;
         int rc = mfa_open_file(&mfa_d, (char*)fname.c_str());
         if (!rc) {
             res = mfa_get_image(mfa_d, (char*)psid.c_str(), image_type, (char*)selector_tag.c_str(), filebuf);
@@ -278,7 +282,7 @@ int ImageAccess::getImage(const string &fname, const string &psid, string &selec
         } else {
             res = -1;
         }
-    } else if (type == IMG_SIG_TYPE_BIN){
+    } else if (type == IMG_SIG_TYPE_BIN) {
         res = getImage(fname, filebuf);
     }
     //else type is unknown and res = -1
@@ -290,8 +294,8 @@ int ImageAccess::getImage(const string &fname, const string &psid, string &selec
 int ImageAccess::getImage(const string &fname, u_int8_t **filebuf)
 {
     int res = -1;
-    FILE* fp;
-    u_int8_t* buf = NULL;
+    FILE *fp;
+    u_int8_t *buf = NULL;
     long int fsize;
     long int sz;
 
@@ -308,7 +312,7 @@ int ImageAccess::getImage(const string &fname, u_int8_t **filebuf)
     }
 
     fsize = ftell(fp);
-    if(fsize <= 0) {
+    if (fsize <= 0) {
         _errMsg = "Ftell failed on file : " + (string)fname.c_str();
         _log    += _errMsg;
         goto clean_up;
@@ -344,50 +348,48 @@ clean_up:
 }
 
 
-int ImageAccess::getFileSignature(string fname)
+int ImageAccess::getFileSignature(const string & fname)
 {
-    FILE* fin;
-    char tmpb[16];
+    static const int header_size = 16;
+    FILE *fin;
+    char tmpb[header_size];
     int res = -2;
+    do {
+        if (!(fin = fopen(fname.c_str(), "r"))) {
+            break;
+        }
 
-    if (!(fin = fopen(fname.c_str(), "r"))) {
-        return res;
+        if (1 > fread(tmpb, header_size, 1, fin)) {
+            break;
+        }
+        res = getBufferSignature((u_int8_t*)tmpb, 4);
+    }while(0);
+
+    if(fin) {
+        fclose(fin);
     }
-    if (!fgets(tmpb, sizeof(tmpb), fin)) {
-        goto clean_up;
-    }
-
-    tmpb[15] = 0;
-    if (strlen(tmpb) < 4) {
-        goto clean_up;
-    }
-
-    res = getBufferSignature((u_int8_t*)tmpb, 4);
-
-clean_up:
-    fclose(fin);
     return res;
 }
 
 
-int ImageAccess::getBufferSignature(u_int8_t* buf, u_int32_t size)
+int ImageAccess::getBufferSignature(u_int8_t *buf, u_int32_t size)
 {
-    int res = 0;
+    int res = IMG_SIG_TYPE_UNKNOWN;
 
-    if (size < 4) {
-        return 0;
-    }
-    if (!strncmp((char*)buf, "MTFW", 4)) {
+    if (0 == memcmp(buf, "MTFW", 4)) {
         res = IMG_SIG_TYPE_BIN;
     }
-    if (!strncmp((char*)buf, "MFAR", 4)) {
+    else if (0 == memcmp(buf, "MFAR", 4)) {
         res = IMG_SIG_TYPE_MFA;
     }
+    else if(0 == memcmp(buf, PldmPkg::UUID, size)) {
+        res = IMG_SIG_TYPE_PLDM;
+    }
 
     return res;
 }
 
-int ImageAccess::get_bin_content(string fname, vector<PsidQueryItem> &riv)
+int ImageAccess::get_bin_content(const string & fname, vector<PsidQueryItem> &riv)
 {
     fw_info_t img_query;
     int res = 0;
@@ -395,17 +397,17 @@ int ImageAccess::get_bin_content(string fname, vector<PsidQueryItem> &riv)
     PsidQueryItem item;
     ImgVersion imgv;
 
-    if(_imgFwOps != NULL) {
+    if (_imgFwOps != NULL) {
         _errMsg  = "_imgFwOPs should be null";
         _log += _errMsg;
         return -1;
     }
-    if(!openImg(FHT_FW_FILE, NULL, (char*)fname.c_str())) {
+    if (!openImg(FHT_FW_FILE, NULL, (char*)fname.c_str())) {
         return -1;
     }
 
     memset(&img_query, 0, sizeof(img_query));
-    if(!_imgFwOps->FwQuery(&img_query, true)) {
+    if (!_imgFwOps->FwQuery(&img_query, true)) {
         _errMsg  = "Failed to query " +  (string)_imgFwOps->err();
         _log += _errMsg;
         res = -1;
@@ -418,13 +420,14 @@ int ImageAccess::get_bin_content(string fname, vector<PsidQueryItem> &riv)
     item.pns = "";
     item.board_rev = "";
 
-    imgv.setVersion("FW", 3, img_query.fw_info.fw_ver);
+    imgv.setVersion("FW", 3, img_query.fw_info.fw_ver,
+            img_query.fw_info.branch_ver);
     item.imgVers.push_back(imgv);
     for (int i = 0; i < img_query.fw_info.roms_info.num_of_exp_rom; i++) {
         ImgVersion imgVer;
-        const char* tpc = _imgFwOps->expRomType2Str(img_query.fw_info.roms_info.rom_info[i].exp_rom_product_id);
+        const char *tpc = _imgFwOps->expRomType2Str(img_query.fw_info.roms_info.rom_info[i].exp_rom_product_id);
         if (tpc == NULL) {
-	    //imgVer.setExpansionRomtoUnknown();
+            //imgVer.setExpansionRomtoUnknown();
             tpc = "UNKNOWN_ROM";
         }
         int sz = img_query.fw_info.roms_info.rom_info[i].exp_rom_num_ver_fields;
@@ -445,22 +448,22 @@ clean_up:
 }
 
 
-int ImageAccess::get_mfa_content(string fname, vector<PsidQueryItem> &riv)
+int ImageAccess::get_mfa_content(const string & fname, vector<PsidQueryItem> &riv)
 {
-    mfa_desc* mfa_d;
-    map_entry_hdr* me = NULL;
+    mfa_desc *mfa_d;
+    map_entry_hdr *me = NULL;
     char psid[33];
 
     if (mfa_open_file(&mfa_d, (char*)fname.c_str())) {
         return -1;
     }
     me = NULL;
-    while((me = mfa_get_next_mentry(mfa_d, me))) {
+    while ((me = mfa_get_next_mentry(mfa_d, me))) {
         PsidQueryItem item;
         strncpy(psid, me->board_type_id, 32);
         psid[32] = 0;
         item.psid = psid;
-        char* val = mfa_get_map_entry_metadata(me, (char*)"PN");
+        char *val = mfa_get_map_entry_metadata(me, (char*)"PN");
         if (val == NULL) {
             item.pns = "";
         } else {
@@ -481,14 +484,21 @@ int ImageAccess::get_mfa_content(string fname, vector<PsidQueryItem> &riv)
             item.description = val;
             free(val);
         }
+        val = mfa_get_map_entry_metadata(me, (char*) "BRANCH");
+        if (val == NULL) {
+            item.branch = "";
+        } else {
+            item.branch = val;
+            free(val);
+        }
 
-        map_image_entry* imge;
+        map_image_entry *imge;
         for (int i = 0; i < me->nimages; i++) {
             imge = mfa_get_map_image(me, i);
             if (imge == NULL) {
                 continue;
             }
-            toc_entry* toc = mfa_get_image_toc(mfa_d, imge);
+            toc_entry *toc = mfa_get_image_toc(mfa_d, imge);
             if (toc == NULL) {
                 continue;
             }
@@ -498,11 +508,12 @@ int ImageAccess::get_mfa_content(string fname, vector<PsidQueryItem> &riv)
                 strncpy(select_tag, imge->select_tag, 32);
                 select_tag[32] = 0;
                 item.selector_tag = select_tag;
-                char* subImgType;
+                char *subImgType;
                 if ((subImgType = mfasec_get_sub_image_type_str(toc->subimage_type)) != NULL) {
                     ImgVersion imgv;
                     if (subImgType[0] != '\0') { //Not a padding section
-                        imgv.setVersion(subImgType, toc->num_ver_fields, toc->version);
+                        imgv.setVersion(subImgType, toc->num_ver_fields,
+                                toc->version, item.branch);
                         item.imgVers.push_back(imgv);
                     }
                 }
@@ -513,19 +524,112 @@ int ImageAccess::get_mfa_content(string fname, vector<PsidQueryItem> &riv)
     mfa_close(mfa_d);
     return 0;
 }
+#define ITOC_ASCII 0x49544f43
+#define TOC_HEADER_SIZE 0x20
+#define TOC_ENTRY_SIZE  0x20
+#define FS3_DEFAULT_SECTOR_SIZE 0x1000
+
+bool ImageAccess::extract_pldm_image_info(const u_int8_t * buff, u_int32_t size, PsidQueryItem &query_item) {
+    FImage fimage;
+    if (!fimage.open((u_int32_t*)buff, size)) {
+        return false;
+    }
+    static const u_int32_t sector_size = FS3_DEFAULT_SECTOR_SIZE;
+    u_int32_t offset = 0;
+    offset = (offset % sector_size == 0) ? offset : (offset + sector_size - offset % 0x1000);
+    u_int8_t buffer[TOC_HEADER_SIZE], entry_buffer[TOC_ENTRY_SIZE];
+    struct connectx4_itoc_header itoc_header;
+    while (offset < fimage.get_size()) {
+        //read ITOC header
+        fimage.read(offset, buffer, TOC_HEADER_SIZE);
+        connectx4_itoc_header_unpack(&itoc_header, buffer);
+        if (itoc_header.signature0 != ITOC_ASCII) {
+            offset += sector_size;
+            continue;
+        }
+        int section_index = 0;
+        struct connectx4_itoc_entry toc_entry;
+        do {
+            //read ITOC entry
+            u_int32_t entry_addr = offset + TOC_HEADER_SIZE + section_index *  TOC_ENTRY_SIZE;
+            fimage.read(entry_addr, entry_buffer, TOC_ENTRY_SIZE);
+
+            connectx4_itoc_entry_unpack(&toc_entry, entry_buffer);
+            if (toc_entry.type == FS3_IMAGE_INFO) {
+                u_int32_t flash_addr = toc_entry.flash_addr << 2;
+                u_int32_t entry_size_in_bytes = toc_entry.size * 4;
+                u_int8_t *buff = new u_int8_t[entry_size_in_bytes];
+                fimage.read(flash_addr, buff, entry_size_in_bytes);
+                //read image info
+                struct connectx4_image_info image_info;
+                connectx4_image_info_unpack(&image_info, buff);
+                query_item.name = image_info.name;
+                query_item.pns = query_item.name;
+                query_item.description = image_info.description;
+                static const int FW_VER_SIZE = 3;
+                u_int16_t fw_ver[FW_VER_SIZE];
+                char fw_branch[BRANCH_LEN + 1] = { 0 };
+                fw_ver[0] = image_info.FW_VERSION.MAJOR;
+                fw_ver[1] = image_info.FW_VERSION.MINOR;
+                fw_ver[2] = image_info.FW_VERSION.SUBMINOR;
+                strncpy(fw_branch, image_info.vsd, BRANCH_LEN);
+                fw_branch[BRANCH_LEN] = (char) 0;
+                ImgVersion imgv;
+                imgv.setVersion("FW", FW_VER_SIZE, fw_ver, fw_branch);
+                query_item.imgVers.push_back(imgv);
+
+                delete [] buff;
+                break;
+            }
+
+            section_index++;
+        } while (toc_entry.type != FS3_END);
+        break;
+    }
+    fimage.close();
+    return 0;
+}
+
+int ImageAccess::get_pldm_content(const string & fname,
+        vector<PsidQueryItem> &riv) {
+    PldmBuffer pldm_buff;
+    pldm_buff.loadFile(fname);
+    PldmPkg pldm;
+    pldm.unpack(pldm_buff);
+
+    u_int8_t dev_count = pldm.getDeviceIDRecordCount();
+    for(u_int8_t i=0; i<dev_count; i++) {
+        PldmDevIdRecord * rec = pldm.getDeviceIDRecord(i);
+        PsidQueryItem item;
+        item.psid = rec->getDevicePsid();
+        item.description = rec->getDescription();
+        int image_index = rec->getComponentImageIndex();
+        PldmComponenetImage * image_obj = pldm.getComponentImage(image_index);
+        extract_pldm_image_info(image_obj->getComponentData(),
+                image_obj->getComponentSize(), item);
+        riv.push_back(item);
+    }
+
+    return 0;
+}
 
 
-int ImageAccess::get_file_content(string fname, vector<PsidQueryItem> &riv)
+int ImageAccess::get_file_content(const string & fname, vector<PsidQueryItem> &riv)
 {
     int type = getFileSignature(fname);
-
-    if (type == IMG_SIG_TYPE_BIN) {
-        return get_bin_content(fname, riv);
-    } else if (type == IMG_SIG_TYPE_MFA) {
-        return get_mfa_content(fname, riv);
-    } else  {
-        return -1;
+    int res = -1;
+    switch(type) {
+    case IMG_SIG_TYPE_BIN:
+        res = get_bin_content(fname, riv);
+        break;
+    case IMG_SIG_TYPE_MFA:
+        res = get_mfa_content(fname, riv);
+        break;
+    case IMG_SIG_TYPE_PLDM:
+        res = get_pldm_content(fname, riv);
+        break;
     }
+    return res;
 }
 
 string ImageAccess::getLastErrMsg()
@@ -533,7 +637,8 @@ string ImageAccess::getLastErrMsg()
     return _errMsg;
 }
 
-string ImageAccess::getlastWarning() {
+string ImageAccess::getlastWarning()
+{
     return _warning;
 }
 
