@@ -34,6 +34,14 @@
 
 
 #include "mlxfwmanager.h"
+#ifdef _MSC_VER
+#include <direct.h>
+#include <dirent.h>
+#endif
+#include <string>
+#include <iostream>
+#include <algorithm>
+#include <set>
 
 int main(int argc, char *argv[])
 {
@@ -49,6 +57,7 @@ int mainEntry(int argc, char *argv[])
     int res = 0;
     config_t config;
     bool early_cleanup = true;
+    bool os_valid;
     int rc, rc0;
     vector<string>  dev_names;
     vector<MlnxDev*> devs;
@@ -105,7 +114,6 @@ int mainEntry(int argc, char *argv[])
         res = ERR_CODE_BAD_CMD_ARGS;
         return res;
     }
-
     if (cmd_params.calc_crc) {
         if (config.mfa_path == ".") {
             fprintf(stderr, "-E- Bad command line arguments, image file is mandatory\n");
@@ -160,6 +168,20 @@ int mainEntry(int argc, char *argv[])
             goto early_err_clean_up;
         }
     }
+    if (cmd_params.download_os.length()) {
+        IS_OKAY_To_INTERRUPT = true;
+        res = check_valid_os_type(cmd_params, &os_valid);
+        if (res) {
+            goto early_err_clean_up;
+        }
+        if (!os_valid) {
+            fprintf(stderr, "-E- Invalid --download-os argument \"%s\"\n",
+                    cmd_params.download_os.c_str());
+            res = ERR_CODE_INVALID_CHOICE;
+            goto early_err_clean_up;
+        }
+        IS_OKAY_To_INTERRUPT = false;
+    }
     if (cmd_params.onlineQueryPsids.length()) {
         if (cmd_params.queryFormat == "text") {
             res = handleOnlinePsidsQuery(srq, cmd_params, config);
@@ -187,9 +209,7 @@ int mainEntry(int argc, char *argv[])
 
     //Listing of binary files content
     if (cmd_params.list_file_contents) {
-        if (list_files_content(config)) {
-            res = ERR_CODE_FILE_PARSE_FAILED;
-        }
+        res = list_files_content(config);
         goto clean_up;
     }
 
@@ -1029,10 +1049,10 @@ int checkAndDisplayDeviceQuery1D(vector<MlnxDev*> &devs,
 
                 //u_int16_t fwVer[3] = {3,4,141};
                 //psidUpdateInfo[devs[i]->getPsid()].imgVers[1].setVersion("PXE", 3, fwVer);
-                if (devImgVer->compare(*fileImgVer) < 0) {
-                    nKeepReasons++;
-                } else if (devImgVer->compare(*fileImgVer) > 0) {
+                if (devImgVer->compareFw(*fileImgVer) < 0) {
                     nUpdateReasons++;
+                } else if (devImgVer->compareFw(*fileImgVer) > 0) {
+                    nKeepReasons++;
                 }
             }
         }
@@ -1130,7 +1150,7 @@ int checkAndDisplayDeviceQuery1D(vector<MlnxDev*> &devs,
             print_err_str += tmpstr;
             res = ERR_CODE_MIXED_VERSIONS_FOUND;
         }
-        if (multiple_img_cnt or errorMsg.length()) {
+        if (multiple_img_cnt || errorMsg.length()) {
             print_err_str += errorMsg;
             res = ERR_CODE_MULTI_IMG_SRC_FOUND;
         }
@@ -1222,7 +1242,9 @@ void getUniqueMFAList(vector<MlnxDev*> &devs, map<string, PsidQueryItem> &psidUp
         if (!psidUpdateInfo[devs[i]->getPsid()].found) {
             continue;
         }
-        if (force_update || (devs[i]->compareFWVer(psidUpdateInfo[devs[i]->getPsid()].imgVers[0].getVerArray()) > 0)) {
+        if (force_update
+                || (devs[i]->compareFWVer(
+                        psidUpdateInfo[devs[i]->getPsid()].imgVers[0]) < 0)) {
             if (mfa2download.count(psidUpdateInfo[devs[i]->getPsid()].url) == 0) {
                 mfa_list.push_back(psidUpdateInfo[devs[i]->getPsid()].url);
                 mfa_base_name_list.push_back(psidUpdateInfo[devs[i]->getPsid()].name);
@@ -1546,70 +1568,75 @@ int list_files_content(config_t &config)
     DIR *d;
     struct dirent *dir;
     int rc;
-    int res = 0;
+    int res = MLX_FWM_SUCCESS;
     ImageAccess imgacc(CompareFFV);
-    vector<PsidQueryItem> items;
     bool show_titles = true;
     int displayed_files_cnt = 0;
 
     if (config.path_is_file) {
+        vector<PsidQueryItem> items;
         string fpath = config.mfa_path;
         //TODO: list content by image type
         rc = imgacc.get_file_content(fpath, items);
         if (rc) {
             print_err("-E- Error parsing file: %s\n", fpath.c_str());
+            res = ERR_CODE_FILE_PARSE_FAILED;
+        }
+        else {
+            display_file_listing(items, config.psid, show_titles);
+        }
+    }
+    else {
+        d = opendir(config.mfa_path.c_str());
+        if (d == NULL) {
             return -1;
         }
-        display_file_listing(items, config.psid, show_titles);
-        return 0;
-    }
 
-    d = opendir(config.mfa_path.c_str());
-    if (d == NULL) {
-        return -1;
-    }
-
-    while ((dir = readdir(d)) != NULL) {
-        string fl = dir->d_name;
-        if (fl == "." || fl == "..") {
-            continue;
-        }
-        string fpath = config.mfa_path;
-        if (fpath.length() > 0) {
-            #if defined(__WIN__)
-            if (fpath[fpath.length() - 1] != '\\') {
-                fpath += "\\";
+        while ((dir = readdir(d)) != NULL) {
+            string fl = dir->d_name;
+            if (fl == "." || fl == "..") {
+                continue;
             }
-            #else
-            if (fpath[fpath.length() - 1] != '/') {
-                fpath += "/";
+            string fpath = config.mfa_path;
+            if (fpath.length() > 0) {
+                #if defined(__WIN__)
+                if (fpath[fpath.length() - 1] != '\\') {
+                    fpath += "\\";
+                }
+                #else
+                if (fpath[fpath.length() - 1] != '/') {
+                    fpath += "/";
+                }
+                #endif
             }
-            #endif
-        }
-        fpath += fl;
+            fpath += fl;
 
-        if (imgacc.getFileSignature(fpath) <= 0) {
-            continue;
+            if (imgacc.getFileSignature(fpath) <= 0) {
+                continue;
+            }
+            vector<PsidQueryItem> items;
+            rc = imgacc.get_file_content(fpath, items);
+            if (rc) {
+                print_err("-E- Error parsing file: %s\n", fpath.c_str());
+                res = ERR_CODE_FILE_PARSE_FAILED;
+                continue;
+            }
+            if (config.display_file_names) {
+                printf("Supported Boards in File: %s\n", fpath.c_str());
+                show_titles = true;
+            }
+            display_file_listing(items, config.psid, show_titles);
+            items.clear();
+            show_titles = false;
+            displayed_files_cnt++;
         }
-        items.clear();
-        rc = imgacc.get_file_content(fpath, items);
-        if (rc) {
-            print_err("-E- Error parsing file: %s\n", fpath.c_str());
-            res = -1;
-            continue;
+        closedir(d);
+        if (displayed_files_cnt == 0) {
+            print_out("No image files found\n");
+            if (res != ERR_CODE_FILE_PARSE_FAILED) {
+                res = ERR_CODE_IMG_NOT_FOUND;
+            }
         }
-        if (config.display_file_names) {
-            printf("Supported Boards in File: %s\n", fpath.c_str());
-            show_titles = true;
-        }
-        display_file_listing(items, config.psid, show_titles);
-        show_titles = false;
-        displayed_files_cnt++;
-    }
-
-    closedir(d);
-    if (displayed_files_cnt == 0) {
-        print_out("No image files found\n");
     }
 
     return res;
@@ -1647,7 +1674,7 @@ void display_file_listing(vector<PsidQueryItem> &items, string psid, bool show_t
         print_out(" ");
         display_field_str("PSID", 18);
         print_out(" ");
-        display_field_str("Version", 16);
+        display_field_str("Version", 28);
         print_out(" ");
         display_field_str("Tag", 14);
         print_out(" ");
@@ -1674,7 +1701,7 @@ void display_file_listing(vector<PsidQueryItem> &items, string psid, bool show_t
         display_field_str(items[i].psid, 18);
         print_out(" ");
         if ((imv = (ImgVersion*)items[i].findImageVersion("FW"))) {
-            display_field_str(imv->getPrintableVersion(0), 16);
+            display_field_str(imv->getPrintableVersion(0), 28);
         } else {
             display_field_str("", 16);
         }
@@ -2003,10 +2030,9 @@ int handleOnlinePsidsQueryXml(ServerRequest *srq, CmdLineParams &cmd_params, con
         return res;
     }
 
-    PsidQueryItem ri;
     string output = "<Psids>\n";
     for (unsigned int i = 0; i < psidUpdateInfo.size(); i++) {
-        ri = psidUpdateInfo[i];
+        PsidQueryItem ri = psidUpdateInfo[i];
         string status;
         string ver = "N/A";
         string mfa = "N/A";
@@ -2069,9 +2095,8 @@ int handleOnlinePsidsQuery(ServerRequest *srq, CmdLineParams &cmd_params, config
         return res;
     }
 
-    PsidQueryItem ri;
     for (unsigned int i = 0; i < psidUpdateInfo.size(); i++) {
-        ri = psidUpdateInfo[i];
+        PsidQueryItem ri = psidUpdateInfo[i];
         string status;
         string ver = "N/A";
         string mfa = "N/A";
@@ -2133,6 +2158,38 @@ static int getFileProperties(ServerRequest *srq, vector<DownloadedFileProperties
     return MLX_FWM_SUCCESS;
 }
 
+int check_valid_os_type(CmdLineParams &cmd_params, bool* os_valid) {
+    vector <DownloadedFileProperties> filePropVec;
+    int res = MLX_FWM_SUCCESS;
+    *os_valid = false;
+    if (cmd_params.download_os == "all") {
+        *os_valid = true;
+        return res;
+    }
+    ServerRequest *srq = new ServerRequest(cmd_params.server_url.c_str(),
+                                cmd_params.proxy.c_str(), cmd_params.compare_ffv,
+                                cmd_params.show_progress, cmd_params.download_key,
+                                cmd_params.certificate, cmd_params.numberOfRetrials);
+    if (srq == NULL) {
+        return ERR_CODE_MEM_ALLOC_FAIL;
+    }
+    res = getFileProperties(srq, filePropVec, "", "", "self_extractor");
+    if (res) {
+        print_err("-E- Failed to query files on server\n");
+        string errorMsg;
+        srq->getError(res, errorMsg);
+        print_err("%s\n", errorMsg.c_str());
+    }
+    for (std::vector<DownloadedFileProperties>::iterator it = filePropVec.begin(); it != filePropVec.end(); it++) {
+        if (cmd_params.download_os == it->os) {
+            *os_valid = true;
+            break;
+        }
+    }
+    delete srq;
+    return res;
+}
+
 int handleGetDownloadOptionsRequest(ServerRequest *srq, CmdLineParams &cmd_params)
 {
     std::set<string> availOpt;
@@ -2204,7 +2261,7 @@ int handleDownloadRequest(ServerRequest *srq, CmdLineParams &cmd_params, config_
                     sleep(1);
                     invalid = true;
                 }
-                #if !defined(__FreeBSD__)
+                #if !defined(__FreeBSD__) && !(defined(_MSC_VER) && defined(_ARM64_))
                 if (iss.fail()) {
                     sleep(1);
                     invalid = true;
