@@ -30,7 +30,6 @@
  * SOFTWARE.
  *
  */
-#include <boost/assign.hpp>
 #include <map>
 #include <iostream>
 #include <vector>
@@ -43,13 +42,7 @@
 #include <signal.h>
 #include <errno.h>
 #include <string.h>
-#ifdef HAVE_INI_PARSER
-extern "C" {
 #include <iniparser.h>
-}
-#else
-    #include <iniparser.h>
-#endif
 #include <mft_sig_handler.h>
 #include <fcntl.h>
 #include <dirent.h>
@@ -152,6 +145,8 @@ void  fixXmlString(string & stringToFix);
 
 bool isServerRequestorRequired(CmdLineParams& cmd_params);
 
+int check_valid_os_type(CmdLineParams &cmd_params, bool* os_valid);
+
 int handleGetDownloadOptionsRequest(ServerRequest *srq, CmdLineParams &cmd_params);
 
 int   handleDownloadRequest(ServerRequest *srq,
