@@ -45,6 +45,9 @@ class PsidQueryItem {
 public:
 
     PsidQueryItem() : devId(0), revId(0), found(0), isFailSafe(true) {};
+    virtual ~PsidQueryItem()
+    {
+    }
     const ImgVersion* findImageVersion(string type);
     string psid;
     string url;
@@ -61,6 +64,7 @@ public:
     string type;
     vector<ImgVersion> imgVers;
     bool isFailSafe;
+    string branch;
 };
 
 #endif
