@@ -44,8 +44,8 @@
 #include <cmdif/icmd_cif_open.h>
 #endif
 #include <cmdparser/cmdparser.h>
-#include <mtcr.h>
-#include "mlxlink_utils.h"
+#include "mlxlink_cables_commander.h"
+#include "mlxlink_eye_opener.h"
 
 #ifdef MST_UL
 #define MLXLINK_EXEC "mstlink"
@@ -74,7 +74,6 @@
 #define NODE_FLAG_SHORT              ' '
 #define PRINT_JSON_OUTPUT_FLAG       "json"
 #define PRINT_JSON_OUTPUT_FLAG_SHORT ' '
-
 //------------------------------------------------------------
 //        Mlxlink QUERIES Flags
 
@@ -128,6 +127,12 @@
 #define PPRT_RATE_FLAG_SHORT               ' '
 #define PPTT_RATE_FLAG                     "tx_rate"
 #define PPTT_RATE_FLAG_SHORT               ' '
+#define PRBS_LANES_FLAG                    "lanes"
+#define PRBS_LANES_FLAG_SHORT              ' '
+#define PRBS_INVERT_TX_POL_FLAG            "invert_tx_polarity"
+#define PRBS_INVERT_TX_POL_FLAG_SHORT      ' '
+#define PRBS_INVERT_RX_POL_FLAG            "invert_rx_polarity"
+#define PRBS_INVERT_RX_POL_FLAG_SHORT      ' '
 #define PPRT_TUNING_TYPE_FLAG              "tuning_type"
 #define PPRT_TUNING_TYPE_FLAG_SHORT        ' '
 #define BER_COLLECT_FLAG                   "ber_collect"
@@ -147,6 +152,42 @@
 #define PTYS_LINK_MODE_FORCE_FLAG          "link_mode_force"
 #define PTYS_LINK_MODE_FORCE_FLAG_SHORT    ' '
 
+//------------------------------------------------------------
+//        Mlxlink Cable info flags
+#define CABLE_FLAG                          "cable"
+#define CABLE_FLAG_SHORT                    ' '
+#define CABLE_DUMP_FLAG                     "dump"
+#define CABLE_DUMP_FLAG_SHORT               ' '
+#define CABLE_DDM_FLAG                      "ddm"
+#define CABLE_DDM_FLAG_SHORT                ' '
+#define CABLE_WRITE_FLAG                    "write"
+#define CABLE_WRITE_FLAG_SHORT              ' '
+#define CABLE_READ_FLAG                     "read"
+#define CABLE_READ_FLAG_SHORT               ' '
+#define WRITE_PAGE_FLAG                     "page"
+#define WRITE_PAGE_FLAG_SHORT               ' '
+#define WRITE_OFFSET_FLAG                   "offset"
+#define WRITE_OFFSET_FLAG_SHORT             ' '
+#define READ_LEN_FLAG                       "length"
+#define READ_LEN_FLAG_SHORT                 ' '
+#define SHOW_TX_GROUP_MAP_FLAG              "show_tx_group_map"
+#define SHOW_TX_GROUP_MAP_FLAG_SHORT        ' '
+#define SET_TX_GROUP_MAP_FLAG               "tx_group_map"
+#define SET_TX_GROUP_MAP_FLAG_SHORT         ' '
+#define TX_GROUP_PORTS_FLAG                 "ports"
+#define TX_GROUP_PORTS_FLAG_SHORT           ' '
+//------------------------------------------------------------
+//        Mlxlink Eye Opener Flags
+#define MARGIN_SCAN_FLAG                     "margin"
+#define MARGIN_SCAN_FLAG_SHORT               ' '
+#define EYE_SEL_FLAG                        "eye_select"
+#define EYE_SEL_FLAG_SHORT                  ' '
+#define EYE_MEASURE_TIME_FLAG               "measure_time"
+#define EYE_MEASURE_TIME_FLAG_SHORT         ' '
+#define LANE_INDEX_FLAG                     "lane"
+#define LANE_INDEX_FLAG_SHORT               ' '
+#define FORCE_YES_FLAG                      "yes"
+#define FORCE_YES_FLAG_SHORT                ' '
 //------------------------------------------------------------
 //        Mlxlink HIDDEN Flags
 
@@ -170,6 +211,7 @@
 
 #define PDDR_STATUS_MESSAGE_LENGTH_HCA  236
 #define PDDR_STATUS_MESSAGE_LENGTH_SWITCH 59
+#define BIT_MASK_ALL_DWORD          0xffffffff
 
 #define OB_TAP_SUM                  120
 #define OB_TAP_DIFF                 10
@@ -181,11 +223,22 @@
 #define MAX_LOCAL_PORT_ETH          64
 #define MAX_LOCAL_PORT_IB           36
 #define MAX_LOCAL_PORT_QUANTUM      82
+#define MAX_LOCAL_PORT_QUANTUM2     128
 #define MAX_LOCAL_PORT_SPECTRUM2    128
 
 #define DBN_TO_LOCAL_PORT_BASE      60
 
 #define MAX_LANES_NUMBER            4
+#define MAX_DWORD_BLOCK_SIZE        32
+#define MAX_TX_GROUP_COUNT          10
+
+#define PCAM_FORCE_DOWN_CAP_MASK    0x2000
+
+#define OPERATIONAL_ERROR_STR       "ME_ICMD_OPERATIONAL_ERROR"
+
+#define MAX_SBYTE                   127
+#define MIN_SBYTE                   -128
+#define MAX_LABEL_PORT_LENGTH       5
 
 //------------------------------------------------------------
 //        Mlxlink enumerations
@@ -202,6 +255,12 @@ enum SHOW_FUNCTIONS {
     SHOW_BER_MONITOR,
     SHOW_EXTERNAL_PHY,
     SHOW_LINK_DOWN_BLAME,
+    SHOW_TX_GROUP_MAP,
+    CABLE_EEPROM_INI,
+    CABLE_SHOW_DUMP,
+    CABLE_SHOW_DDM,
+    CABLE_EEPROM_WRITE,
+    CABLE_EEPROM_READ,
     SEND_BER_COLLECT,
     SEND_PAOS,
     SEND_PTYS,
@@ -212,14 +271,12 @@ enum SHOW_FUNCTIONS {
     SEND_CLEAR_COUNTERS,
     SEND_PEPC,
     SHOW_PCIE_LINKS,
-    // Any new function's index should be added before SHOW_SEND_LAST in this enum
+    SET_TX_GROUP_MAP,
+    GRADE_SCAN_ENABLE,
+    // Any new function's index should be added before FUNCTION_LAST in this enum
     FUNCTION_LAST
 };
 
-enum PNAT_ACCESS {
-    PNAT_LOCAL = 0, PNAT_LABEL = 1, PNAT_PCIE = 3
-};
-
 enum PDDR_PAGES {
     PDDR_OPERATIONAL_INFO_PAGE = 0,
     PDDR_TROUBLESHOOTING_INFO_PAGE = 1,
@@ -245,6 +302,12 @@ enum GROUP_OPCODE {
     MONITOR_OPCODE = 0, ADVANCED_OPCODE = 1
 };
 
+enum TX_INDEX_SELECTOR
+{
+    TX_LOCAL_PORT,
+    TX_GROUP
+};
+
 enum SLTP_PARAMS {
     POLARITY,
     OB_TAP0,
@@ -253,7 +316,8 @@ enum SLTP_PARAMS {
     OB_BIAS,
     OB_PREEMP_MODE,
     OB_REG,
-    OB_LEVA
+    OB_LEVA,
+    PARAMS_40NM_LAST // add the new enums before last
 };
 
 enum SLTP_16NM_PARAMS {
@@ -263,7 +327,8 @@ enum SLTP_16NM_PARAMS {
     POST_TAP,
     OB_M2LP,
     OB_AMP,
-    OB_ALEV_OUT
+    OB_ALEV_OUT,
+    PARAMS_16NM_LAST // add the new enums before last
 };
 
 enum BER_LIMIT {
@@ -285,18 +350,36 @@ enum PAOS_ADMIN {
     PAOS_UP = 1, PAOS_DOWN = 2
 };
 
-enum PRODUCT_TECHNOLOGY {
-    PRODUCT_40NM = 0,
-    PRODUCT_28NM = 1,
-    PRODUCT_16NM = 3
-};
-
 enum PCIE_PORT_TYPE {
     PORT_TYPE_EP = 0,
     PORT_TYPE_US = 5,
     PORT_TYPE_DS = 6
 };
 
+// Cable ops
+enum MODULE_ID {
+    MODULE_ID_SFP       = 0x3,
+    MODULE_ID_QSFP      = 0xC,
+    MODULE_ID_QSFP_PLUS = 0xD,
+    MODULE_ID_QSFP28    = 0x11,
+};
+
+enum STATUS_OPCODE {
+    CABLE_IS_UNPLUGGED = 1024
+};
+
+enum LOCAL_PORT_TYPE {
+    PORT_TYPE_NETWORK = 0
+};
+
+enum CABLE_MULTIPLIER {
+    CABLE_0_0_MUL,
+    CABLE_0_1_MUL,
+    CABLE_1_0_MUL,
+    CABLE_1_1_MUL
+};
+
+///////////
 struct DPN {
     DPN()
     {
@@ -315,33 +398,37 @@ struct DPN {
     u_int32_t node;
 };
 
-#define PRINT_LOG(mlxlinklogger, title)\
-    if (mlxlinklogger) {\
-        mlxlinklogger->printHeaderWithUnderLine(title);\
-    }\
+struct PortGroup {
+    u_int32_t localPort;
+    u_int32_t labelPort;
+    u_int32_t groupId;
+    u_int32_t split;
+    PortGroup() {
+        localPort = 0;
+        labelPort = 0;
+        groupId = 0;
+        split = 0;
+    }
+    PortGroup(u_int32_t ilocalPort, u_int32_t ilabelPort, u_int32_t igroupId, u_int32_t isplit) {
+        localPort = ilocalPort;
+        labelPort = ilabelPort;
+        groupId =igroupId;
+        split = isplit;
+    }
+};
+using namespace std;
 
-#define DEBUG_LOG(mlxlinklogger, format, ...)\
-    if (mlxlinklogger) {\
-        mlxlinklogger->debugLog(format, __VA_ARGS__);\
-    }\
-
-using namespace mlxreg;
-
-class MlxlinkCommander :public RegAccessParser{
+class MlxlinkCommander: public MlxlinkRegParser {
 
 public:
     MlxlinkCommander();
     virtual ~MlxlinkCommander();
 
-    void setParseMethod();
-    void resetParser(const string &regName);
-    void genBuffSendRegister(const string &regName, maccess_reg_method_t method);
-    void updateField(const string &field_name, u_int32_t value);
-    u_int32_t getFieldValue(const string &field_name, std::vector<u_int32_t>& buff);
-    string getFieldStr(const string &field);
     void checkRegCmd();
+    virtual void validatePortType();
+    virtual void updatePortType() {};
+    void checkLocalPortDPNMapping(u_int32_t localPort);
     int getLocalPortFromMPIR(DPN& dpn);
-
     void checkValidFW();
     int getProductTechnology();
     bool checkPortStatus(u_int32_t localPort);
@@ -355,6 +442,7 @@ public:
     void checkWidthSplit(u_int32_t localPort);
     void checkUnSplit(u_int32_t localPort);
     u_int32_t maxLocalPort();
+    void checkStrLength(const string &str);
     string getAscii(const string & name, u_int32_t size = 4);
     string getRxTxCDRState(u_int32_t state);
     void getActualNumOfLanes(u_int32_t linkSpeedActive, bool extended);
@@ -368,44 +456,60 @@ public:
     bool checkPpaosTestMode();
     u_int32_t getPtysCap();
     void getSltpParamsFromVector(std::vector<string> sltpParams);
+    void getprbsLanesFromParams(std::vector<string> prbsLanesParams);
     std::vector<string> parseParamsFromLine(const string & ParamsLine);
-    void setPrintTitle(MlxlinkCmdPrint &mlxlinkCmdPrint, string title,
-            u_int32_t size, bool print = true);
-    void setPrintVal(MlxlinkCmdPrint &mlxlinkCmdPrint, int index, string key,
-            string value, string color = ANSI_COLOR_RESET, bool print = true,
-            bool valid = true, bool arrayValue = false);
-    void writeGvmi(u_int32_t data);
+    bool isSpect2WithGb();
+    void fillIbPortGroupMap(u_int32_t localPort,u_int32_t labelPort, u_int32_t group,
+            bool splitReady);
+    void fillEthPortGroupMap(u_int32_t localPort,u_int32_t labelPort, u_int32_t group,
+            u_int32_t width, bool spect2WithGb);
+    int handleIBLocalPort(u_int32_t labelPort, bool ibSplitReady);
+    int handleEthLocalPort(u_int32_t labelPort, bool spect2WithGb);
+    void handleLabelPorts(std::vector<string> labelPortsStr);
+    vector<string> localToPortsPerGroup(vector<u_int32_t> localPorts);
+    u_int32_t getPortGroup(u_int32_t localPort);
 
     //Mlxlink query functions
-    void showModuleInfo();
+    virtual void showModuleInfo();
     virtual void operatingInfoPage();
     virtual void supportedInfoPage();
     virtual void troubInfoPage();
     virtual void showPddr();
     void getPtys();
     virtual void showBer();
-    void showEye();
-    void showFEC();
+    virtual void showEye();
+    virtual void showFEC();
     virtual void showSltp();
-    void showDeviceData();
+    virtual void showDeviceData();
     void showBerMonitorInfo();
     void showExternalPhy();
     void showPcie();
     void showPcieLinks();
     void collectBER();
+    void showTxGroupMapping();
 
     // Query helper functions
+    string getCableTechnologyStr(u_int32_t cableTechnology);
     string getCompliance(u_int32_t compliance, std::map<u_int32_t,
             std::string> complianceMap, bool bitMasked = false);
-    string getComplianceLabel(u_int32_t compliance,  u_int32_t extCompliance,
-            u_int32_t cable_identifier, bool ignoreExtBitChk = false);
+    string getCompliaceLabelForCIMIS(u_int32_t hostCompliance, u_int32_t mediaCompliance);
+    string getComplianceLabel(u_int32_t compliance,  u_int32_t extCompliance, bool ignoreExtBitChk = false);
+    string getPowerClass(u_int32_t powerClass, u_int32_t maxPower);
+    string getCableLengthStr(u_int32_t cableLength);
+    string getCableTypeStr(u_int32_t cableType);
+    void prepareStaticInfoSection(bool valid);
+    void prepareAttenuationAndFwSection(bool valid);
+    void preparePowerAndCdrSection(bool valid);
+    void prepareDDMSection(bool valid);
     string getLosAlarm();
+    void strToInt32(char *str, u_int32_t &value);
     template<typename T, typename Q> string getValueAndThresholdsStr(T value, Q lowTH, Q highTH);
     void prepareSltp28_40nm(std::vector<std::vector<string> > &sltpLanes,
             u_int32_t laneNumber);
     void prepareSltp16nm(std::vector<std::vector<string> > &sltpLanes,
             u_int32_t laneNumber);
     void initValidDPNList();
+    u_int32_t readBitFromField(const string &fieldName, u_int32_t bitIndex);
 
     void showTestMode();
     void showTestModeBer();
@@ -420,9 +524,13 @@ public:
 
     std::map<std::string, float>  getRawEffectiveErrors();
     std::map<std::string, float>  getRawEffectiveErrorsinTestMode();
+    int prbsModeToMask(const string &mode);
+    string prbsMaskToMode(u_int32_t mask, u_int32_t modeSelector);
     string getPrbsModeRX();
     u_int32_t getPrbsRateRX();
-    virtual u_int32_t getLoopbackMode(const string &lb);
+    string getSupportedPrbsModes(u_int32_t modeSelector);
+    u_int32_t getLoopbackMode(const string &lb);
+    string getLoopbackStr(u_int32_t loopbackCapMask);
     int getLinkDown();
     float getRawBERLimit();
     bool getResult(std::map<std::string, float>  errors_vector,
@@ -433,8 +541,19 @@ public:
     string getDeviceRev(bool queryMSGI = false);
     string getDeviceFW();
     string getVendorRev(const string & name);
+    void printOuptputVector(vector<MlxlinkCmdPrint> &cmdOut);
+    virtual void prepareJsonOut();
 
-    void prepareJsonOut();
+    // Cable operation
+    bool isPassiveQSFP();
+    bool isSFP51Paging();
+    void initCablesCommander();
+    void initEyeOpener();
+    void showCableDump();
+    void showCableDDM();
+    vector<u_int8_t> validateBytes(const vector<string> &strBytes);
+    void writeCableEEPROM();
+    void readCableEEPROM();
 
     MlxlinkCmdPrint _operatingInfoCmd;
     MlxlinkCmdPrint _supportedInfoCmd;
@@ -454,6 +573,9 @@ public:
     MlxlinkCmdPrint _extPhyInfoCmd;
     MlxlinkCmdPrint _linkBlameInfoCmd;
     MlxlinkCmdPrint _validPcieLinks;
+    MlxlinkCmdPrint _cableDumpRawCmd;
+    MlxlinkCmdPrint _cableDDMCmd;
+    MlxlinkCmdPrint _portGroupMapping;
 
     // Mlxlink config functions
     void clearCounters();
@@ -464,34 +586,41 @@ public:
     virtual void sendSltp();
     void sendPplr();
     virtual void sendPepc();
+    void setTxGroupMapping();
 
     // Config helper functions
-    void sendPaosCmd(PAOS_ADMIN adminStatus);
+    bool isForceDownSupported();
+    void sendPaosCmd(PAOS_ADMIN adminStatus, bool forceDown = false);
     void sendPaosDown();
     void sendPaosUP();
     void sendPaosToggle();
-    void checkPprtCap();
-    void checkPpttCap();
+    void checkPRBSModeCap(u_int32_t modeSelector, u_int32_t capMask);
+    void checkPrbsRegsCap(const string &prbsReg, const string &laneRate);
+    void checkPrbsPolCap(const string &prbsReg);
+    void checkPprtPptt();
+    void checkPplrCap();
     void sendPrbsPpaos(bool);
     void startTuning();
-    void sendPprt();
-    void sendPptt();
+    void prbsConfiguration(const string &prbsReg, bool enable, u_int32_t laneRate,
+            u_int32_t prbsMode, bool perLaneConfig, bool prbsPolInv);
+    void sendPprtPptt();
+    void resetPprtPptt();
     u_int32_t ptysSpeedToMask(const string & speed, u_int32_t cap);
     u_int32_t ptysSpeedToExtMask(const string & speed);
+    void validateSpeedStr();
     void checkSupportedSpeed(const string & speed, u_int32_t cap, bool extSpeed = false);
     void checkPplmCap();
     void updateSltp28_40nmFields();
     void updateSltp16nmFields();
+    void getSltpAlevOut(u_int32_t lane);
     void getSltpRegAndLeva(u_int32_t lane);
     virtual void checkSltpParamsSize();
 
     // Mlxlink params
-    MlxRegLib *_mlxLinkLib;
     UserInput _userInput;
-    mfile *_mf;
-    MlxlinkLogger *_mlxlinkLogger;
     dm_dev_id_t _devID;
     u_int32_t _localPort;
+    u_int32_t _portType;
     DPN _dpn;
     u_int32_t _numOfLanes;
     u_int32_t _numOfLanesPcie;
@@ -499,10 +628,12 @@ public:
     u_int32_t _fecActive;
     u_int32_t _protoActive;
     u_int32_t _uniqueCmds;
+    u_int32_t _uniqueCableCmds;
     u_int32_t _networkCmds;
     u_int32_t _anDisable;
     u_int32_t _speedBerCsv;
     u_int32_t _cableIdentifier;
+    u_int32_t _cableTechnology;
     u_int32_t _cableAtten12G;
     u_int32_t _cableLen;
     u_int32_t _activeSpeed;
@@ -511,6 +642,8 @@ public:
     u_int32_t _protoAdmin;
     u_int32_t _protoAdminEx;
     u_int32_t _productTechnology;
+    u_int32_t _moduleNumber;
+    u_int32_t _linkSpeed;
     string _extAdbFile;
     string _device;
     string _fwVersion;
@@ -520,7 +653,6 @@ public:
     string _cableSN;
     string _moduleTemp;
     bool _protoCapabilityEx;
-    bool _writeGvmi;
     bool _splitted;
     bool _linkUP;
     bool _prbsTestMode;
@@ -528,11 +660,21 @@ public:
     bool _linkModeForce;
     bool _useExtAdb;
     bool _isHCA;
+    bool _ddmSupported;
+    bool _cmisCable;
+    bool _qsfpCable;
+    bool _portPolling;
+    bool _mngCableUnplugged;
+    bool _isPam4Speed;
+    bool _ignorePortType;
     std::vector<std::string> _ptysSpeeds;
+    std::vector<PortGroup> _localPortsPerGroup;
     std::vector<DPN> _validDpns;
     string _allUnhandledErrors;
     Json::Value _jsonRoot;
     MlxlinkMaps* _mlxlinkMaps;
+    MlxlinkCablesCommander* _cablesCommander;
+    MlxlinkEyeOpener* _eyeOpener;
 };
 
 #endif /* MLXLINK_COMMANDER_H */
