@@ -177,9 +177,9 @@ int mtcr_check_signature(mfile *mf)
 
     switch (signature & 0xffff) {
     case 0x190:     /* 400 */
-        if (signature == 0xa00190 && mf->ptr) {
+        if (signature == 0xa00190 && mf->bar_virtual_addr) {
             mf->connectx_flush = 1;
-            mtcr_connectx_flush(mf->ptr, mf->fdlock);
+            mtcr_connectx_flush(mf->bar_virtual_addr, mf->fdlock);
         }
 
     case 0x5a44:     /* 23108 */
@@ -2023,7 +2023,7 @@ static int mreg_send_raw(mfile *mf, u_int16_t reg_id,
     memcpy(reg_data, buffer + OP_TLV_SIZE + REG_TLV_HEADER_LEN, reg_size);
 
 #ifdef _ENABLE_DEBUG_
-    fprintf(stdout, "-I-Tlv's of Data Recieved:\n");
+    fprintf(stdout, "-I-Tlv's of Data Received:\n");
     fprintf(stdout, "\tOperation Tlv\n");
     OperationTlv_dump(&tlv, stdout);
     fprintf(stdout, "\tReg Tlv\n");
@@ -2482,4 +2482,35 @@ int supports_reg_access_gmp(mfile *mf, maccess_reg_method_t reg_method)
     return 0;
 }
 
+int allocate_kernel_memory_page(mfile *mf, mtcr_alloc_page* user_alloc_page)
+{
+    (void)mf;
+    (void)user_alloc_page;
+    return ME_UNSUPPORTED_OPERATION;
+}
 
+
+int mset_i2c_addr_width(mfile *mf, u_int8_t addr_width)
+{
+    (void)mf;
+    (void)addr_width;
+    return 1;
+}
+int mget_i2c_addr_width(mfile *mf, u_int8_t *addr_width)
+{
+    (void)mf;
+    (void)addr_width;
+    return 1;
+}
+
+int set_i2c_freq(mfile* mf, u_int8_t freq) {
+    (void) mf;
+    (void) freq;
+    return 1;
+}
+
+int get_i2c_freq(mfile* mf, u_int8_t* freq) {
+    (void) mf;
+    (void) freq;
+    return 1;
+}
\ No newline at end of file
