@@ -434,7 +434,7 @@ unsigned long long mtcr_sysfs_get_offset(unsigned domain, unsigned bus, unsigned
     unsigned long long offset = (unsigned long long) -1;
     FILE *f;
     int cnt;
-    char mbuf[] = "/sys/bus/pci/devices/XXXX:XX:XX.X/resource";
+    char mbuf[99] = "/sys/bus/pci/devices/XXXX:XX:XX.X/resource";
     sprintf(mbuf, "/sys/bus/pci/devices/%4.4x:%2.2x:%2.2x.%1.1x/resource", domain, bus, dev, func);
 
     f = fopen(mbuf, "r");
@@ -818,9 +818,16 @@ static int driver_mread4_block(mfile *mf, unsigned int offset, u_int32_t *data,
             read4_buf.address_space = (unsigned int)mf->address_space;
             read4_buf.offset = offset;
             read4_buf.size = toread;
-            int ret = ioctl(mf->fd, PCICONF_READ4_BUFFER, &read4_buf);
-            if (ret < 0) {
-                return -1;
+
+            // We support backward compatibility.
+            // There is a known bug with PCICONF_READ4_BUFFER ioctl and data may be corrupted.
+            int ret;
+            if ((ret = ioctl(mf->fd, PCICONF_READ4_BUFFER_EX, &read4_buf)) < 0) {
+                if ((ret = ioctl(mf->fd, PCICONF_READ4_BUFFER, &read4_buf)) < 0) {
+                    if ((ret = ioctl(mf->fd, PCICONF_READ4_BUFFER_BC, &read4_buf)) < 0) {
+                        return -1;
+                    }
+                }
             }
             memcpy(dest_ptr, read4_buf.data, toread);
             offset += toread;
@@ -1095,6 +1102,9 @@ int mtcr_pciconf_rw(mfile *mf, unsigned int offset, u_int32_t *data, int rw)
 
     //last 2 bits must be zero as we only allow 30 bits addresses
     if (EXTRACT(address, 30, 2)) {
+        if (errno == EEXIST) {  // not sure if it's OK to change errno in all cases
+            errno = EINVAL;
+        }
         return ME_BAD_PARAMS;
     }
 
@@ -1622,15 +1632,18 @@ static long supported_dev_ids[] = {
     0x101b,     //Connect-X6
     0x101d,     //Connect-X6DX
     0x101f,     //Connect-X6LX
+    0x1021,     //Connect-X7
     0xc738,     //SwitchX
     0xcb20,     //Switch-IB
     0xcb84,     //Spectrum
     0xcf08,     //Switch-IB2
     0xd2f0,     //Quantum
+    0xd2f2,     //Quantum2
     0xcf6c,     //Spectrum2
     0xa2d2,     //MT416842 Family BlueField integrated ConnectX-5 network controller
     0xa2d6,     //MT416846 Family BlueField2 integrated ConnectX-6DX network controller
     0xcf70,     //Spectrum3
+    0xcf80,     //Spectrum4
     -1
 };
 
@@ -1652,6 +1665,7 @@ static long live_fish_id_database[] = {
     0x211,
     0x212, //Connect-X6DX
     0x216, //Connect-X6LX
+    0x21a, //Connect-X7
     0x250, //Spectrum3
     -1
 };
@@ -2242,11 +2256,11 @@ mfile* mopen_ul_int(const char *name, u_int32_t adv_opt)
     unsigned domain = 0, bus = 0, dev = 0, func = 0;
     MType dev_type;
     int force;
-    char rbuf[] = "/sys/bus/pci/devices/XXXX:XX:XX.X/resource0";
-    char cbuf[] = "/sys/bus/pci/devices/XXXX:XX:XX.X/config";
-    char pdbuf[] = "/proc/bus/pci/XXXX:XX/XX.X";
-    char pbuf[] = "/proc/bus/pci/XX/XX.X";
-    char pcidev[] = "XXXX:XX:XX.X";
+    char rbuf[99] = "/sys/bus/pci/devices/XXXX:XX:XX.X/resource0";
+    char cbuf[99] = "/sys/bus/pci/devices/XXXX:XX:XX.X/config";
+    char pdbuf[99] = "/proc/bus/pci/XXXX:XX/XX.X";
+    char pbuf[99] = "/proc/bus/pci/XX/XX.X";
+    char pcidev[99] = "XXXX:XX:XX.X";
     int err;
     int rc;
 
