@@ -31,62 +31,94 @@
  */
 
 #include "reg_access.h"
-#define REG_ID_PCNR  0x5050
-#define REG_ID_MIRC  0x9162
-#define REG_ID_MGIR  0x9020
-#define REG_ID_MFPA  0x9010
-#define REG_ID_MFBA  0x9011
-#define REG_ID_MFBE  0x9012
-#define REG_ID_MFMC  0x9013
-#define REG_ID_PMDIO 0x9017
-#define REG_ID_MJTAG 0x901F
+#include "common/compatibility.h"
+//FPGA
+#define REG_ID_FPGA_CAP                 0x4022
+#define REG_ID_FPGA_CTRL                0x4023
 
-#define REG_ID_PMDIC 0x9021
-#define REG_ID_MNVA  0x9024
-#define REG_ID_MNVI  0x9025
+#define REG_ID_PCNR                     0x5050
+#define REG_ID_PPTCS                    0x5801
+#define REG_ID_PGMP                     0x5802
+#define REG_ID_PPTS                     0x5803
+#define REG_ID_PLTS                     0x5804
+#define REG_ID_PLSRP                    0x5805
+#define REG_ID_PGPS                     0x5806
+#define REG_ID_PAOS                     0x5006
+#define REG_ID_PTYS                     0x5004
+#define REG_ID_PMAOS                    0x5012
+#define REG_ID_MCIA                     0x9014
+#define REG_ID_MTMP                     0x900a
+#define REG_ID_MIRC                     0x9162
+#define REG_ID_MGIR                     0x9020
+#define REG_ID_PMLP                     0x5002
+#define REG_ID_MFPA                     0x9010
+#define REG_ID_MFBA                     0x9011
+#define REG_ID_MFBE                     0x9012
+#define REG_ID_MFMC                     0x9013
+#define REG_ID_MMDIO                    0x9017
+#define REG_ID_MJTAG                    0x901F
+#define REG_ID_PMDIC                    0x9021
+#define REG_ID_MNVA                     0x9024
+#define REG_ID_MNVI                     0x9025
+#define REG_ID_MVTS                     0x902c
+#define REG_ID_NVQGC                    0x9034
+#define REG_ID_MNVIA                    0x9029 // 4th gen
+#define REG_ID_NVQC                     0x9030
+#define REG_ID_MNVGN                    0x9035
+#define REG_ID_NVIA                     0x9033 // 5th gen
+#define REG_ID_MGNLE                    0x9036
+#define REG_ID_MLCOK                    0x402D
+#define REG_ID_PLIB                     0x500a
+#define REG_ID_RITR                     0x8002
+#define REG_ID_RICNT                    0x800b
+#define REG_ID_PPCNT                    0x5008
+#define REG_ID_SBPR                     0xb001
+#define REG_ID_SBCM                     0xb002
+#define REG_ID_SBPM                     0xb003
+#define REG_ID_SBMM                     0xb004
+#define REG_ID_STRS_STOP_TOGGLE         0x4027
+#define REG_ID_STRS_FAULT_INJECT        0x4028
+#define REG_ID_STRS_MINI_FLOW           0x4029
+#define REG_ID_STRS_RESOURCE            0x402A
 
-#define REG_ID_MVTS  0x902c
-
-
-#define REG_ID_NVQGC 0x9034
-#define REG_ID_MNVIA 0x9029 // 4th gen
-#define REG_ID_NVQC  0x9030
-#define REG_ID_MNVGN 0x9035
-#define REG_ID_NVIA  0x9033 // 5th gen
-#define REG_ID_MGNLE 0x9036
-#define REG_ID_MLCOK 0x402D
-#define REG_ID_PLIB  0x500a
-#define REG_ID_STRS_STOP_TOGGLE   0x4027
-#define REG_ID_STRS_FAULT_INJECT  0x4028
-#define REG_ID_STRS_MINI_FLOW     0x4029
-#define REG_ID_STRS_RESOURCE      0x402A
-
-#define REG_ID_MTRC_CAP 0x9040
-#define REG_ID_MPEGC 0x9056
-#define REG_ID_MCDD  0x905C
-
-#define REG_ID_MCQS  0x9060
-#define REG_ID_MCC   0x9062
-#define REG_ID_MCQI  0x9061
-#define REG_ID_MCDA  0x9063
-#define REG_ID_MQIS  0x9064
-#define REG_ID_MCAM  0x907f
+#define REG_ID_MTRC_CAP                 0x9040
+#define REG_ID_MCDD                     0x905C
+#define REG_ID_MCQS                     0x9060
+#define REG_ID_MCC                      0x9062
+#define REG_ID_MCQI                     0x9061
+#define REG_ID_MCDA                     0x9063
+#define REG_ID_MQIS                     0x9064
+#define REG_ID_MCAM                     0x907f
 // TODO: get correct register ID for mfrl mfai
-#define REG_ID_MFRL 0x9028
-#define REG_ID_MFAI 0x9029
+#define REG_ID_MFRL                     0x9028
+#define REG_ID_MFAI                     0x9029
 #define REG_ID_MPCIR                    0x905a
 
 
+
+#define REG_ID_MGPIR                    0x9100
+#define REG_ID_MDFCR                    0x9101
+#define REG_ID_MDRCR                    0x9102
+
+#define REG_ID_MDDT                     0x9160
+#define REG_ID_MDDQ                     0x9161
+// For mstdump oob feature:
+#define REG_ID_ICAM                     0x387F
+#define REG_ID_ICSR                     0x38F0
+
+
 //==================================
 //RESOURCE DUMP FEATURE
 #define REG_ID_RES_DUMP                 0xC000
 #define REG_ID_DEBUG_CAP                0x8400
 //==================================
 
+#define REG_ID_MPEGC 0x9056
 
 //WA for MGIR: the reg size is too big so we limit it to INBAND_MAX_REG_SIZE
 #define INBAND_MAX_REG_SIZE 44
 #define MGIR_REG_SIZE INBAND_MAX_REG_SIZE
+#define PMAOS_REG_SIZE INBAND_MAX_REG_SIZE
 // End of WA
 
 // for debug:
@@ -103,6 +135,7 @@
 # define DEBUG_PRINT_RECEIVE(data_struct, struct_name, method, print_func)
 #endif
 
+
 /***************************************************/
 
 // register access for infinite size of arrays with specific size
@@ -154,14 +187,14 @@
                                            unpack_func, size_func, print_func, status) \
     int rc; \
     int max_data_size = size_func(); \
-    u_int8_t *data = (u_int8_t*)malloc(sizeof(u_int8_t) * max_data_size); \
+    u_int8_t *data = NULL; \
+    if (method != REG_ACCESS_METHOD_GET && method != REG_ACCESS_METHOD_SET) { \
+        return ME_REG_ACCESS_BAD_METHOD; \
+    } \
+    data = (u_int8_t*)malloc(sizeof(u_int8_t) * max_data_size); \
     if (!data) { return ME_MEM_ERROR;}; \
     memset(data, 0, max_data_size); \
     pack_func(data_struct, data); \
-    if (method != REG_ACCESS_METHOD_GET && method != REG_ACCESS_METHOD_SET) { \
-        free(data); \
-        return ME_REG_ACCESS_BAD_METHOD; \
-    } \
     DEBUG_PRINT_SEND(data_struct, struct_name, method, print_func); \
     rc = maccess_reg(mf, reg_id, (maccess_reg_method_t)method, data, reg_size, r_reg_size, w_reg_size, status); \
     unpack_func(data_struct, data); \
@@ -189,25 +222,13 @@
     int data_size = prefix##_##struct_name##_size(); \
     REG_ACCCESS_VAR(mf, method, reg_id, data_struct, struct_name, data_size, data_size, data_size, prefix)
 
+#ifndef UEFI_BUILD
 
-
-//=================================================================================================================================
-/************************************
- * * Function: reg_access_resource_dump
- * ************************************/
-reg_access_status_t reg_access_res_dump(mfile *mf, reg_access_method_t method, struct reg_access_hca_resource_dump *resource_dump)
+reg_access_status_t reg_access_mddq(mfile *mf, reg_access_method_t method, struct reg_access_switch_mddq *mddq)
 {
-    REG_ACCCESS(mf, method, REG_ID_RES_DUMP, resource_dump, resource_dump, reg_access_hca);
+    REG_ACCCESS(mf, method, REG_ID_MDDQ, mddq, mddq, reg_access_switch);
 }
 
-reg_access_status_t reg_access_debug_cap(mfile *mf, reg_access_method_t method, struct reg_access_hca_debug_cap *debug_cap)
-{
-    REG_ACCCESS(mf, method, REG_ID_DEBUG_CAP, debug_cap, debug_cap, reg_access_hca);
-}
-
-//=================================================================================================================================
-
-
 /************************************
  * Function: reg_access_pcnr
  ************************************/
@@ -222,22 +243,35 @@ reg_access_status_t reg_access_pcnr(mfile *mf, reg_access_method_t method, struc
 ************************************/
 void reg_access_hca_strs_stop_toggle_reg_special_pack(const struct reg_access_hca_strs_stop_toggle_reg *ptr_struct, u_int8_t *ptr_buff)
 {
-    u_int8_t rxb_hang = 0x9;
+    const u_int8_t RXB_HANG_TYPE = 0x9;
+    const u_int8_t SXP_HANG_TYPE = 0x10;
+    const u_int8_t PAUSE_TX_TYPE = 0x15;
     u_int8_t union_offset = 0x10;
     reg_access_hca_strs_stop_toggle_reg_pack(ptr_struct, ptr_buff);
-    if (ptr_struct->type == rxb_hang) {
-        reg_access_hca_rxb_hang_stop_toggle_modifier_pack((const struct reg_access_hca_rxb_hang_stop_toggle_modifier*)&(ptr_struct->per_type_modifier), ptr_buff + union_offset);
-    } else {
-        reg_access_hca_lock_source_stop_toggle_modifier_pack((const struct reg_access_hca_lock_source_stop_toggle_modifier*)&(ptr_struct->per_type_modifier), ptr_buff + union_offset);
+    if (ptr_struct->type == RXB_HANG_TYPE) {
+        reg_access_hca_rxb_hang_stop_toggle_modifier_pack((const struct reg_access_hca_rxb_hang_stop_toggle_modifier*)&(ptr_struct->per_type_modifier),
+                ptr_buff + union_offset);
+    }
+    else if (ptr_struct->type == PAUSE_TX_TYPE) {
+        reg_access_hca_pause_tx_stop_toggle_modifier_pack((const struct reg_access_hca_pause_tx_stop_toggle_modifier*)&(ptr_struct->per_type_modifier),
+                ptr_buff + union_offset);
+    }
+    else if (ptr_struct->type == SXP_HANG_TYPE) {
+        reg_access_hca_sxp_hang_stop_toggle_modifier_pack((const struct reg_access_hca_sxp_hang_stop_toggle_modifier*)&(ptr_struct->per_type_modifier),
+                ptr_buff + union_offset);
+    }
+    else {
+        reg_access_hca_lock_source_stop_toggle_modifier_pack((const struct reg_access_hca_lock_source_stop_toggle_modifier*)&(ptr_struct->per_type_modifier),
+                ptr_buff + union_offset);
     }
-
 }
 
 reg_access_status_t reg_access_strs_stop_toggle_reg(mfile *mf, reg_access_method_t method, struct reg_access_hca_strs_stop_toggle_reg *streg)
 {
     int data_size = reg_access_hca_strs_stop_toggle_reg_size();
-    REG_ACCESS_GENERIC_VAR(mf, method, REG_ID_STRS_STOP_TOGGLE, streg, reg_access_hca_strs_stop_toggle_reg, data_size, data_size, data_size, reg_access_hca_strs_stop_toggle_reg_special_pack, \
-                           reg_access_hca_strs_stop_toggle_reg_unpack, reg_access_hca_strs_stop_toggle_reg_size, reg_access_hca_strs_stop_toggle_reg_print);
+    REG_ACCESS_GENERIC_VAR(mf, method, REG_ID_STRS_STOP_TOGGLE, streg, reg_access_hca_strs_stop_toggle_reg, data_size, data_size, data_size, \
+            reg_access_hca_strs_stop_toggle_reg_special_pack, reg_access_hca_strs_stop_toggle_reg_unpack, \
+            reg_access_hca_strs_stop_toggle_reg_size, reg_access_hca_strs_stop_toggle_reg_print);
 }
 
 
@@ -253,9 +287,29 @@ reg_access_status_t reg_access_strs_fault_injector_reg(mfile *mf, reg_access_met
 /************************************
 * Function: reg_access_strs_mini_flow_reg
 ************************************/
+void reg_access_hca_strs_mini_flow_reg_special_pack(const struct reg_access_hca_strs_mini_flow_reg *ptr_struct, u_int8_t *ptr_buff) {
+    const u_int8_t IRISC_HANG_TYPE = 0xF;
+    // const u_int8_t PACKET_DROP_TYPE = 0x10;
+    u_int8_t union_offset = 0x10;
+    reg_access_hca_strs_mini_flow_reg_pack(ptr_struct, ptr_buff);
+    if (ptr_struct->type == IRISC_HANG_TYPE) {
+        reg_access_hca_irisc_hang_mini_flow_modifier_pack((const struct reg_access_hca_irisc_hang_mini_flow_modifier*)&(ptr_struct->per_type_modifier),
+                ptr_buff + union_offset);
+    }
+    else {
+        reg_access_hca_packet_drop_mini_flow_modifier_pack((const struct reg_access_hca_packet_drop_mini_flow_modifier*)&(ptr_struct->per_type_modifier),
+                ptr_buff + union_offset);
+    }
+}
+
+
 reg_access_status_t reg_access_strs_mini_flow_reg(mfile *mf, reg_access_method_t method, struct reg_access_hca_strs_mini_flow_reg *strs_mini_flow_reg)
 {
-    REG_ACCCESS(mf, method, REG_ID_STRS_MINI_FLOW, strs_mini_flow_reg, strs_mini_flow_reg, reg_access_hca);
+    // REG_ACCCESS(mf, method, REG_ID_STRS_MINI_FLOW, strs_mini_flow_reg, strs_mini_flow_reg, reg_access_hca);
+    int data_size = reg_access_hca_strs_mini_flow_reg_size();
+    REG_ACCESS_GENERIC_VAR(mf, method, REG_ID_STRS_MINI_FLOW, strs_mini_flow_reg, reg_access_hca_strs_mini_flow_reg, data_size, data_size, data_size, \
+            reg_access_hca_strs_mini_flow_reg_special_pack, reg_access_hca_strs_mini_flow_reg_unpack, \
+            reg_access_hca_strs_mini_flow_reg_size, reg_access_hca_strs_mini_flow_reg_print);
 }
 
 /************************************
@@ -266,9 +320,21 @@ reg_access_status_t reg_access_strs_resource_reg(mfile *mf, reg_access_method_t
     REG_ACCCESS(mf, method, REG_ID_STRS_RESOURCE, strs_resource_reg, strs_resource_reg, reg_access_hca);
 }
 
+/************************************
+* Function: reg_access_pmaos
+************************************/
+reg_access_status_t reg_access_pmaos(mfile *mf, reg_access_method_t method, struct register_access_sib_pmaos *pmaos)
+{
+    REG_ACCCESS(mf, method, REG_ID_PMAOS, pmaos, pmaos, register_access_sib);
+}
 
-
-
+/************************************
+* Function: reg_access_mtmp
+************************************/
+reg_access_status_t reg_access_mtmp(mfile *mf, reg_access_method_t method, struct register_access_sib_mtmp *mtmp)
+{
+    REG_ACCCESS(mf, method, REG_ID_MTMP, mtmp, mtmp, register_access_sib);
+}
 
 /************************************
 * Function: reg_access_mnva
@@ -320,13 +386,6 @@ reg_access_status_t reg_access_nvqgc(mfile *mf, reg_access_method_t method, stru
     REG_ACCCESS(mf, method, REG_ID_NVQGC, nvqgc, nvqgc, tools_open);
 }
 
-
-
-
-
-
-
-
 /************************************
 * Function: reg_access_nvdi
 ************************************/
@@ -339,7 +398,7 @@ reg_access_status_t reg_access_nvdi(mfile *mf, reg_access_method_t method, struc
 }
 
 /************************************
-* Function: reg_access_nvia
+* Function: reg_access_nvdia
 ************************************/
 reg_access_status_t reg_access_nvia(mfile *mf, reg_access_method_t method, struct tools_open_nvia *nvia)
 {
@@ -360,10 +419,6 @@ reg_access_status_t reg_access_nvqc(mfile *mf, reg_access_method_t method, struc
     REG_ACCCESS(mf, method, REG_ID_NVQC, nvqc, nvqc, tools_open);
 }
 
-
-
-
-
 /************************************
 * Function: reg_access_mnvgn
 ************************************/
@@ -384,10 +439,84 @@ reg_access_status_t reg_access_mnvgn(mfile *mf, reg_access_method_t method, stru
     return ME_OK;
 }
 
+/************************************
+* Function: reg_access_mgnle
+************************************/
+reg_access_status_t reg_access_mgnle(mfile *mf, reg_access_method_t method, struct tools_open_mgnle *mgnle, int *status)
+{
+    int data_size = tools_open_mgnle_size();
+    if (method != REG_ACCESS_METHOD_GET) { // this register supports only get method
+        return ME_REG_ACCESS_BAD_METHOD;
+    }
+
+    REG_ACCESS_GENERIC_VAR_WITH_STATUS(mf, method, REG_ID_MGNLE, mgnle, mgnle, data_size, data_size, data_size, \
+                                       tools_open_mgnle_pack, tools_open_mgnle_unpack, tools_open_mgnle_size, \
+                                       tools_open_mgnle_print, status);
+
+    if (rc || *status) { \
+        return (reg_access_status_t)rc; \
+    } \
+
+    return ME_OK;
+}
+
+/************************************
+* Function: reg_access_mmdio
+************************************/
+reg_access_status_t reg_access_mmdio(mfile *mf, reg_access_method_t method, struct reg_access_hca_mmdio *mmdio)
+{
+    if (method != REG_ACCESS_METHOD_SET) {   // this register supports only set method
+        return ME_REG_ACCESS_BAD_METHOD;
+    }
+    REG_ACCCESS(mf, method, REG_ID_MMDIO, mmdio, mmdio, reg_access_hca);
+}
+
+/************************************
+* Function: reg_access_pmdic
+************************************/
+reg_access_status_t reg_access_pmdic(mfile *mf, reg_access_method_t method, struct tools_open_pmdic *pmdic)
+{
+    if (method != REG_ACCESS_METHOD_GET) {  // this register supports only get method
+        return ME_REG_ACCESS_BAD_METHOD;
+    }
+    REG_ACCCESS(mf, method, REG_ID_PMDIC, pmdic, pmdic, tools_open);
+}
+
+/************************************
+* Function: reg_access_fpga_cap
+************************************/
+reg_access_status_t reg_access_fpga_cap(mfile *mf, reg_access_method_t method, struct reg_access_hca_fpga_cap *fpga_cap)
+{
+    REG_ACCCESS(mf, method, REG_ID_FPGA_CAP, fpga_cap, fpga_cap, reg_access_hca);
+}
 
 
+//=================================================================================================================================
+/************************************
+ * * Function: reg_access_resource_dump
+ * ************************************/
+reg_access_status_t reg_access_res_dump(mfile *mf, reg_access_method_t method, struct reg_access_hca_resource_dump *resource_dump)
+{
+    REG_ACCCESS(mf, method, REG_ID_RES_DUMP, resource_dump, resource_dump, reg_access_hca);
+}
+
+reg_access_status_t reg_access_debug_cap(mfile *mf, reg_access_method_t method, struct reg_access_hca_debug_cap *debug_cap)
+{
+    REG_ACCCESS(mf, method, REG_ID_DEBUG_CAP, debug_cap, debug_cap, reg_access_hca);
+}
+
+//=================================================================================================================================
 
 
+/************************************
+* Function: reg_access_fpga_ctrl
+************************************/
+reg_access_status_t reg_access_fpga_ctrl(mfile *mf, reg_access_method_t method, struct reg_access_hca_fpga_ctrl *fpga_ctrl)
+{
+    REG_ACCCESS(mf, method, REG_ID_FPGA_CTRL, fpga_ctrl, fpga_ctrl, reg_access_hca);
+}
+
+#endif //#ifndef UEFI_BUILD
 
 /************************************
 * Function: reg_access_nvda
@@ -428,7 +557,6 @@ reg_access_status_t reg_access_mirc(mfile *mf, reg_access_method_t method, struc
 }
 
 
-
 /************************************
 * Function: reg_access_mfba
 ************************************/
@@ -457,8 +585,6 @@ reg_access_status_t reg_access_mcam(mfile *mf, reg_access_method_t method, struc
     REG_ACCCESS(mf, method, REG_ID_MCAM, mcam, mcam, tools_open);
 }
 
-
-
 /************************************
 * Function: reg_access_mcda
 ************************************/
@@ -483,20 +609,6 @@ reg_access_status_t reg_access_mtrc_cap(mfile *mf, reg_access_method_t method, s
     REG_ACCCESS(mf, method, REG_ID_MTRC_CAP, mtrc_cap, mtrc_cap_reg, reg_access_hca);
 }
 
-
-/************************************
-* Function: reg_access_mpegc
-************************************/
-reg_access_status_t reg_access_mpegc(mfile *mf, reg_access_method_t method, struct reg_access_hca_mpegc_reg *mpegc)
-{
-//    reg_access_hca_mpegc_reg_dump(mpegc, stdout)s;
-    REG_ACCCESS(mf, method, REG_ID_MPEGC, mpegc, mpegc_reg, reg_access_hca);
-}
-
-
-
-
-
 /************************************
 * Function: reg_access_mcc
 ************************************/
@@ -518,8 +630,7 @@ reg_access_status_t reg_access_mcqs(mfile *mf, reg_access_method_t method, struc
 ************************************/
 reg_access_status_t reg_access_mcqi(mfile *mf, reg_access_method_t method, struct reg_access_hca_mcqi_reg *mcqi)
 {
-    REG_ACCESS_GEN_DATA_WITH_STATUS(mf, method, REG_ID_MCQI, mcqi, mcqi_reg,
-                                    reg_access_hca, reg_size, r_size_reg, w_size_reg, reg_access_hca_mcqi_reg_size, data, data_size);
+    REG_ACCCESS(mf, method, REG_ID_MCQI, mcqi, mcqi_reg, reg_access_hca);
 }
 
 /************************************
@@ -538,7 +649,6 @@ reg_access_status_t reg_access_mfpa(mfile *mf, reg_access_method_t method, struc
     REG_ACCCESS(mf, method, REG_ID_MFPA, mfpa, mfpa, register_access);
 }
 
-
 /************************************
 * Function: reg_access_mfrl
 ************************************/
@@ -547,7 +657,6 @@ reg_access_status_t reg_access_mfrl(mfile *mf, reg_access_method_t method, struc
     REG_ACCCESS(mf, method, REG_ID_MFRL, mfrl, mfrl_reg_ext, reg_access_hca);
 }
 
-
 /************************************
 * Function: reg_access_mpcir
 ************************************/
@@ -556,6 +665,7 @@ reg_access_status_t reg_access_mpcir(mfile *mf, reg_access_method_t method, stru
     REG_ACCCESS(mf, method, REG_ID_MPCIR, mpcir, mpcir_ext, reg_access_hca);
 }
 
+
 /************************************
 * Function: reg_access_mfai
 ************************************/
@@ -584,7 +694,7 @@ reg_access_status_t reg_access_mfmc(mfile *mf, reg_access_method_t method, struc
 }
 
 /************************************
-* Function: reg_access_mcdd
+* Function: reg_access_mfmc
 ************************************/
 reg_access_status_t reg_access_mcdd(mfile *mf, reg_access_method_t method, struct tools_open_mcdd_reg *mcdd)
 {
@@ -613,3 +723,11 @@ const char* reg_access_err2str(reg_access_status_t status)
     return m_err2str(status);
 }
 
+/************************************
+* Function: reg_access_mpegc
+************************************/
+reg_access_status_t reg_access_mpegc(mfile *mf, reg_access_method_t method, struct reg_access_hca_mpegc_reg *mpegc)
+{
+//    reg_access_hca_mpegc_reg_dump(mpegc, stdout)s;
+    REG_ACCCESS(mf, method, REG_ID_MPEGC, mpegc, mpegc_reg, reg_access_hca);
+}
