@@ -1,4 +1,3 @@
-
 # Copyright (c) 2004-2010 Mellanox Technologies LTD. All rights reserved.
 #
 # This software is available to you under a choice of one of two
@@ -79,6 +78,7 @@ if REG_ACCESS:
         _fields_ = [
             ("segment_type",c_uint16),
             ("seq_num",c_uint8),
+            ("vhca_id_valid",c_uint8),
             ("inline_dump",c_uint8),
             ("more_dump",c_uint8),
             ("vhca_id",c_uint16),
@@ -133,7 +133,9 @@ if REG_ACCESS:
         _fields_ = [
             ("reset_level", c_uint8),
             ("reset_type", c_uint8),
-            ("rst_type_sel", c_uint8)
+            ("rst_type_sel", c_uint8),
+            ("pci_sync_for_fw_update_resp", c_uint8),
+            ("pci_sync_for_fw_update_start", c_uint8)
         ]
 
     class MPCIR_ST(Structure):
@@ -161,14 +163,11 @@ if REG_ACCESS:
                     ("reg_access_hca_string_db_parameters", c_uint8 * 64)]
 
 
-
-
-
     class MGIR_ST(Structure):
         _fields_ = [("device_id", c_uint16),
                     ("device_hw_revision", c_uint16),
                     ("pvs", c_uint8),
-                    ("num_ports",c_uint8),                    
+                    ("num_ports",c_uint8),
                     ("hw_dev_id", c_uint16),
                     ("manufacturing_base_mac_47_32", c_uint16),
                     ("manufacturing_base_mac_31_0", c_uint32),
@@ -192,9 +191,9 @@ if REG_ACCESS:
                     ("extended_minor", c_uint32),
                     ("extended_sub_minor", c_uint32),
                     ("isfu_major", c_uint16),
-                    ("subminor", c_uint8),
-                    ("minor", c_uint8),
-                    ("major", c_uint8),
+                    ("fw_subminor", c_uint8),
+                    ("fw_minor", c_uint8),
+                    ("fw_major", c_uint8),
                     ("rom3_type", c_uint8),
                     ("rom3_arch", c_uint8),
                     ("rom2_type", c_uint8),
@@ -209,8 +208,43 @@ if REG_ACCESS:
                     ("rom3_version", c_uint32),
                     ("dev_branch_tag", c_uint8 * 28)]
 
-    class RegAccess:
+    class MDDQ_SLOT_INFO(Structure):
+        _fields_ = [("active", c_uint8),
+                    ("ready", c_uint8),
+                    ("sr_valid", c_uint8),
+                    ("provisioned", c_uint8),
+                    ("minor_ini_file_version", c_uint16),
+                    ("major_ini_file_version", c_uint16),
+                    ("card_type", c_uint8)]
 
+    class MDDQ_DEVICE_INFO(Structure):
+        _fields_ = [("device_index", c_uint8),
+                    ("flash_id", c_uint8),
+                    ("thermal_sd", c_uint8),
+                    ("flash_owner", c_uint8),
+                    ("uses_flash", c_uint8),
+                    ("device_type", c_uint16),
+                    ("fw_major", c_uint16),
+                    ("fw_sub_minor", c_uint16),
+                    ("fw_minor", c_uint16)]
+
+    class MDDQ_DATA_UN(Union):
+        _fields_ = [("mddq_slot_info", MDDQ_SLOT_INFO),
+                    ("mddq_device_info", MDDQ_DEVICE_INFO)]
+
+    class MDDQ_ST(Structure):
+        _fields_ = [("slot_index", c_uint8),
+                    ("query_type", c_uint8),
+                    ("sie", c_uint8),
+                    ("request_message_sequence", c_uint8),
+                    ("response_message_sequence", c_uint8),
+                    ("query_index", c_uint8),
+                    ("frst", c_uint8),
+                    ("mid", c_uint8),
+                    ("lst", c_uint8),
+                    ("data", MDDQ_DATA_UN)]
+
+    class RegAccess:
         GET = REG_ACCESS_METHOD_GET
         SET = REG_ACCESS_METHOD_SET
 
@@ -219,8 +253,6 @@ if REG_ACCESS:
             self._mstDev = dev
             if pci_device is not None:
                 self._mstDev = mtcr.MstDevice(pci_device)
-            self._err2str = REG_ACCESS.reg_access_err2str
-            self._err2str.restype = c_char_p
             self._reg_access_mcam = REG_ACCESS.reg_access_mcam
             self._reg_access_mtrc_cap = REG_ACCESS.reg_access_mtrc_cap
             self._reg_access_mgir = REG_ACCESS.reg_access_mgir
@@ -229,7 +261,12 @@ if REG_ACCESS:
             self._reg_access_mpcir = REG_ACCESS.reg_access_mpcir
             self._reg_access_res_dump = REG_ACCESS.reg_access_res_dump
             self._reg_access_debug_cap = REG_ACCESS.reg_access_debug_cap
+            self._reg_access_mddq = REG_ACCESS.reg_access_mddq
 
+        def _err2str(self, rc):
+            err2str = REG_ACCESS.reg_access_err2str
+            err2str.restype = c_char_p
+            return err2str(rc).decode("utf-8")
 
         ##########################
         def close(self):
@@ -240,6 +277,9 @@ if REG_ACCESS:
             if self._mstDev:
                 self.close()
 
+
+
+
         def sendMtrcCapTakeOwnership(self):
             
             mcamRegP = pointer(MCAM_REG_ST())
@@ -270,13 +310,14 @@ if REG_ACCESS:
                 iter += 1
 
 
-        def sendResDump(self, segment_type, seq_num, inline_mode, more_dump, vhca_id, index1, index2, num_of_obj2, num_of_obj1, device_opaque, mkey, size, address):
+        def sendResDump(self, segment_type, seq_num, inline_mode, more_dump, vhca_id, vhca_id_valid, index1, index2, num_of_obj2, num_of_obj1, device_opaque, mkey, size, address):
             resDumpRegP = pointer(RES_DUMP_ST())
             resDumpRegP.contents.segment_type = c_uint16(segment_type)
             resDumpRegP.contents.seq_num = c_uint8(seq_num)
             resDumpRegP.contents.inline_dump = c_uint8(inline_mode)
             resDumpRegP.contents.more_dump = c_uint8(more_dump)
             resDumpRegP.contents.vhca_id = c_uint16(vhca_id)
+            resDumpRegP.contents.vhca_id_valid = c_uint8(vhca_id_valid)
             resDumpRegP.contents.index_1 = c_uint32(index1)
             resDumpRegP.contents.index_2 = c_uint32(index2)
             resDumpRegP.contents.num_of_obj_2 = c_uint16(num_of_obj2)
@@ -294,6 +335,7 @@ if REG_ACCESS:
                      "inline_dump": resDumpRegP.contents.inline_dump,
                      "more_dump": resDumpRegP.contents.more_dump,
                      "vhca_id": resDumpRegP.contents.vhca_id,
+                     "vhca_id_valid": resDumpRegP.contents.vhca_id_valid,
                      "index_1": resDumpRegP.contents.index_1,
                      "index_2": resDumpRegP.contents.index_2,
                      "num_of_obj_2": resDumpRegP.contents.num_of_obj_2,
@@ -362,16 +404,17 @@ if REG_ACCESS:
 
 
         ##########################
-        def sendMFRL(self, method, resetLevel=None, reset_type=None):
+        def sendMFRL(self, method, resetLevel=None, reset_type=None, reset_sync=None):
 
             mfrlRegisterP = pointer(MFRL_ST())
             
             if method == REG_ACCESS_METHOD_SET:
-                if resetLevel is None or reset_type is None:
-                    raise RegAccException("Failed to sendMFRL (reset-level or reset-type is None for SET command)")
+                if resetLevel is None or reset_type is None or reset_sync is None:
+                    raise RegAccException("Failed to sendMFRL (reset level/type/sync is None for SET command)")
                 mfrlRegisterP.contents.reset_level = c_uint8(resetLevel)
                 mfrlRegisterP.contents.rst_type_sel = c_uint8(reset_type)
-            
+                mfrlRegisterP.contents.pci_sync_for_fw_update_start = c_uint8(reset_sync)
+
             c_method = c_uint(method)
             rc = self._reg_access_mfrl(self._mstDev.mf, c_method, mfrlRegisterP)
             # FW bug first mfrl register might fail
@@ -384,6 +427,55 @@ if REG_ACCESS:
                 return mfrlRegisterP.contents.reset_level, mfrlRegisterP.contents.reset_type
 
         ##########################
+        def getMCAM(self):
+
+            def to_bits(list_of_bytes):
+                list_of_bits = []
+                for byte_ in list_of_bytes:
+                    # print("{0:08b}".format(byte_))
+                    for ii in range(7,-1,-1):
+                        val = 0 if byte_ & (1<<ii) == 0 else 1
+                        list_of_bits.append(val)
+                return list_of_bits[::-1]
+
+            mcamRegP = pointer(MCAM_REG_ST())
+            rc = self._reg_access_mcam(self._mstDev.mf, c_uint(REG_ACCESS_METHOD_GET), mcamRegP)            
+            if rc:
+                raise RegAccException("Failed to send Register MCAM (rc: {0})".format(rc))
+            
+            return {
+                "access_reg_group" :mcamRegP.contents.access_reg_group,
+                "feature_group":mcamRegP.contents.feature_group,
+                "mng_access_reg_cap_mask": to_bits(mcamRegP.contents.mng_access_reg_cap_mask),
+                "mng_feature_cap_mask": to_bits(mcamRegP.contents.mng_feature_cap_mask)
+            }
+
+
+        ##########################
+        def getFwVersion(self):
+
+            mgirRegisterP = pointer(MGIR_ST())
+
+            rc = self._reg_access_mgir(self._mstDev.mf, REG_ACCESS_METHOD_GET, mgirRegisterP)
+            if rc:
+                info_ver = ""
+            else:
+                # get dev branch tag
+                info_ver = "".join(chr(val) for val in mgirRegisterP.contents.dev_branch_tag if val != 0)
+
+                # if dev branch is empty, try to get the master version
+                if info_ver == "":
+                    if mgirRegisterP.contents.fw_major != 0:
+                        info_ver = "{0}.{1}.{2}".format(mgirRegisterP.contents.fw_major,
+                                                        mgirRegisterP.contents.fw_minor,
+                                                        mgirRegisterP.contents.fw_subminor)
+                    elif mgirRegisterP.contents.extended_major != 0:
+                        info_ver = "{0}.{1}.{2}".format(mgirRegisterP.contents.extended_major,
+                                                        mgirRegisterP.contents.extended_minor,
+                                                        mgirRegisterP.contents.extended_sub_minor)
+
+            return info_ver
+
         def getFWUptime(self):
 
             mgirRegisterP = pointer(MGIR_ST())
@@ -423,6 +515,46 @@ if REG_ACCESS:
             result = True if result else False
             return result
 
+        def sendMddq(self, query_type, request_message_sequence, slot_index, query_index=0):
+            mddqRegisterP = pointer(MDDQ_ST())
+            mddqRegisterP.contents.query_type = c_uint8(query_type)
+            mddqRegisterP.contents.slot_index = c_uint8(slot_index)
+            mddqRegisterP.contents.request_message_sequence = c_uint8(request_message_sequence)
+            mddqRegisterP.contents.query_index = c_uint8(query_index)
+            rc = self._reg_access_mddq(self._mstDev.mf, REG_ACCESS_METHOD_GET, mddqRegisterP)
+            if rc:
+                raise RegAccException("Failed to send Register: %s (%d)" % (self._err2str(rc), rc))
+
+            data = {}
+            if query_type == 1:
+                data.update({
+                    "active": mddqRegisterP.contents.data.mddq_slot_info.active,
+                    "ready": mddqRegisterP.contents.data.mddq_slot_info.ready,
+                    "sr_valid": mddqRegisterP.contents.data.mddq_slot_info.sr_valid,
+                    "provisioned": mddqRegisterP.contents.data.mddq_slot_info.provisioned,
+                    "minor_ini_file_version": mddqRegisterP.contents.data.mddq_slot_info.minor_ini_file_version,
+                    "major_ini_file_version": mddqRegisterP.contents.data.mddq_slot_info.major_ini_file_version
+                })
+            elif query_type == 2:
+                data.update({
+                    "device_index": mddqRegisterP.contents.data.mddq_device_info.device_index,
+                    "flash_id": mddqRegisterP.contents.data.mddq_device_info.flash_id,
+                    "flash_owner": mddqRegisterP.contents.data.mddq_device_info.flash_owner,
+                    "uses_flash": mddqRegisterP.contents.data.mddq_device_info.uses_flash,
+                    "device_type": mddqRegisterP.contents.data.mddq_device_info.device_type,
+                    "fw_major": mddqRegisterP.contents.data.mddq_device_info.fw_major,
+                    "fw_sub_minor": mddqRegisterP.contents.data.mddq_device_info.fw_sub_minor,
+                    "fw_minor": mddqRegisterP.contents.data.mddq_device_info.fw_minor
+                })
+            return ({"slot_index": mddqRegisterP.contents.slot_index,
+                     "query_type": mddqRegisterP.contents.query_type,
+                     "request_message_sequence": mddqRegisterP.contents.request_message_sequence,
+                     "response_message_sequence": mddqRegisterP.contents.response_message_sequence,
+                     "frst": mddqRegisterP.contents.frst,
+                     "mid": mddqRegisterP.contents.mid,
+                     "lst": mddqRegisterP.contents.lst},
+                     data)
+
 else:
     raise RegAccException("Failed to load rreg_access.so/libreg_access.dll")
 
