@@ -1,3 +1,4 @@
+
 /*
  * Copyright (C) Jan 2013 Mellanox Technologies Ltd. All rights reserved.
  *
@@ -31,7 +32,7 @@
  */ 
 
 /***
-         *** This file was generated at "2018-01-07 08:13:17"
+         *** This file was generated at "2019-08-11 16:47:37"
          *** by:
          ***    > /mswg/release/tools/a-me/last_stable/adabe_plugins/adb2c/adb2pack.py --input adb/cibfw/cibfw.adb --file-prefix cibfw --prefix cibfw_
          ***/
@@ -50,22 +51,20 @@ struct cibfw_uint64 {
 /*---------------- DWORD[0] (Offset 0x0) ----------------*/
 	/* Description -  */
 	/* 0x0.0 - 0x4.31 */
-	 u_int64_t uint64;
+	u_int64_t uint64;
 };
 
 /* Description -   */
-/* Size in bytes - 4 */
-struct cibfw_module_version {
+/* Size in bytes - 8 */
+struct cibfw_reset_version {
 /*---------------- DWORD[0] (Offset 0x0) ----------------*/
-	/* Description -  */
-	/* 0x0.0 - 0x0.7 */
-	 u_int8_t branch;
-	/* Description -  */
-	/* 0x0.8 - 0x0.19 */
-	 u_int16_t minor;
-	/* Description -  */
-	/* 0x0.20 - 0x0.31 */
-	 u_int16_t major;
+	/* Description - Match means upgrade supported, mismatch implies reset not supported */
+	/* 0x0.0 - 0x0.31 */
+	u_int32_t major;
+/*---------------- DWORD[1] (Offset 0x4) ----------------*/
+	/* Description - If Major matches. Match means seamless reset. Greater means reset supported. Less then upgrade not supported */
+	/* 0x4.0 - 0x4.7 */
+	u_int8_t minor;
 };
 
 /* Description -   */
@@ -74,15 +73,15 @@ struct cibfw_uid_entry {
 /*---------------- DWORD[0] (Offset 0x0) ----------------*/
 	/* Description - Number of allocated UIDs in this entry */
 	/* 0x0.0 - 0x0.7 */
-	 u_int8_t num_allocated;
+	u_int8_t num_allocated;
 	/* Description - Step size by which to derive the UIDs for this entry
 See struct description */
 	/* 0x0.8 - 0x0.15 */
-	 u_int8_t step;
+	u_int8_t step;
 /*---------------- DWORD[2] (Offset 0x8) ----------------*/
 	/* Description - For MACs, the upper 16 bits in the 'hi' dword are reserved */
 	/* 0x8.0 - 0xc.31 */
-	 u_int64_t uid;
+	u_int64_t uid;
 };
 
 /* Description -   */
@@ -91,34 +90,34 @@ struct cibfw_FW_VERSION {
 /*---------------- DWORD[0] (Offset 0x0) ----------------*/
 	/* Description -  */
 	/* 0x0.16 - 0x0.31 */
-	 u_int16_t MAJOR;
+	u_int16_t MAJOR;
 /*---------------- DWORD[1] (Offset 0x4) ----------------*/
 	/* Description -  */
 	/* 0x4.0 - 0x4.15 */
-	 u_int16_t SUBMINOR;
+	u_int16_t SUBMINOR;
 	/* Description -  */
 	/* 0x4.16 - 0x4.31 */
-	 u_int16_t MINOR;
+	u_int16_t MINOR;
 /*---------------- DWORD[2] (Offset 0x8) ----------------*/
 	/* Description -  */
 	/* 0x8.8 - 0x8.15 */
-	 u_int8_t Hour;
+	u_int8_t Hour;
 	/* Description -  */
 	/* 0x8.16 - 0x8.23 */
-	 u_int8_t Minutes;
+	u_int8_t Minutes;
 	/* Description -  */
 	/* 0x8.24 - 0x8.31 */
-	 u_int8_t Seconds;
+	u_int8_t Seconds;
 /*---------------- DWORD[3] (Offset 0xc) ----------------*/
 	/* Description -  */
 	/* 0xc.0 - 0xc.7 */
-	 u_int8_t Day;
+	u_int8_t Day;
 	/* Description -  */
 	/* 0xc.8 - 0xc.15 */
-	 u_int8_t Month;
+	u_int8_t Month;
 	/* Description -  */
 	/* 0xc.16 - 0xc.31 */
-	 u_int16_t Year;
+	u_int16_t Year;
 };
 
 /* Description -   */
@@ -127,14 +126,14 @@ struct cibfw_TRIPPLE_VERSION {
 /*---------------- DWORD[0] (Offset 0x0) ----------------*/
 	/* Description -  */
 	/* 0x0.16 - 0x0.31 */
-	 u_int16_t MAJOR;
+	u_int16_t MAJOR;
 /*---------------- DWORD[1] (Offset 0x4) ----------------*/
 	/* Description -  */
 	/* 0x4.0 - 0x4.15 */
-	 u_int16_t SUBMINOR;
+	u_int16_t SUBMINOR;
 	/* Description -  */
 	/* 0x4.16 - 0x4.31 */
-	 u_int16_t MINOR;
+	u_int16_t MINOR;
 };
 
 /* Description -   */
@@ -156,11 +155,11 @@ Then the GUIDs for this port would be:
     0x2c9030001010
     0x2c9030001018 */
 	/* 0x0.0 - 0x1c.31 */
-	 struct cibfw_uid_entry guids[2];
+	struct cibfw_uid_entry guids[2];
 /*---------------- DWORD[8] (Offset 0x20) ----------------*/
 	/* Description -  */
 	/* 0x20.0 - 0x3c.31 */
-	 struct cibfw_uid_entry macs[2];
+	struct cibfw_uid_entry macs[2];
 };
 
 /* Description -   */
@@ -169,43 +168,66 @@ struct cibfw_image_size {
 /*---------------- DWORD[0] (Offset 0x0) ----------------*/
 	/* Description - log of next address in bytes to search for an image. Address in bytes is 2^log_step */
 	/* 0x0.0 - 0x0.7 */
-	 u_int8_t log_step;
+	u_int8_t log_step;
 	/* Description - this image can run from any partition starting at address 0x0000000, 0x4000000, 0x800000. The code supports only two partitions. */
 	/* 0x0.31 - 0x0.31 */
-	 u_int8_t run_from_any;
+	u_int8_t run_from_any;
 /*---------------- DWORD[1] (Offset 0x4) ----------------*/
 	/* Description - Max possible size in bytes of image. Image read / write should not occure beyond this address */
 	/* 0x4.0 - 0x4.31 */
-	 u_int32_t max_size;
+	u_int32_t max_size;
 };
 
 /* Description -   */
 /* Size in bytes - 64 */
-struct cibfw_module_versions {
+struct cibfw_lfwp_version_vector {
 /*---------------- DWORD[0] (Offset 0x0) ----------------*/
 	/* Description -  */
-	/* 0x0.0 - 0x0.31 */
-	 struct cibfw_module_version core;
-/*---------------- DWORD[1] (Offset 0x4) ----------------*/
-	/* Description -  */
-	/* 0x4.0 - 0x4.31 */
-	 struct cibfw_module_version phy;
+	/* 0x0.0 - 0x4.31 */
+	struct cibfw_reset_version scratchpad;
 /*---------------- DWORD[2] (Offset 0x8) ----------------*/
 	/* Description -  */
-	/* 0x8.0 - 0x8.31 */
-	 struct cibfw_module_version kernel;
-/*---------------- DWORD[3] (Offset 0xc) ----------------*/
-	/* Description -  */
-	/* 0xc.0 - 0xc.31 */
-	 struct cibfw_module_version iron_image;
+	/* 0x8.0 - 0xc.31 */
+	struct cibfw_reset_version icm_context;
 /*---------------- DWORD[4] (Offset 0x10) ----------------*/
 	/* Description -  */
-	/* 0x10.0 - 0x10.31 */
-	 struct cibfw_module_version host_management;
-/*---------------- DWORD[5] (Offset 0x14) ----------------*/
+	/* 0x10.0 - 0x14.31 */
+	struct cibfw_reset_version pci_code;
+/*---------------- DWORD[6] (Offset 0x18) ----------------*/
 	/* Description -  */
-	/* 0x14.0 - 0x14.31 */
-	 struct cibfw_module_version mad;
+	/* 0x18.0 - 0x1c.31 */
+	struct cibfw_reset_version phy_code;
+/*---------------- DWORD[8] (Offset 0x20) ----------------*/
+	/* Description -  */
+	/* 0x20.0 - 0x24.31 */
+	struct cibfw_reset_version ini;
+/*---------------- DWORD[10] (Offset 0x28) ----------------*/
+	/* Description -  */
+	/* 0x28.0 - 0x2c.31 */
+	struct cibfw_reset_version reserved1;
+/*---------------- DWORD[12] (Offset 0x30) ----------------*/
+	/* Description -  */
+	/* 0x30.0 - 0x34.31 */
+	struct cibfw_reset_version reserved2;
+/*---------------- DWORD[14] (Offset 0x38) ----------------*/
+	/* Description -  */
+	/* 0x38.0 - 0x3c.31 */
+	struct cibfw_reset_version reserved3;
+};
+
+/* Description -   */
+/* Size in bytes - 4 */
+struct cibfw_module_version {
+/*---------------- DWORD[0] (Offset 0x0) ----------------*/
+	/* Description -  */
+	/* 0x0.0 - 0x0.7 */
+	u_int8_t branch;
+	/* Description -  */
+	/* 0x0.8 - 0x0.19 */
+	u_int16_t minor;
+	/* Description -  */
+	/* 0x0.20 - 0x0.31 */
+	u_int16_t major;
 };
 
 /* Description -   */
@@ -214,11 +236,11 @@ struct cibfw_operation_key {
 /*---------------- DWORD[0] (Offset 0x0) ----------------*/
 	/* Description -  */
 	/* 0x0.0 - 0x0.15 */
-	 u_int16_t key_modifier;
+	u_int16_t key_modifier;
 /*---------------- DWORD[2] (Offset 0x8) ----------------*/
 	/* Description -  */
 	/* 0x8.0 - 0xc.31 */
-	 u_int64_t key;
+	u_int64_t key;
 };
 
 /* Description -   */
@@ -227,115 +249,159 @@ struct cibfw_device_info {
 /*---------------- DWORD[0] (Offset 0x0) ----------------*/
 	/* Description -  */
 	/* 0x0.0 - 0x0.31 */
-	 u_int32_t signature0;
+	u_int32_t signature0;
 /*---------------- DWORD[1] (Offset 0x4) ----------------*/
 	/* Description -  */
 	/* 0x4.0 - 0x4.31 */
-	 u_int32_t signature1;
+	u_int32_t signature1;
 /*---------------- DWORD[2] (Offset 0x8) ----------------*/
 	/* Description -  */
 	/* 0x8.0 - 0x8.31 */
-	 u_int32_t signature2;
+	u_int32_t signature2;
 /*---------------- DWORD[3] (Offset 0xc) ----------------*/
 	/* Description -  */
 	/* 0xc.0 - 0xc.31 */
-	 u_int32_t signature3;
+	u_int32_t signature3;
 /*---------------- DWORD[4] (Offset 0x10) ----------------*/
 	/* Description - Format version for this struct */
 	/* 0x10.0 - 0x10.7 */
-	 u_int8_t minor_version;
+	u_int8_t minor_version;
 	/* Description - Format version for this struct */
 	/* 0x10.8 - 0x10.16 */
-	 u_int16_t major_version;
+	u_int16_t major_version;
 /*---------------- DWORD[8] (Offset 0x20) ----------------*/
 	/* Description -  */
 	/* 0x20.0 - 0x5c.31 */
-	 struct cibfw_guids guids;
+	struct cibfw_guids guids;
 /*---------------- DWORD[27] (Offset 0x6c) ----------------*/
 	/* Description -  */
 	/* 0x6c.0 - 0x6c.15 */
-	 u_int16_t vsd_vendor_id;
+	u_int16_t vsd_vendor_id;
 /*---------------- DWORD[28] (Offset 0x70) ----------------*/
 	/* Description -  */
 	/* 0x70.24 - 0x140.23 */
-	 char vsd[209];
+	char vsd[209];
 /*---------------- DWORD[88] (Offset 0x160) ----------------*/
 	/* Description -  */
 	/* 0x160.0 - 0x19c.31 */
-	 struct cibfw_operation_key keys[4];
+	struct cibfw_operation_key keys[4];
 };
 
 /* Description -   */
 /* Size in bytes - 1024 */
 struct cibfw_image_info {
 /*---------------- DWORD[0] (Offset 0x0) ----------------*/
+	/* Description - Indicate that this binary support long keys (up to 4096bits) */
+	/* 0x0.6 - 0x0.6 */
+	u_int8_t long_keys;
+	/* Description - when set, debug-fw tokens are enabled. */
+	/* 0x0.7 - 0x0.7 */
+	u_int8_t debug_fw_tokens_supported;
 	/* Description - The image can be updated using the MCC/MCDA commands */
 	/* 0x0.8 - 0x0.8 */
-	 u_int8_t mcc_en;
+	u_int8_t mcc_en;
+	/* Description - OEM lifecycle NVCONFIG files are signed */
+	/* 0x0.9 - 0x0.9 */
+	u_int8_t signed_vendor_nvconfig_files;
+	/* Description - Mellanox lifecycle NVCONFIG files are signed */
+	/* 0x0.10 - 0x0.10 */
+	u_int8_t signed_mlnx_nvconfig_files;
+	/* Description - Factory re-customizationflow is supported */
+	/* 0x0.11 - 0x0.11 */
+	u_int8_t frc_supported;
+	/* Description - Customer Support Tokens are supported */
+	/* 0x0.12 - 0x0.12 */
+	u_int8_t cs_tokens_supported;
 	/* Description - This is a debug firmware */
 	/* 0x0.13 - 0x0.13 */
-	 u_int8_t debug_fw;
+	u_int8_t debug_fw;
 	/* Description - [MCC/MCDA flow] if set, the SHA 256 digest is encrypted - enabled by default for secure_fw - cr-space not closed */
 	/* 0x0.14 - 0x0.14 */
-	 u_int8_t signed_fw;
+	u_int8_t signed_fw;
 	/* Description - [MCC/MCDA flow] cr-space closed */
 	/* 0x0.15 - 0x0.15 */
-	 u_int8_t secure_fw;
+	u_int8_t secure_fw;
 	/* Description - IMAGE_INFO section minor version */
 	/* 0x0.16 - 0x0.23 */
-	 u_int8_t minor_version;
+	u_int8_t minor_version;
 	/* Description - IMAGE_INFO section major version */
 	/* 0x0.24 - 0x0.31 */
-	 u_int8_t major_version;
+	u_int8_t major_version;
 /*---------------- DWORD[1] (Offset 0x4) ----------------*/
 	/* Description -  */
 	/* 0x4.0 - 0x10.31 */
-	 struct cibfw_FW_VERSION FW_VERSION;
+	struct cibfw_FW_VERSION FW_VERSION;
 /*---------------- DWORD[5] (Offset 0x14) ----------------*/
 	/* Description -  */
 	/* 0x14.0 - 0x18.31 */
-	 struct cibfw_TRIPPLE_VERSION mic_version;
+	struct cibfw_TRIPPLE_VERSION mic_version;
 /*---------------- DWORD[7] (Offset 0x1c) ----------------*/
+	/* Description -  */
+	/* 0x1c.0 - 0x1c.15 */
+	u_int16_t pci_vendor_id;
 	/* Description -  */
 	/* 0x1c.16 - 0x1c.31 */
-	 u_int16_t pci_device_id;
+	u_int16_t pci_device_id;
+/*---------------- DWORD[8] (Offset 0x20) ----------------*/
+	/* Description -  */
+	/* 0x20.0 - 0x20.15 */
+	u_int16_t pci_sub_vendor_id;
+	/* Description -  */
+	/* 0x20.16 - 0x20.31 */
+	u_int16_t pci_subsystem_id;
 /*---------------- DWORD[9] (Offset 0x24) ----------------*/
 	/* Description -  */
 	/* 0x24.24 - 0x34.23 */
-	 char psid[17];
+	char psid[17];
 /*---------------- DWORD[13] (Offset 0x34) ----------------*/
 	/* Description -  */
 	/* 0x34.0 - 0x34.15 */
-	 u_int16_t vsd_vendor_id;
+	u_int16_t vsd_vendor_id;
 /*---------------- DWORD[14] (Offset 0x38) ----------------*/
 	/* Description -  */
 	/* 0x38.24 - 0x108.23 */
-	 char vsd[209];
+	char vsd[209];
 /*---------------- DWORD[66] (Offset 0x108) ----------------*/
 	/* Description - image size parameters */
 	/* 0x108.0 - 0x10c.31 */
-	 struct cibfw_image_size image_size;
+	struct cibfw_image_size image_size;
 /*---------------- DWORD[70] (Offset 0x118) ----------------*/
 	/* Description - HW device(s) supported by this FW image.
 0 means invalid entry.
 For Golan A0, first entry should be 0x1ff
  */
 	/* 0x118.0 - 0x124.31 */
-	 u_int32_t supported_hw_id[4];
+	u_int32_t supported_hw_id[4];
 /*---------------- DWORD[74] (Offset 0x128) ----------------*/
 	/* Description -  */
 	/* 0x128.0 - 0x128.31 */
-	 u_int32_t ini_file_num;
+	u_int32_t ini_file_num;
+/*---------------- DWORD[76] (Offset 0x130) ----------------*/
+	/* Description -  */
+	/* 0x130.0 - 0x16c.31 */
+	struct cibfw_lfwp_version_vector lfwp_version_vector;
 /*---------------- DWORD[112] (Offset 0x1c0) ----------------*/
 	/* Description - Product Version is the unified version of the FW and expansion ROM.
 Format is defined by the packager.
 When set to a non-empty string the FW update tool burns the image as a monolythic entity and refuses to update rom only or FW only. */
 	/* 0x1c0.24 - 0x1d0.23 */
-	 char prod_ver[17];
-/*---------------- DWORD[192] (Offset 0x300) ----------------*/
+	char prod_ver[17];
+/*---------------- DWORD[116] (Offset 0x1d0) ----------------*/
+	/* Description - Product description */
+	/* 0x1d0.24 - 0x2d0.23 */
+	char description[257];
+/*---------------- DWORD[197] (Offset 0x314) ----------------*/
 	/* Description -  */
-	/* 0x300.0 - 0x33c.31 */
-	 struct cibfw_module_versions module_versions;
+	/* 0x314.0 - 0x314.31 */
+	struct cibfw_module_version isfu;
+/*---------------- DWORD[208] (Offset 0x340) ----------------*/
+	/* Description - Product name */
+	/* 0x340.24 - 0x380.23 */
+	char name[65];
+/*---------------- DWORD[224] (Offset 0x380) ----------------*/
+	/* Description - PRS used to generate the FW binary */
+	/* 0x380.24 - 0x400.23 */
+	char prs_name[129];
 };
 
 /* Description -   */
@@ -344,7 +410,7 @@ struct cibfw_itoc_entry {
 /*---------------- DWORD[0] (Offset 0x0) ----------------*/
 	/* Description -  */
 	/* 0x0.2 - 0x0.23 */
-	 u_int32_t size;
+	u_int32_t size;
 	/* Description - Section ID Section Type DESCRIPTION
 0x1 BOOT_CODE FW loader code
 0x2 PCI_CODE Code that is required to raise PCIe link.
@@ -369,44 +435,44 @@ struct cibfw_itoc_entry {
 All other values Reserved 
  */
 	/* 0x0.24 - 0x0.31 */
-	 u_int8_t type;
+	u_int8_t type;
 /*---------------- DWORD[1] (Offset 0x4) ----------------*/
 	/* Description - if partition type is code or ini then the load address is in here */
 	/* 0x4.0 - 0x4.29 */
-	 u_int32_t param0;
+	u_int32_t param0;
 	/* Description - When this bit is set, Data within the section is protected by per-line crc. See yu.flash.replacement.crc_en */
 	/* 0x4.30 - 0x4.30 */
-	 u_int8_t cache_line_crc;
+	u_int8_t cache_line_crc;
 	/* Description - When this bit is set, image is zipped */
 	/* 0x4.31 - 0x4.31 */
-	 u_int8_t zipped_image;
+	u_int8_t zipped_image;
 /*---------------- DWORD[2] (Offset 0x8) ----------------*/
 	/* Description - if partition type is code then the jump address is in here */
 	/* 0x8.0 - 0x8.31 */
-	 u_int32_t param1;
+	u_int32_t param1;
 /*---------------- DWORD[5] (Offset 0x14) ----------------*/
 	/* Description -  */
 	/* 0x14.2 - 0x14.30 */
-	 u_int32_t flash_addr;
+	u_int32_t flash_addr;
 	/* Description -  */
 	/* 0x14.31 - 0x14.31 */
-	 u_int8_t relative_addr;
+	u_int8_t relative_addr;
 /*---------------- DWORD[6] (Offset 0x18) ----------------*/
 	/* Description -  */
 	/* 0x18.0 - 0x18.15 */
-	 u_int16_t section_crc;
+	u_int16_t section_crc;
 	/* Description -  */
 	/* 0x18.16 - 0x18.16 */
-	 u_int8_t no_crc;
+	u_int8_t no_crc;
 	/* Description - When this bit is set, the section pointed by this entry in belongs to teh device tather than to teh FW image.
 A device_data section should not be updated in a regular FW update.
 Example for device_data section: VPD_R, GUIDs. */
 	/* 0x18.17 - 0x18.17 */
-	 u_int8_t device_data;
+	u_int8_t device_data;
 /*---------------- DWORD[7] (Offset 0x1c) ----------------*/
 	/* Description -  */
 	/* 0x1c.0 - 0x1c.15 */
-	 u_int16_t itoc_entry_crc;
+	u_int16_t itoc_entry_crc;
 };
 
 /* Description -   */
@@ -415,27 +481,27 @@ struct cibfw_itoc_header {
 /*---------------- DWORD[0] (Offset 0x0) ----------------*/
 	/* Description - 49 54 4f 43 */
 	/* 0x0.0 - 0x0.31 */
-	 u_int32_t signature0;
+	u_int32_t signature0;
 /*---------------- DWORD[1] (Offset 0x4) ----------------*/
 	/* Description - 04 08 15 16 */
 	/* 0x4.0 - 0x4.31 */
-	 u_int32_t signature1;
+	u_int32_t signature1;
 /*---------------- DWORD[2] (Offset 0x8) ----------------*/
 	/* Description - 23 42 ca fa */
 	/* 0x8.0 - 0x8.31 */
-	 u_int32_t signature2;
+	u_int32_t signature2;
 /*---------------- DWORD[3] (Offset 0xc) ----------------*/
 	/* Description - ba ca fe 00 */
 	/* 0xc.0 - 0xc.31 */
-	 u_int32_t signature3;
+	u_int32_t signature3;
 /*---------------- DWORD[4] (Offset 0x10) ----------------*/
 	/* Description - Current version: 1 */
 	/* 0x10.0 - 0x10.7 */
-	 u_int8_t version;
+	u_int8_t version;
 /*---------------- DWORD[7] (Offset 0x1c) ----------------*/
 	/* Description -  */
 	/* 0x1c.0 - 0x1c.15 */
-	 u_int16_t itoc_entry_crc;
+	u_int16_t itoc_entry_crc;
 };
 
 /* Description -   */
@@ -444,22 +510,22 @@ struct cibfw_mfg_info {
 /*---------------- DWORD[0] (Offset 0x0) ----------------*/
 	/* Description -  */
 	/* 0x0.24 - 0x10.23 */
-	 char psid[17];
+	char psid[17];
 /*---------------- DWORD[7] (Offset 0x1c) ----------------*/
 	/* Description - When this bit is set, the GUIDs should be taken from the device_info node.
 When this bit is cleared, the GUIDs should be taken from the mfg_info node. */
 	/* 0x1c.0 - 0x1c.0 */
-	 u_int8_t guids_override_en;
+	u_int8_t guids_override_en;
 	/* Description - MFG_INFO section minor version */
 	/* 0x1c.16 - 0x1c.23 */
-	 u_int8_t minor_version;
+	u_int8_t minor_version;
 	/* Description - MFG_INFO section major version */
 	/* 0x1c.24 - 0x1c.31 */
-	 u_int8_t major_version;
+	u_int8_t major_version;
 /*---------------- DWORD[8] (Offset 0x20) ----------------*/
 	/* Description -  */
 	/* 0x20.0 - 0x5c.31 */
-	 struct cibfw_guids guids;
+	struct cibfw_guids guids;
 };
 
 /* Description -   */
@@ -468,20 +534,20 @@ struct cibfw_register_mfai {
 /*---------------- DWORD[0] (Offset 0x0) ----------------*/
 	/* Description -  */
 	/* 0x0.0 - 0x0.23 */
-	 u_int32_t address;
+	u_int32_t address;
 	/* Description - if set make image permamnent */
 	/* 0x0.29 - 0x0.29 */
-	 u_int8_t permanent;
+	u_int8_t permanent;
 	/* Description - if set use address */
 	/* 0x0.30 - 0x0.30 */
-	 u_int8_t use_address;
+	u_int8_t use_address;
 	/* Description - if set use image id and not address */
 	/* 0x0.31 - 0x0.31 */
-	 u_int8_t use_image_id;
+	u_int8_t use_image_id;
 /*---------------- DWORD[1] (Offset 0x4) ----------------*/
 	/* Description -  */
 	/* 0x4.0 - 0x4.7 */
-	 u_int8_t image_id;
+	u_int8_t image_id;
 };
 
 /* Description -   */
@@ -496,7 +562,7 @@ struct cibfw_register_mfrl {
  7-server warm reboot
  8-server cold reboot */
 	/* 0x4.0 - 0x4.7 */
-	 u_int8_t reset_level;
+	u_int8_t reset_level;
 };
 
 /* Description -   */
@@ -505,28 +571,28 @@ union cibfw_cibfw_Nodes {
 /*---------------- DWORD[0] (Offset 0x0) ----------------*/
 	/* Description -  */
 	/* 0x0.0 - 0xc.31 */
-	 struct cibfw_register_mfai register_mfai;
+	struct cibfw_register_mfai register_mfai;
 	/* Description -  */
 	/* 0x0.0 - 0x1c.31 */
-	 struct cibfw_itoc_entry itoc_entry;
+	struct cibfw_itoc_entry itoc_entry;
 	/* Description -  */
 	/* 0x0.0 - 0x3c.31 */
-	 struct cibfw_guids guids;
+	struct cibfw_guids guids;
 	/* Description -  */
 	/* 0x0.0 - 0x1c.31 */
-	 struct cibfw_itoc_header itoc_header;
+	struct cibfw_itoc_header itoc_header;
 	/* Description -  */
 	/* 0x0.0 - 0xc.31 */
-	 struct cibfw_register_mfrl register_mfrl;
+	struct cibfw_register_mfrl register_mfrl;
 	/* Description -  */
 	/* 0x0.0 - 0x1fc.31 */
-	 struct cibfw_device_info device_info;
+	struct cibfw_device_info device_info;
 	/* Description -  */
 	/* 0x0.0 - 0x13c.31 */
-	 struct cibfw_mfg_info mfg_info;
+	struct cibfw_mfg_info mfg_info;
 	/* Description -  */
 	/* 0x0.0 - 0x3fc.31 */
-	 struct cibfw_image_info image_info;
+	struct cibfw_image_info image_info;
 };
 
 
@@ -538,13 +604,13 @@ void cibfw_uint64_print(const u_int64_t *ptr_struct, FILE *fd, int indent_level)
 unsigned int cibfw_uint64_size(void);
 #define CIBFW_UINT64_SIZE    (0x8)
 void cibfw_uint64_dump(const u_int64_t *ptr_struct, FILE *fd);
-/* module_version */
-void cibfw_module_version_pack(const struct cibfw_module_version *ptr_struct, u_int8_t *ptr_buff);
-void cibfw_module_version_unpack(struct cibfw_module_version *ptr_struct, const u_int8_t *ptr_buff);
-void cibfw_module_version_print(const struct cibfw_module_version *ptr_struct, FILE *fd, int indent_level);
-unsigned int cibfw_module_version_size(void);
-#define CIBFW_MODULE_VERSION_SIZE    (0x4)
-void cibfw_module_version_dump(const struct cibfw_module_version *ptr_struct, FILE *fd);
+/* reset_version */
+void cibfw_reset_version_pack(const struct cibfw_reset_version *ptr_struct, u_int8_t *ptr_buff);
+void cibfw_reset_version_unpack(struct cibfw_reset_version *ptr_struct, const u_int8_t *ptr_buff);
+void cibfw_reset_version_print(const struct cibfw_reset_version *ptr_struct, FILE *fd, int indent_level);
+unsigned int cibfw_reset_version_size(void);
+#define CIBFW_RESET_VERSION_SIZE    (0x8)
+void cibfw_reset_version_dump(const struct cibfw_reset_version *ptr_struct, FILE *fd);
 /* uid_entry */
 void cibfw_uid_entry_pack(const struct cibfw_uid_entry *ptr_struct, u_int8_t *ptr_buff);
 void cibfw_uid_entry_unpack(struct cibfw_uid_entry *ptr_struct, const u_int8_t *ptr_buff);
@@ -580,13 +646,20 @@ void cibfw_image_size_print(const struct cibfw_image_size *ptr_struct, FILE *fd,
 unsigned int cibfw_image_size_size(void);
 #define CIBFW_IMAGE_SIZE_SIZE    (0x8)
 void cibfw_image_size_dump(const struct cibfw_image_size *ptr_struct, FILE *fd);
-/* module_versions */
-void cibfw_module_versions_pack(const struct cibfw_module_versions *ptr_struct, u_int8_t *ptr_buff);
-void cibfw_module_versions_unpack(struct cibfw_module_versions *ptr_struct, const u_int8_t *ptr_buff);
-void cibfw_module_versions_print(const struct cibfw_module_versions *ptr_struct, FILE *fd, int indent_level);
-unsigned int cibfw_module_versions_size(void);
-#define CIBFW_MODULE_VERSIONS_SIZE    (0x40)
-void cibfw_module_versions_dump(const struct cibfw_module_versions *ptr_struct, FILE *fd);
+/* lfwp_version_vector */
+void cibfw_lfwp_version_vector_pack(const struct cibfw_lfwp_version_vector *ptr_struct, u_int8_t *ptr_buff);
+void cibfw_lfwp_version_vector_unpack(struct cibfw_lfwp_version_vector *ptr_struct, const u_int8_t *ptr_buff);
+void cibfw_lfwp_version_vector_print(const struct cibfw_lfwp_version_vector *ptr_struct, FILE *fd, int indent_level);
+unsigned int cibfw_lfwp_version_vector_size(void);
+#define CIBFW_LFWP_VERSION_VECTOR_SIZE    (0x40)
+void cibfw_lfwp_version_vector_dump(const struct cibfw_lfwp_version_vector *ptr_struct, FILE *fd);
+/* module_version */
+void cibfw_module_version_pack(const struct cibfw_module_version *ptr_struct, u_int8_t *ptr_buff);
+void cibfw_module_version_unpack(struct cibfw_module_version *ptr_struct, const u_int8_t *ptr_buff);
+void cibfw_module_version_print(const struct cibfw_module_version *ptr_struct, FILE *fd, int indent_level);
+unsigned int cibfw_module_version_size(void);
+#define CIBFW_MODULE_VERSION_SIZE    (0x4)
+void cibfw_module_version_dump(const struct cibfw_module_version *ptr_struct, FILE *fd);
 /* operation_key */
 void cibfw_operation_key_pack(const struct cibfw_operation_key *ptr_struct, u_int8_t *ptr_buff);
 void cibfw_operation_key_unpack(struct cibfw_operation_key *ptr_struct, const u_int8_t *ptr_buff);
