@@ -172,10 +172,35 @@ static struct device_sem_info g_dev_sem_info_db[] = {
         1,                       // vsec_sem_supported
     },
     {
-        DeviceSpectrum3,          // dev_id
+        DeviceConnectX7,         // dev_id
+        {0xe74e0},               // hw_sem_addr
+        1,                       // vsec_sem_supported
+    },
+    {
+        DeviceGearBox,           // dev_id
+        {0xe74e0},               // hw_sem_addr
+        1,                       // vsec_sem_supported
+    },
+    {
+        DeviceGearBoxManager,    // dev_id
+        {0xe74e0},               // hw_sem_addr
+        1,                       // vsec_sem_supported
+    },
+    {
+        DeviceSpectrum3,         // dev_id
         {0xa68f8},               // hw_sem_addr
         0,                       // vsec_sem_supported
-    }
+    },
+    {
+        DeviceQuantum2,          // dev_id
+        {0xe74e0},               // hw_sem_addr
+        1,                       // vsec_sem_supported
+    },
+    {
+        DeviceSpectrum4,         // dev_id
+        {0xe74e0},               // hw_sem_addr
+        1,                       // vsec_sem_supported
+    },
 };
 
 #define MAX_SEMAPHORE_ADDRES 8
